/** 
 *******************************************************************************
 * @file    txz_fuart_mdma.h
 * @brief   This file provides all the functions prototypes for FUART driver.
 * @brief   If MDMA transfer is used, it is necessary.
 * @version V1.0.0.0
 * $Date:: 2017-08-28 10:52:31 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __FUART_MDMA_H
#define __FUART_MDMA_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"
#include "txz_fuart.h"
#include "txz_fuart_include.h"
#include "txz_mdma.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup FUART
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Exported_define
 *  @{
 */
/** 
 *  @defgroup FUART_MDMA_TransferType  Transfer Type
 *  @brief    MDMA transfer type.
 *  @{
 */
#define FUART_MDMA_TYPE_UNUSE        ((uint32_t)0x00)    /*!< MDMA UNUSE    */
#define FUART_MDMA_TYPE_AREA_TO_AREA ((uint32_t)0x01)    /*!< MDMA AREA to AREA */
#define FUART_MDMA_TYPE_FIX_TO_AREA  ((uint32_t)0x02)    /*!< MDMA FIX to AREA */
#define FUART_MDMA_TYPE_AREA_TO_FIX  ((uint32_t)0x03)    /*!< MDMA AREA to FIX */
#define FUART_MDMA_TYPE_FIX_TO_FIX   ((uint32_t)0x04)    /*!< MDMA FIX to AREA */
/**
 *  @}
 */ /* End of group FUART_MDMA_TransferType */
/** 
 *  @defgroup FUART_MDMA_TransferType  Transfer Type
 *  @brief    MDMA transfer type.
 *  @{
 */
#define FUART_MDMA_FUART_CH0          ((uint32_t)0x00)    /*!< MDMA FUART CH0    */
#define FUART_MDMA_FUART_CH1          ((uint32_t)0x01)    /*!< MDMA FUART CH1    */
/**
 *  @}
 */ /* End of group FUART_MDMA_TransferType */

/**
 *  @}
 */ /* End of group FUART_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Exported_define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group FUART_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Exported_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  MDMA setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t type;      /*!< MDMA transfer type. 
                            : Use @ref FUART_MDMA_TransferType          */
    uint32_t ch;        /*!< MDMA fuart channel.  
                            : If MDMA isn't used, it isn't necessary.  */
} fuart_mdma_setting_t;

/*----------------------------------*/
/** 
 * @brief  FUART MDMA initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    mdma_t              *p_obj;  /*!< MDMA object address.  */
    fuart_mdma_setting_t tx;      /*!< Transmit setting.    */
    fuart_mdma_setting_t rx;      /*!< Receive setting.     */
} fuart_mdma_initial_setting_t;




/*----------------------------------*/
/** 
 * @brief  FUART MDMA handle structure definition.
*/
/*----------------------------------*/
typedef struct
{
    fuart_t super;                       /*!< Uart super class. 
                                            : Use @ref fuart_t                       */
    fuart_mdma_initial_setting_t init;    /*!< Initial setting.
                                            : Use @ref fuart_mdma_initial_setting_t   */
} fuart_mdma_t;


/**
 *  @}
 */ /* End of group FUART_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Exported_functions
 *  @{
 */
TXZ_Result fuart_mdma_init(fuart_mdma_t *p_obj);
TXZ_Result fuart_mdma_deinit(fuart_mdma_t *p_obj);
TXZ_Result fuart_mdma_discard_transmit(fuart_mdma_t *p_obj);
TXZ_Result fuart_mdma_discard_receive(fuart_mdma_t *p_obj);
TXZ_Result fuart_mdma_transmitIt(fuart_mdma_t *p_obj, fuart_transmit_t *p_info);
TXZ_Result fuart_mdma_receiveIt(fuart_mdma_t *p_obj, fuart_receive_t *p_info);
TXZ_Result fuart_mdma_get_fuart_ch_to_mdma_ch(uint32_t fuart_ch, uint32_t *tx_ch, uint32_t *rx_ch);
TXZ_Result fuart_mdma_TXE_disable(fuart_mdma_t *p_obj);
TXZ_Result fuart_mdma_TXE_enable(fuart_mdma_t *p_obj);
TXZ_Result fuart_mdma_RXE_disable(fuart_mdma_t *p_obj);
TXZ_Result fuart_mdma_RXE_enable(fuart_mdma_t *p_obj);
/**
 *  @}
 */ /* End of group FUART_Exported_functions */

/**
 *  @}
 */ /* End of group FUART */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __FUART_DMA_H */


