/** 
 *******************************************************************************
 * @file    txz_hdma.h
 * @brief   This file provides all the functions prototypes for HDMA driver.
 * @version V1.0.0.1
 * $Date:: 2018-02-23 14:22:19 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __HDMA_H
#define __HDMA_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @defgroup HDMA HDMA
 *  @brief    HDMA Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Exported_macro HDMA Exported Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group HDMA_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Exported_define HDMA Exported Define
 *  @{
 */

/** 
 *  @defgroup HDMA_ChannelNum  Channel Num
 *  @brief    HDMA Channel.
 *  @{
 */
#define HDMA_CH_NUM                      ((uint32_t)2)            /*!< Num of total channel (0<= value <= 1). */
/**
 *  @}
 */ /* End of group HDMA_ChannelNum */

/** 
 *  @defgroup HDMA_TransferType  Transfer Type Single/Burst
 *  @brief    Transfer Type Single/Burst.
 *  @{
 */
#define HDMA_TRANS_TYPE_SINGLE           ((uint32_t)0x00000000)    /*!< Single Transfer.   */
#define HDMA_TRANS_TYPE_BURST            ((uint32_t)0x00000001)    /*!< Burst Transfer.    */
/**
 *  @}
 */ /* End of group HDMA_TransferCompInt */

/** 
 *  @defgroup HDMA_SrcReqCh  Source DMA Request Channel
 *  @brief    Source DMA Request Channel.
 *  @{
 */
#define HDMA_SRC_REQ_CH_MEM_USE          ((uint32_t)0x00000000)    /*!< Memory Use         */
#define HDMA_SRC_REQ_CH_1                ((uint32_t)0x00000001)    /*!< channel 1          */
#define HDMA_SRC_REQ_CH_2                ((uint32_t)0x00000002)    /*!< channel 2          */
#define HDMA_SRC_REQ_CH_3                ((uint32_t)0x00000003)    /*!< channel 3          */
#define HDMA_SRC_REQ_CH_4                ((uint32_t)0x00000004)    /*!< channel 4          */
#define HDMA_SRC_REQ_CH_5                ((uint32_t)0x00000005)    /*!< channel 5          */
#define HDMA_SRC_REQ_CH_6                ((uint32_t)0x00000006)    /*!< channel 6          */
#define HDMA_SRC_REQ_CH_7                ((uint32_t)0x00000007)    /*!< channel 7          */
#define HDMA_SRC_REQ_CH_8                ((uint32_t)0x00000008)    /*!< channel 8          */
#define HDMA_SRC_REQ_CH_9                ((uint32_t)0x00000009)    /*!< channel 9          */
#define HDMA_SRC_REQ_CH_10               ((uint32_t)0x0000000A)    /*!< channel 10         */
#define HDMA_SRC_REQ_CH_11               ((uint32_t)0x0000000B)    /*!< channel 11         */
#define HDMA_SRC_REQ_CH_12               ((uint32_t)0x0000000C)    /*!< channel 12         */
#define HDMA_SRC_REQ_CH_13               ((uint32_t)0x0000000D)    /*!< channel 13         */
#define HDMA_SRC_REQ_CH_14               ((uint32_t)0x0000000E)    /*!< channel 14         */
#define HDMA_SRC_REQ_CH_15               ((uint32_t)0x0000000F)    /*!< channel 15         */
/**
 *  @}
 */ /* End of group HDMA_SrcReqCh */

/** 
 *  @defgroup HDMA_DstReqCh  Destination DMA Request Channel
 *  @brief    Destination DMA Request Channel.
 *  @{
 */
#define HDMA_DST_REQ_CH_MEM_USE          ((uint32_t)0x00000000)    /*!< Memory Use         */
#define HDMA_DST_REQ_CH_1                ((uint32_t)0x00000001)    /*!< channel 1          */
#define HDMA_DST_REQ_CH_2                ((uint32_t)0x00000002)    /*!< channel 2          */
#define HDMA_DST_REQ_CH_3                ((uint32_t)0x00000003)    /*!< channel 3          */
#define HDMA_DST_REQ_CH_4                ((uint32_t)0x00000004)    /*!< channel 4          */
#define HDMA_DST_REQ_CH_5                ((uint32_t)0x00000005)    /*!< channel 5          */
#define HDMA_DST_REQ_CH_6                ((uint32_t)0x00000006)    /*!< channel 6          */
#define HDMA_DST_REQ_CH_7                ((uint32_t)0x00000007)    /*!< channel 7          */
#define HDMA_DST_REQ_CH_8                ((uint32_t)0x00000008)    /*!< channel 8          */
#define HDMA_DST_REQ_CH_9                ((uint32_t)0x00000009)    /*!< channel 9          */
#define HDMA_DST_REQ_CH_10               ((uint32_t)0x0000000A)    /*!< channel 10         */
#define HDMA_DST_REQ_CH_11               ((uint32_t)0x0000000B)    /*!< channel 11         */
#define HDMA_DST_REQ_CH_12               ((uint32_t)0x0000000C)    /*!< channel 12         */
#define HDMA_DST_REQ_CH_13               ((uint32_t)0x0000000D)    /*!< channel 13         */
#define HDMA_DST_REQ_CH_14               ((uint32_t)0x0000000E)    /*!< channel 14         */
#define HDMA_DST_REQ_CH_15               ((uint32_t)0x0000000F)    /*!< channel 15         */
/**
 *  @}
 */ /* End of group HDMA_dstReqCh */
 
/** 
 *  @defgroup HDMA_TransferCompInt  Transfer Complete Interrupt Enable/Disable
 *  @brief    Transfer Complete Interrupt Enable/Disable.
 *  @{
 */
#define HDMA_TRANS_INT_COMP_DISABLE      ((uint32_t)0x00000000)    /*!< INTHDMACxTC Enable.   */
#define HDMA_TRANS_INT_COMP_ENABLE       ((uint32_t)0x80000000)    /*!< INTHDMACxTC Enable.   */
/**
 *  @}
 */ /* End of group HDMA_TransferCompInt */

/** 
 *  @defgroup HDMA_LliAddress  Lli Address
 *  @brief    Address of Lli Macro Definition.
 *  @brief    Address of Lli be set "(HDMA_LLI_ADDRESS_MIN <= Address <= HDMA_LLI_ADDRESS_MAX)".
 *  @{
 */
#define HDMA_LLI_ADDRESS_MIN         ((uint32_t)0x00000000)    /*!< Minimum  Address :1    */
#define HDMA_LLI_ADDRESS_MAX         ((uint32_t)0x3FFFFFFF)    /*!< Maximum  Address :1073741823 */
/**
 *  @}
 */ /* End of group HDMA_LliAddress */

/** 
 *  @defgroup HDMA_TransferNumRange  Transfer Num Range
 *  @brief    Range of Transfer Num Macro Definition.
 *  @brief    Range of Transfer Num be set "(HDMA_TRANS_NUM_RANGE_MIN <= Value <= HDMA_TRANS_NUM_RANGE_MAX)".
 *  @{
 */
#define HDMA_TRANS_NUM_RANGE_MIN         ((uint32_t)0x00000001)    /*!< Minimum  Value :1    */
#define HDMA_TRANS_NUM_RANGE_MAX         ((uint32_t)0x00000400)    /*!< Maximum  Value :1024 */
/**
 *  @}
 */ /* End of group HDMA_TransferNumRange */
 
/** 
 *  @defgroup HDMA_DestinationAddressInc  Destination Address Increment
 *  @brief    Destination Address Increment.
 *  @{
 */
#define HDMA_DST_ADDR_FIX                ((uint32_t)0x00000000)    /*!< Fix Address.          */
#define HDMA_DST_ADDR_INC                ((uint32_t)0x08000000)    /*!< Address increment.    */
/**
 *  @}
 */ /* End of group HDMA_DestinationAddressInc */

/** 
 *  @defgroup HDMA_SourceAddressInc  Source Address Increment
 *  @brief    Source Address Increment.
 *  @{
 */
#define HDMA_SRC_ADDR_FIX                ((uint32_t)0x00000000)    /*!< Fix Address.          */
#define HDMA_SRC_ADDR_INC                ((uint32_t)0x04000000)    /*!< Address increment.    */
/**
 *  @}
 */ /* End of group HDMA_SourcenAddressInc */

/** 
 *  @defgroup HDMA_DestinationBitWidth  Destination Bit Width
 *  @brief    Destination address bit width.
 *  @{
 */
#define HDMA_DST_BITWIDTH_1BYTE          ((uint32_t)0x00000000)    /*!< 1 byte.               */
#define HDMA_DST_BITWIDTH_2BYTE          ((uint32_t)0x00200000)    /*!< 2 byte.               */
#define HDMA_DST_BITWIDTH_4BYTE          ((uint32_t)0x00400000)    /*!< 4 byte.               */
/**
 *  @}
 */ /* End of group HDMA_DestinationBitWidth */

/** 
 *  @defgroup HDMA_DestinationBurstSize  Destination Burst Size
 *  @brief    Destination Burst Size.
 *  @{
 */
#define HDMA_DST_BURSTSIZE_1BEAT         ((uint32_t)0x00000000)    /*!<   1 beat.             */
#define HDMA_DST_BURSTSIZE_4BEAT         ((uint32_t)0x00008000)    /*!<   4 beat.             */
#define HDMA_DST_BURSTSIZE_8BEAT         ((uint32_t)0x00010000)    /*!<   8 beat.             */
#define HDMA_DST_BURSTSIZE_16BEAT        ((uint32_t)0x00018000)    /*!<  16 beat.             */
#define HDMA_DST_BURSTSIZE_32BEAT        ((uint32_t)0x00020000)    /*!<  32 beat.             */
#define HDMA_DST_BURSTSIZE_64BEAT        ((uint32_t)0x00028000)    /*!<  64 beat.             */
#define HDMA_DST_BURSTSIZE_128BEAT       ((uint32_t)0x00030000)    /*!< 128 beat.             */
#define HDMA_DST_BURSTSIZE_256BEAT       ((uint32_t)0x00038000)    /*!< 256 beat.             */
/**
 *  @}
 */ /* End of group HDMA_DestinationBurstSize */
 
 /** 
 *  @defgroup HDMA_SourceBitWidth  Source Bit Width
 *  @brief    Source bit width.
 *  @{
 */
#define HDMA_SRC_BITWIDTH_1BYTE          ((uint32_t)0x00000000)    /*!< 1 byte.               */
#define HDMA_SRC_BITWIDTH_2BYTE          ((uint32_t)0x00040000)    /*!< 2 byte.               */
#define HDMA_SRC_BITWIDTH_4BYTE          ((uint32_t)0x00080000)    /*!< 4 byte.               */
/**
 *  @}
 */ /* End of group HDMA_SourceBitWidth */

/** 
 *  @defgroup HDMA_SourceBurstSize  Source Burst Size
 *  @brief    Source data size.
 *  @{
 */
#define HDMA_SRC_BURSTSIZE_1BEAT         ((uint32_t)0x00000000)    /*!<   1 beat.             */
#define HDMA_SRC_BURSTSIZE_4BEAT         ((uint32_t)0x00001000)    /*!<   4 beat.             */
#define HDMA_SRC_BURSTSIZE_8BEAT         ((uint32_t)0x00002000)    /*!<   8 beat.             */
#define HDMA_SRC_BURSTSIZE_16BEAT        ((uint32_t)0x00003000)    /*!<  16 beat.             */
#define HDMA_SRC_BURSTSIZE_32BEAT        ((uint32_t)0x00004000)    /*!<  32 beat.             */
#define HDMA_SRC_BURSTSIZE_64BEAT        ((uint32_t)0x00005000)    /*!<  64 beat.             */
#define HDMA_SRC_BURSTSIZE_128BEAT       ((uint32_t)0x00006000)    /*!< 128 beat.             */
#define HDMA_SRC_BURSTSIZE_256BEAT       ((uint32_t)0x00007000)    /*!< 256 beat.             */
/**
 *  @}
 */ /* End of group HDMA_SourceBurstSize */

/** 
 *  @defgroup HDMAxCnCfg  HDMAxCnCfg Register
 *  @brief    HDMAxCnCfg Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol          |
 *  | :---  | :---                |
 *  | 31-19 | -                   |
 *  | 18    | Halt                |
 *  | 17    | Active              |
 *  | 16    | Lock                |
 *  | 15    | ITC                 |
 *  | 14    | IE                  |
 *  | 13-11 | FlowCntrl[2:0]      |
 *  | 10    | -                   |
 *  | 9-6   | DestPeripheral[3:0] |
 *  | 5     | -                   |
 *  | 4-1   | SrcPeripheral[3:0]  |
 *  | 0     | E                   |
 *  @{
 */
/* E */
/** 
 *  @defgroup HDMAxCnCfg_E  HDMA channel Enable/Disable
 *  @brief    HDMA channel Enable/Disable.
 *  @{
 */
#define HDMA_CH_E_DISABLE             ((uint32_t)0x00000000)    /*!< channelx Disable.   */
#define HDMA_CH_E_ENABLE              ((uint32_t)0x00000001)    /*!< channelx Enable.   */
/**
 *  @}
 */ /* End of group HDMAxCnCfg_E */
/* SrcPeripheral */
/** 
 *  @defgroup HDMAxCnCfg_SrcPeri  HDMA SrcPeripheral MASK
 *  @brief    HDMA SrcPeripheral MASK.
 *  @{
 */
#define HDMA_CH_SRCPERI_MASK          ((uint32_t)0x0000001E)    /*!< SrcPeripheral MASK.   */
/**
 *  @}
 */ /* End of group HDMAxCnCfg_SrcPeri */
/* FlowCntrl */
/** 
 *  @defgroup HDMAxCnCfg_FlowCnt  HDMA FlowControl
 *  @brief    HDMA FlowControl.
 *  @{
 */
#define HDMA_CH_FLOWCNT_MEM_MEM       ((uint32_t)0x00000000)    /*!< Memory -> Memory.         */
#define HDMA_CH_FLOWCNT_MEM_PERI      ((uint32_t)0x00000800)    /*!< Memory -> Peripheral.     */
#define HDMA_CH_FLOWCNT_PERI_MEM      ((uint32_t)0x00001000)    /*!< Peripheral -> Memory.     */
#define HDMA_CH_FLOWCNT_PERI_PERI     ((uint32_t)0x00001800)    /*!< Peripheral ->Peripheral.  */
#define HDMA_CH_FLOWCNT_MASK          ((uint32_t)0x00001800)    /*!< FlowCnt MASK              */
/**
 *  @}
 */ /* End of group HDMAxCnCfg_DstPeri */
/* DestPeripheral */
/** 
 *  @defgroup HDMAxCnCfg_DstPeri  HDMA DstPeripheral MASK
 *  @brief    HDMA DstPeripheral MASK.
 *  @{
 */
#define HDMA_CH_DSTPERI_MASK          ((uint32_t)0x000003C0)    /*!< DstPeripheral MASK.   */
/**
 *  @}
 */ /* End of group HDMAxCnCfg_DstPeri */
/* IE */
/** 
 *  @defgroup HDMAxCnCfg_IE  HDMA channel ErrInterrupt Enable/Disable
 *  @brief    HDMA channel ErrInterrupt Enable/Disable.
 *  @{
 */
#define HDMA_CH_IE_DISABLE             ((uint32_t)0x00000000)    /*!< HDMAxCnCfg_IE Disable.   */
#define HDMA_CH_IE_ENABLE              ((uint32_t)0x00004000)    /*!< HDMAxCnCfg_IE Enable.    */
/**
 *  @}
 */ /* End of group HDMAxCnCfg_IE */
/* ITC */
/** 
 *  @defgroup HDMAxCnCfg_ITC  HDMA channel TransferCompleteInterrupt Enable/Disable
 *  @brief    HDMA channel TransferCompleteInterrupt Enable/Disable.
 *  @{
 */
#define HDMA_CH_ITC_DISABLE             ((uint32_t)0x00000000)    /*!< HDMAxCnCfg_ITC Disable.   */
#define HDMA_CH_ITC_ENABLE              ((uint32_t)0x00008000)    /*!< HDMAxCnCfg_ITC Enable.    */
/**
 *  @}
 */ /* End of group HDMAxCnCfg_ITC */
/* Lock */
/** 
 *  @defgroup HDMAxCnCfg_LOCK  HDMA channel Lock Transfer Enable/Disable
 *  @brief    HDMA channel Lock Transfer Enable/Disable.
 *  @{
 */
#define HDMA_CH_LOCK_DISABLE             ((uint32_t)0x00000000)    /*!< HDMAxCnCfg_LOCK Disable.   */
#define HDMA_CH_LOCK_ENABLE              ((uint32_t)0x00010000)    /*!< HDMAxCnCfg_LOCK Enable.    */
/**
 *  @}
 */ /* End of group HDMAxCnCfg_LOCK */
/* Active */
/** 
 *  @defgroup HDMAxCnCfg_ACT  HDMA channel Active Mask
 *  @brief    HDMA channel Active Mask.
 *  @{
 */
#define HDMA_CH_ACT_MASK                 ((uint32_t)0x00020000)    /*!< Active MASK.      */

/**
 *  @}
 */ /* End of group HDMAxCnCfg_ACT */
/* Halt */
/** 
 *  @defgroup HDMAxCnCfg_HALT  HDMA channel Transfer Request Enable/Disable
 *  @brief    HDMA channel Transfer Request Enable/Disable.
 *  @{
 */
#define HDMA_CH_HALT_ENABLE               ((uint32_t)0x00000000)    /*!< HDMAxCnCfg_HALT Enable.   */
#define HDMA_CH_HALT_DISABLE              ((uint32_t)0x00040000)    /*!< HDMAxCnCfg_HALT Disable.  */
/**
 *  @}
 */ /* End of group HDMAxCnCfg_HALT */
/**
 *  @}
 */ /* End of group HDMAxCnCfg*/
/* ITC */
/** 
 *  @defgroup HDMAxCnCnt_ITC  HDMA channel TransferCompleteInterrupt Enable/Disable
 *  @brief    HDMA TransferCompleteInterrupt Enable/Disable.
 *  @{
 */
#define HDMA_ITC_DISABLE               ((uint32_t)0x00000000)    /*!< HDMAxCnCnt_ITC Disable.   */
#define HDMA_ITC_ENABLE                ((uint32_t)0x80000000)    /*!< HDMAxCnCnt_ITC Enable.    */
/**
 *  @}
 */ /* End of group HDMAxCnCnt_ITC */
/**
 *  @}
 */ /* End of group HDMA_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Exported_define HDMA Exported Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group HDMA_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Exported_typedef HDMA Exported Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t srcAdd;           /*!< Source address.              */
    uint32_t dstAdd;           /*!< Destination address.          */
} hdma_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  Channel control setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t dstInc;            /*!< Destination address increment type. 
                                    : Use @ref HDMA_DestinationAddressInc      */
    uint32_t dstSize;          /*!< Destination data size(Set it as the same size as "srcSize"). 
                                    : Use @ref HDMA_DestinationBitWidth        */
    uint32_t dstBurstSize;  /*!< Destination Burst size(Set it as the same size as srctBurstSiz"). 
                                    : Use @ref HDMA_DestinationBurstSize       */
    uint32_t srcInc;           /*!< Source address increment type. 
                                    : Use @ref HDMA_SourceAddressInc           */
    uint32_t srcSize;         /*!< Source data size(Set it as the same size as "dstSize").
                                    : Use @ref HDMA_SourceBitWidth             */
    uint32_t srcBurstSize;  /*!< Source Burst size(Set it as the same size as dstBurstSiz"). 
                                    : Use @ref HDMA_SourceBurstSize            */
    uint32_t transsize;       /*!< Num of transfer.
                                    : Range (HDMA_TRANS_NUM_RANGE_MIN <= Value <= HDMA_TRANS_NUM_RANGE_MAX)  @ref HDMA_TransferNumRange */
    uint32_t lli;       /*!< Start Address of Link List.
                                    : Address (HDMA_LLI_ADDRESS_MIN <= Address <= HDMA_LLI_ADDRESS_MAX)  @ref HDMA_LliAddress */
} hdma_ch_cnt_t;


/*----------------------------------*/
/** 
 * @brief  Channel config setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t halt;              /*!< Channel Transfer Request Enable/Disable. 
                                    : Use @ref HDMAxCnCfg_HALT                         */
    uint32_t lock;              /*!< Channel Lock Transfer Enable/Disable. 
                                    : Use @ref HDMAxCnCfg_LOCK                         */
    uint32_t itc;               /*!< Channel TransferCompleteInterrupt Enable/Disable. 
                                    : Use @ref HDMAxCnCfg_ITC                          */
    uint32_t ie;                /*!< Channel ErrInterrupt Enable/Disable. 
                                    : Use @ref HDMAxCnCfg_IE                           */
    uint32_t flowCnt;           /*!< FlowControl. 
                                    : Use @ref HDMAxCnCfg_DstPeri                      */
    uint32_t e;                 /*!< Channel Enable/Disable. 
                                    : Use @ref HDMAxCnCfg_E                            */
} hdma_ch_cfg_t;

/*----------------------------------*/
/** 
 * @brief  Channel setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    void (*handler)(uint32_t id, TXZ_Result result);        /*!< Notify converted value.  */
    uint32_t         id;         /*!< ID: User Value.                                     */
    uint32_t         type;       /*!< Single Transfer or Burst Transfer                   */
    uint32_t srcReqCh;           /*!< source Request Channel.                             */
    uint32_t dstReqCh;           /*!< Destination Request Channel.                        */
    hdma_ch_cnt_t    cnt;        /*!< Channel control setting. 
                                    : Use @ref hdma_ch_cnt_t                              */
    hdma_ch_cfg_t    cfg;        /*!< Channel config setting. 
                                    : Use @ref hdma_ch_cfg_t                              */
} hdma_ch_setting_t;

/*----------------------------------*/
/** 
 * @brief  HDMA handle structure definition.
*/
/*----------------------------------*/
typedef struct hdma_handle
{
    TSB_DMAC_TypeDef        *p_instance;      /*!< Registers base address.           */
    hdma_initial_setting_t   init;            /*!< Initial setting.                  */
    hdma_ch_setting_t        ch[HDMA_CH_NUM]; /*!< Channel setting.                  */
} hdma_t;

/**
 *  @}
 */ /* End of group HDMA_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Exported_functions HDMA Exported Functions
 *  @{
 */
TXZ_Result hdma_init(hdma_t *p_obj);
TXZ_Result hdma_deinit(hdma_t *p_obj);
TXZ_Result hdma_get_error(hdma_t *p_obj, uint32_t *p_err);
TXZ_Result hdma_clear_error(hdma_t *p_obj);
TXZ_Result hdma_error_irq_handler(hdma_t *p_obj);
TXZ_Result hdma_channel_init(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_irq_handler(hdma_t *p_obj);
TXZ_Result hdma_HALT_enable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_ITC_enable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_ITC_disable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_LOCK_enable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_LOCK_disable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_IE_enable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_IE_disable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_E_enable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_E_disable(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_FlowType_Set(hdma_t *p_obj, uint32_t dma_ch, uint32_t type);
TXZ_Result hdma_srcPeri_Set(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_dstPeri_Set(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_lli_set(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_startIt(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_stopIt(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_startPeriIt(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result hdma_stopPeriIt(hdma_t *p_obj, uint32_t dma_ch);
TXZ_Result get_src_address(hdma_t *p_obj, uint32_t dma_ch, uint32_t *addr);
TXZ_Result get_dst_address(hdma_t *p_obj, uint32_t dma_ch, uint32_t *addr);

/**
 *  @}
 */ /* End of group HDMA_Exported_functions */

/**
 *  @}
 */ /* End of group HDMA */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __HDMA_H */


