/** 
 *******************************************************************************
 * @file    txz_i2c_api_dma.h
 * @brief   This file provides all the functions prototypes for I2C Driver.
 * @version V0.0.0.0
 * $Date:: 2016-11-24 00:00:00 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __I2C_API_DMA_H
#define __I2C_API_DMA_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_i2c_api.h"
#include "txz_dma_ex.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_define
 *  @{
 */

/* no define*/

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  I2C DMA internal information structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t event;                     /*!< driver event.       */
    uint32_t rxch;                      /*!< DMA receive channel.  */
    uint32_t txch;                      /*!< DMA transmit channel. */
} i2c_dma_internal_info_t;

/*----------------------------------*/
/** 
 * @brief  I2C DMA initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    struct
    {
        dma_t    *p_obj;                /*!< DMA object address.                  */
        TXZ_FunctionalState rx;         /*!< DMA receive channel enable/disable. 
                                                :TXZ_ENABLE/TXZ_DISABLE :Enable/Disable. */
        TXZ_FunctionalState tx;         /*!< DMA transmit channel enable/disable.
                                                :TXZ_ENABLE/TXZ_DISABLE :Enable/Disable. */
    } dma;
    struct
    {
        i2c_port_t sda;                 /*!< SDA port.                            
                                            : Use @ref i2c_port_t                   */
        i2c_port_t scl;                 /*!< SCL port.
                                            : Use @ref i2c_port_t                   */
    } port;
    uint32_t frequency;                 /*!< SCL clock frequency(unit Hz)
                                            : but if zero, default setting is used. */
} i2c_dma_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  I2C DMA handle structure definition.
*/
/*----------------------------------*/
typedef struct
{
    i2c_t   super;                       /*!< i2c super class.                     */
    i2c_dma_initial_setting_t init;      /*!< Initial setting.
                                            : Use @ref i2c_dma_initial_setting_t   */
    i2c_dma_internal_info_t   info;      /*!< Internal information.                */
} i2c_dma_t;

/**
 *  @}
 */ /* End of group UTILITIES_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_functions
 *  @{
 */

/* For DMA Master */
TXZ_Result i2c_init_dma(i2c_dma_t *p_obj);
int32_t i2c_read_dma(i2c_dma_t *p_obj, uint8_t *p_rx, int32_t rx_length, int32_t address, int32_t stop);
int32_t i2c_write_dma(i2c_dma_t *p_obj, uint8_t *p_tx, int32_t tx_length, int32_t address, int32_t stop);
uint32_t i2c_irq_handler_dma(i2c_dma_t *p_obj);
void i2c_abort_dma(i2c_dma_t *p_obj);

/* For DMA Slave */
TXZ_Result i2c_slave_init_dma(i2c_dma_t *p_obj, int32_t address);
int32_t i2c_slave_read_dma(i2c_dma_t *p_obj, uint8_t *p_rx, int32_t rx_length);
int32_t i2c_slave_write_dma(i2c_dma_t *p_obj, uint8_t *p_tx, int32_t tx_length);
uint32_t i2c_slave_irq_handler_dma(i2c_dma_t *p_obj);
void i2c_slave_abort_dma(i2c_dma_t *p_obj);

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_functions */

/**
 *  @}
 */ /* End of group UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __I2C_API_DMA_H */

