/** 
 *******************************************************************************
 * @file    txz_mdma.h
 * @brief   This file provides all the functions prototypes for MDMA driver.
 * @version V1.0.1.0
 * $Date:: 2018-04-02 19:31:41 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __MDMA_H
#define __MDMA_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @defgroup MDMA MDMA
 *  @brief    MDMA Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Exported_macro MDMA Exported Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group MDMA_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Exported_define MDMA Exported Define
 *  @{
 */

/** 
 *  @defgroup MDMA_ChannelNum  Channel Num
 *  @brief    MDMA Channel.
 *  @{
 */
#define MDMA_CH_NUM                     ((uint32_t)32)            /*!< Num of total channel (1<= value <= 32). */
/**
 *  @}
 */ /* End of group DMA_ChannelNum */

/** 
 *  @defgroup MDMA_DMODE  Address Increment Mode
 *  @brief    Address increment mode.
 *  @{
 */
#define MDMA_DMODE_INC                  ((uint32_t)0x00000000)    /*!< increment.               */
#define MDMA_DMODE_DEC                  ((uint32_t)0x00100000)    /*!< decrement.               */
/**
 *  @}
 */ /* End of group MDMA_DMODE */


/** 
 *  @defgroup MDMA_UMODE  Transfer Mode
 *  @brief    Transfer mode.
 *  @{
 */
#define MDMA_UMODE_TRANS_ALL            ((uint32_t)0x00000000)    /*!< trans all.               */
#define MDMA_UMODE_TRANS_UNI            ((uint32_t)0x00010000)    /*!< trans uni.              */
/**
 *  @}
 */ /* End of group MDMA_UMODE */


/** 
 *  @defgroup MDMA_USIZE  Unit Size
 *  @brief    Unit size.
 *  @{
 */
#define MDMA_USIZE_1BYTE                ((uint32_t)0x00000000)    /*!< 1byte.                   */
#define MDMA_USIZE_2BYTE                ((uint32_t)0x00000100)    /*!< 2byte.                   */
#define MDMA_USIZE_4BYTE                ((uint32_t)0x00000200)    /*!< 4byte.                   */
#define MDMA_USIZE_8BYTE                ((uint32_t)0x00000300)    /*!< 8byte.                   */
#define MDMA_USIZE_16BYTE               ((uint32_t)0x00000400)    /*!< 16byte.                  */
#define MDMA_USIZE_32BYTE               ((uint32_t)0x00000500)    /*!< 32byte.                  */
/**
 *  @}
 */ /* End of group MDMA_USIZE */


/** 
 *  @defgroup MDMA_TTYPE  Transfer Type
 *  @brief    Transfer type.
 *  @{
 */
#define MDMA_TTYPE_AREA_AREA            ((uint32_t)0x00000000)    /*!< Area to Area.            */
#define MDMA_TTYPE_FIX_AREA             ((uint32_t)0x00000001)    /*!< Fix to Area.             */
#define MDMA_TTYPE_AREA_FIX             ((uint32_t)0x00000002)    /*!< Area to Fix.             */
#define MDMA_TTYPE_FIX_FIX              ((uint32_t)0x00000003)    /*!< Fix to Fix.              */
/**
 *  @}
 */ /* End of group MDMA_UMODE */

/** 
 *  @defgroup MDMA_INFINITE  INFINITE FLAG
 *  @brief    INFINITE FLAG.
 *  @{
 */
#define MDMA_INFINITE_DISABLE        ((uint32_t)0x00000000)    /*!< INFINITE DISABLE */
#define MDMA_INFINITE_ENABLE         ((uint32_t)0x00000001)    /*!< INFINITE ENABLE */
/**
 *  @}
 */ /* End of group MDMA_UMODE */

/** 
 *  @defgroup MDMA_TransferSizeRange  Transfer Size Range
 *  @brief    Range of Transfer Size Macro Definition.
 *  @brief    Range of Transfer Size be set "(MDMA_TRANS_SIZE_RANGE_MIN <= Value <= MDMA_TRANS_SIZE_RANGE_MAX)".
 *  @{
 */
#define MDMA_TRANS_SIZE_RANGE_MIN         ((uint32_t)0x00000001)    /*!< Minimum  Value :1       */
#define MDMA_TRANS_SIZE_RANGE_MAX         ((uint32_t)0x000FFFFF)    /*!< Maximum  Value :1048575 */
/**
 *  @}
 */ /* End of group MDMA_TransferNumRange */

/** 
 *  @defgroup MDMA_NumberOfDedcriptorRange  Number of Descriptor Range
 *  @brief    Range of Number of Descriptor Macro Definition.
 *  @brief    Range of Number of Descriptor be set "(MDMA_NUM_OF_DESCRIPTOR_RANGE_MIN <= Value <= MDMA_NUM_OF_DESCRIPTOR_RANGE_MAX)".
 *  @{
 */
#define MDMA_NUM_OF_DESCRIPTOR_RANGE_MIN         ((uint32_t)0x00000000)    /*!< Minimum  Value :0       */
#define MDMA_NUM_OF_DESCRIPTOR_RANGE_MAX         ((uint32_t)0x000000FF)    /*!< Maximum  Value :255     */
/**
 *  @}
 */ /* End of group MDMA_TransferNumRange */
/** 
 *  @defgroup MDMA_Priority  Priority
 *  @brief    Priority.
 *  @{
 */
#define MDMA_PRIORITY_NORMAL             ((uint32_t)0x00000000)    /*!< Normal.    */
#define MDMA_PRIORITY_HIGH               ((uint32_t)0x00000001)    /*!< High.      */
/**
 *  @}
 */ /* End of group MDMA_Priority */

/** 
 *  @defgroup MDMA_Error_Reason  error reason
 *  @brief    Error Reason.
 *  @{
 */
#define MDMA_ERROR_BUS_ERROR             ((uint32_t)TXZ_ERROR+0x00000000)    /*!< Bus Error.        */
#define MDMA_ERROR_DESCRIPTOR_ERROR      ((uint32_t)TXZ_ERROR+0x00000001)    /*!< Descriptor Error. */
/**
 *  @}
 */ /* End of group MDMA_Error_Reason */

/** 
 *  @defgroup MDMA_Interrupt_Enable  transfer end interrupt enable
 *  @brief    Interrupt Enable.
 *  @{
 */
#define MDMA_INTERRUPT_ENABLE            ((uint32_t)0x00000001)    /*!< interrupt enable.  */
#define MDMA_INTERRUPT_DISABLE           ((uint32_t)0x00000000)    /*!< interrupt disable. */
/**
 *  @}
 */ /* End of group MDMA_Error_Reason */

 /**
 *  @}
 */ /* End of group MDMA_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Exported_define MDMA Exported Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group MDMA_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Exported_typedef MDMA Exported Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Descriptor  base address definition.
 * @details  Descriptor address. \n
 * Need Size(byte) = ((The total number of the channel) * (8 * 4byte) = 32 * (32) = 1024 = 0x400;
 * | Use channel | 
 * | 0           | MDMAxC0XFTYP,  MDMAxC0XFSAD,  MDMDxC0XFDAD,  MDMAxC0XFSIZ,  MDMAxC0DSADS,  MDMAxC0DSNUM,  Reserved, Reserved
 * | 1           | MDMAxC1XFTYP,  MDMAxC1XFSAD,  MDMDxC1XFDAD,  MDMAxC1XFSIZ,  MDMAxC1DSADS,  MDMAxC1DSNUM,  Reserved, Reserved
 * | 2           | MDMAxC2XFTYP,  MDMAxC2XFSAD,  MDMDxC2XFDAD,  MDMAxC2XFSIZ,  MDMAxC2DSADS,  MDMAxC2DSNUM,  Reserved, Reserved
 * | ~           | ~
 * | 31          | MDMAxC31XFTYP, MDMAxC31XFSAD, MDMDxC31XFDAD, MDMAxC31XFSIZ, MDMAxC31DSADS, MDMAxC31DSNUM, Reserved, Reserved
*/

/*----------------------------------*/
typedef struct
{
    uint32_t base;              /*!< descriptor base address.  */
} mdma_descriptor_address_t;


/*----------------------------------*/
/** 
 * @brief  Descriptor setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t     dmode;        /*!< Address increment mode.  : Use @ref MDMA_DMODE             */
    uint32_t     umode;         /*!< Transfer mode..         : Use @ref MDMA_UMODE             */
    uint32_t     usize;         /*!< Unit size.              : Use @ref MDMA_USIZE             */
    uint32_t     ttype;         /*!< Transfer type.          : Use @ref MDMA_TTYPE             */
    uint32_t     xfsad;         /*!< source address. */
    uint32_t     xfdad;         /*!< destination address. */
    uint32_t     xfsiz;         /*!< trasnfer size. */
    uint32_t     dsads;         /*!< next descriptor address. */
} mdma_descriptor_set_t;

/*----------------------------------*/
/** 
 * @brief  Descriptor Table structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t     xftyp;         /*!< transfer type. */
    uint32_t     xfsad;         /*!< source address. */
    uint32_t     xfdad;         /*!< destination address. */
    uint32_t     xfsiz;         /*!< transfer size. */
    uint32_t     dsads;         /*!< next descriptor address. */
    uint32_t     dsnum;         /*!< number of descriptors. */
    uint32_t     res1;          /*!< Reserved. */
    uint32_t     res2;          /*!< Reserved. */
} mdma_descriptor_table_t;

/*----------------------------------*/
/** 
 * @brief  ch bit structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t ch0: 1;          /*!< channel 0. */
    uint32_t ch1: 1;          /*!< channel 1. */
    uint32_t ch2: 1;          /*!< channel 2. */
    uint32_t ch3: 1;          /*!< channel 3. */
    uint32_t ch4: 1;          /*!< channel 4. */
    uint32_t ch5: 1;          /*!< channel 5. */
    uint32_t ch6: 1;          /*!< channel 6. */
    uint32_t ch7: 1;          /*!< channel 7. */
    uint32_t ch8: 1;          /*!< channel 8. */
    uint32_t ch9: 1;          /*!< channel 9. */
    uint32_t ch10: 1;         /*!< channel 10. */
    uint32_t ch11: 1;         /*!< channel 11. */
    uint32_t ch12: 1;         /*!< channel 12. */
    uint32_t ch13: 1;         /*!< channel 13. */
    uint32_t ch14: 1;         /*!< channel 14. */
    uint32_t ch15: 1;         /*!< channel 15. */
    uint32_t ch16: 1;         /*!< channel 16. */
    uint32_t ch17: 1;         /*!< channel 17. */
    uint32_t ch18: 1;         /*!< channel 18. */
    uint32_t ch19: 1;         /*!< channel 19. */
    uint32_t ch20: 1;         /*!< channel 20. */
    uint32_t ch21: 1;         /*!< channel 21. */
    uint32_t ch22: 1;         /*!< channel 22. */
    uint32_t ch23: 1;         /*!< channel 23. */
    uint32_t ch24: 1;         /*!< channel 24. */
    uint32_t ch25: 1;         /*!< channel 25. */
    uint32_t ch26: 1;         /*!< channel 26. */
    uint32_t ch27: 1;         /*!< channel 27. */
    uint32_t ch28: 1;         /*!< channel 28. */
    uint32_t ch29: 1;         /*!< channel 29. */
    uint32_t ch30: 1;         /*!< channel 30. */
    uint32_t ch31: 1;         /*!< channel 31. */
} mdma_bit_t;

/*----------------------------------*/
/** 
 * @brief  mdma ch bit union definition.
*/
/*----------------------------------*/
typedef union{
	mdma_bit_t	bit;		/*!< bit. */
	uint32_t	uword;		/*!< unsigned word */
} mdma_ch_bit_t;

/*----------------------------------*/
/** 
 * @brief  Channel setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    void (*handler)(uint32_t id, TXZ_Result result);        /*!< Notify converted value.  */
    uint32_t        id;                                     /*!< ID: User Value.          */
    uint32_t        priority;                               /*!< Priority. */
} mdma_ch_interrupt_setting_t;


/*----------------------------------*/
/** 
 * @brief  MDMA handle structure definition.
*/
/*----------------------------------*/
typedef struct mdma_handle                           /*!< MDMA interrupt handle. */
{
    TSB_MDMA_TypeDef                 *p_instance;    /*!< Registers base address.           */
    mdma_descriptor_address_t        init_desc;      /*!< Initial descriptor address.       */
    mdma_ch_interrupt_setting_t      int_ch[MDMA_CH_NUM];    /*!<  ch interrupt setting.    */
} mdma_t;

/**
 *  @}
 */ /* End of group MDMA_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Exported_functions DMA Exported Functions
 *  @{
 */
TXZ_Result mdma_init(mdma_t *p_obj);
TXZ_Result mdma_deinit(void);
void mdma_irq_end_handler(void);
void mdma_irq_error_handler(uint32_t reason);
TXZ_Result mdma_desc_ch_configure(uint32_t ch, mdma_descriptor_set_t *desc, uint32_t desc_num, uint32_t desc_infinite);
TXZ_Result mdma_desc_chain_set(uint32_t addr, mdma_descriptor_set_t *desc);
TXZ_Result mdma_unmask_ch(uint32_t ch);
TXZ_Result mdma_mask_ch(uint32_t ch);
TXZ_Result mdma_enable_ch(uint32_t ch);
TXZ_Result mdma_request_ch(uint32_t ch);
TXZ_Result mdma_suspend_ch(uint32_t ch);
TXZ_Result mdma_restart_ch(uint32_t ch);
TXZ_Result mdma_stop_ch(uint32_t ch);
TXZ_Result mdma_end_clear_ch(uint32_t ch);
TXZ_Result mdma_transfer_end_interrupt_enable_ch(uint32_t ch);
TXZ_Result mdma_transfer_end_interrupt_disable_ch(uint32_t ch);
TXZ_Result mdma_basic_start_ch(uint32_t ch, uint32_t interrupt_enable);
TXZ_Result mdma_ch_init(uint32_t ch, mdma_ch_interrupt_setting_t int_ch);
TXZ_Result mdma_ch_deinit(uint32_t ch);
TXZ_Result mdma_force_stop_ch(uint32_t stop_ch, uint32_t work_ch);
/**
 *  @}
 */ /* End of group MDMA_Exported_functions */

/**
 *  @}
 */ /* End of group MDMA */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __MDMA_H */

