/** 
 *******************************************************************************
 * @file    txz_mdma_include.h
 * @brief   This file provides internal common definition.
 * @version V1.0.1.0
 * $Date:: 2018-04-02 19:31:41 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __MDMA_INCLUDE_H
#define __MDMA_INCLUDE_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup MDMA
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup MDMA_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group MDMA_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup MDMA_Private_define
 *  @{
 */

/** 
 *  @defgroup MDMA_NullPointer  Null Pointer
 *  @brief    NULL Pointer.
 *  @{
 */
#define MDMA_NULL        ((void *)0)                       /*!< NULL Pointer.         */
/**
 *  @}
 */ /* End of group MDMA_NullPointer */

/** 
 *  @defgroup MDMA_ParameterResult  Parameter Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define MDMA_PARAM_OK             ((int32_t)0)              /*!< Parameter is valid(specified).         */
#define MDMA_PARAM_NG             ((int32_t)-1)             /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group MDMA_ParameterResult */

/** 
 *  @defgroup MDMA_FORCE_STOP_STATUS   FORCE STOP STATUS
 *  @brief    FORCE STOP STATUS.
 *  @{
 */
#define MDMA_FORCE_STOP_STATUS_IDLE    ((uint32_t)0)       /*!< IDLE.  */
#define MDMA_FORCE_STOP_STATUS_RUN     ((uint32_t)1)       /*!< RUN.   */
#define MDMA_FORCE_STOP_STATUS_DONE    ((uint32_t)2)       /*!< DONE.  */
#define MDMA_FORCE_STOP_STATUS_ERROR   ((uint32_t)3)       /*!< ERROR. */
/**
 *  @}
 */ /* End of group MDMA_FORCE_STOP_STATUS */

/** 
 *  @defgroup TIME_OUT_COUNT  TIME OUT COUNT
 *  @brief    TIME OUT COUNT.
 *  @{
 */
#define MDMA_TIME_OUT_COUNT        ((uint32_t)0x1000000)     /*!< TIME OUT COUNT. */
/**
 *  @}
 */ /* End of group TIME_OUT_COUNT */
/** 
 *  @}
 */ /* End of group MDMA_Private_define */
 
/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup MDMA_Private_functions
 *  @{
 */
__STATIC_INLINE uint32_t convert_ch_to_reg(uint32_t ch);
/*--------------------------------------------------*/
/** 
  * @brief  Convert channel to register.
  * @param  ch :Channel(Interface format).
  * @retval Channel(Register format).
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t convert_ch_to_reg(uint32_t ch)
{
    uint32_t result = (uint32_t)((uint32_t)1 << (uint8_t)ch);

    return (result);
}

/**
 *  @}
 */ /* End of group MDMA_Private_functions */


/**
 *  @}
 */ /* End of group MDMA */
/**
 *  @}
 */ /* End of group Periph_Driver */


#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __MDMA_INCLUDE_H */


