/** 
 *******************************************************************************
 * @file    txz_smif.h
 * @brief   This file provides all the functions prototypes for SMIF Class.
 * @version V1.0.0.0
 * $Date:: 2017-10-24 10:56:00 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __SMIF_H
#define __SMIF_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"
/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup SMIF
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_define
 *  @{
 */

/** 
 *  @name  SMIF_NULL Pointer
 *  @brief NULL Pointer.
 *  @{
 */
#define SMIF_NULL                ((void *)0)
/**
 *  @}
 */ /* End of name SMIF_NULL Pointer */

/** 
 *  @name  SMIF MAX Channel
 *  @brief MAX Chanel.
 *  @{
 */
#define SMIF_MAX_CHANNEL        (1)     /*!< MAX CHANNEL */

/**
 *  @}
 */ /* End of SMIF MAX Channel */

/** 
 *  @}
 */ /* End of group SMIF_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_define
 *  @{
 */

/** 
 *  @defgroup SMIF_BASE_Range  SMIF BASE Range
 *  @brief    Range of SMIF BASE Macro Definition.
 *  @brief    Range of SMIF BASE "(SMIF_MAP_BASE_MIN <= Value <= SMIF_MAP_BASE_MAX)".
 *  @{
 */
#define SMIF_MAP_BASE_MIN              ((uint32_t)0x00000000)    /*!< Minimum  Value:0x0  */
#define SMIF_MAP_BASE_MAX              ((uint32_t)0x00FFFFFF)    /*!< Maximum  Value:0xFFFFFF */
/**
 *  @}
 */ /* End of group SMIF_BASE_Range */

/** 
 *  @defgroup SMIF_Capacity  SMIF Capacity
 *  @brief    SMIF Capacity Macro Definition.
 *  @{
 */
    #define SMIF_FDEN_64KB               (0x00)      /*!<  64KB */
    #define SMIF_FDEN_128KB              (0x01)      /*!< 128KB */
    #define SMIF_FDEN_256KB              (0x02)      /*!< 256KB */
    #define SMIF_FDEN_512KB              (0x03)      /*!< 512KB */
    #define SMIF_FDEN_1MB                (0x04)      /*!<   1MB */
    #define SMIF_FDEN_2MB                (0x05)      /*!<   2MB */
    #define SMIF_FDEN_4MB                (0x06)      /*!<   4MB */
    #define SMIF_FDEN_8MB                (0x07)      /*!<   8MB */
    #define SMIF_FDEN_16MB               (0x08)      /*!<  16MB */
/**
 *  @}
 */ /* End of group SMIF_Capacity */

/** 
 *  @defgroup SMIF_MAP_ENABLE/DISABLE  SMIF MAP ENABLE/DISABLE
 *  @brief    SMIF MAP ENABLE/DISABLE Macro Definition.
 *  @{
 */
    #define SMIF_MAP_DISABLE     (0x00)      /*!< MAP DISABLE.  */
    #define SMIF_MAP_ENABLE      (0x01)      /*!< MAP ENABLE.  */
/**
 *  @}
 */ /* End of group SMIF_WIP_POLLING */

/** 
 *  @defgroup SMIF_TRANS_CLOCK_Range  SMIF TRANS CLOCK Range
 *  @brief    Range of SMIF TRANS CLOCK Macro Definition.
 *  @brief    Range of TRANS CLOCK "(SMIF_TRANS_CLOCK_MIN <= Value <= SMIF_TRANS_CLOCK_MAX)".
 *  @{
 */
#define SMIF_TRANS_CLOCK_MIN              ((uint32_t)0x00000001)    /*!< Minimum  Value:0x1  */
#define SMIF_TRANS_CLOCK_MAX              ((uint32_t)0x0000001F)    /*!< Maximum  Value:0x1F */
/**
 *  @}
 */ /* End of group SMIF_TRANS_CLOCK_Range */
 
/** 
 *  @defgroup SMIF_DEASERT_TIME_Range  SMIF DEASERT_TIME Range
 *  @brief    Range of SMIF DEASERT TIME Macro Definition.
 *  @brief    Range of DEASERT TIME "(SMIF_DEASERT_TIME_MIN <= Value <= SMIF_DEASERT_TIME_MAX)".
 *  @{
 */
#define SMIF_DEASERT_TIME_MIN              ((uint32_t)0x00000000)    /*!< Minimum  Value:0x0  */
#define SMIF_DEASERT_TIME_MAX              ((uint32_t)0x000000FF)    /*!< Maximum  Value:0xFF */
/**
 *  @}
 */ /* End of group SMIF_DEASERT_TIME_Range */

/** 
 *  @defgroup SMIF_WIP_POLLING  SMIF WIP Polling
 *  @brief    SMIF WIP Polling Enable/Disable Macro Definition.
 *  @{
 */
    #define SMIF_WIP_POLLING_DISABLE     (0x00)      /*!< WIP Polling Disable.  */
    #define SMIF_WIP_POLLING_ENABLE      (0x01)      /*!< WIP Polling Enable.  */
/**
 *  @}
 */ /* End of group SMIF_WIP_POLLING */
    
/** 
 *  @defgroup SMIF_DIRECT_COMMAND_OPCODE  SMIF Direct Command Opcode
 *  @brief    SMIF Direct Command Opcode Macro Definition.
 *  @{
 */
    #define SMIF_CMD_FAST_READ           (0x0B)      /*!< Instruction: Fast Read */
    #define SMIF_CMD_DUAL_OUTPUT         (0x3B)      /*!< Instruction: Fast Dual Output */
    #define SMIF_CMD_DUAL_I_O            (0xBB)      /*!< Instruction: Fast Dual I/O */
    #define SMIF_CMD_QUAD_OUTPUT         (0x6B)      /*!< Instruction: Fast QUAD Output */
    #define SMIF_CMD_QUAD_I_O            (0xEB)      /*!< Instruction: Fast QUAD I/O */
/**
 *  @}
 */ /* End of group SMIF_DIRECT_COMMAND_OPCODE */

/** 
 *  @defgroup SMIF_READ_ID_LENGTH  SMIF Read ID Length
 *  @brief    SMIF Read ID Length Macro Definition.
 *  @{
 */
    #define SMIF_READ_ID_LEN        (0x03)      /*!< Read ID Length.  */
/**
 *  @}
 */ /* End of group SMIF_READ_ID_LENGTH */

/** 
 *  @}
 */ /* End of group SMIF_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Mapping setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t            base;           /*!< Serial Memory Mapping Base Address.  */
    uint32_t            capacity;       /*!< Serial Memory Capacity.  */
    uint32_t            enable;         /*!< Mapping Enable. */
} SMIF_mapping_setting_t;

/*----------------------------------*/
/** 
 * @brief  Clock setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t            trans_clock;    /*!< Transfer Clock.      */
    uint32_t            deasert_time;   /*!< CSn_N Deasert time.  */
    uint32_t            poll_wip;       /*!< polling WIP.  */
} SMIF_clock_setting_t;

/*----------------------------------*/
/** 
 * @brief  Direct Read setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t            CmdOP;          /*!< Command Opcode.      */
} SMIF_direct_setting_t;
/*----------------------------------*/
/** 
 * @brief  Initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    SMIF_mapping_setting_t  map[SMIF_MAX_CHANNEL+1];            /* mapping setting.  */
    SMIF_clock_setting_t    clock;          /*!< clock setting.  */
    SMIF_direct_setting_t   direct[SMIF_MAX_CHANNEL+1];         /*!< direct setting. */
} SMIF_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  SMIF handle structure definition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_SMI_TypeDef       *p_instance;     /*!< Registers base address.           */
    SMIF_initial_setting_t init;            /*!< Initial setting.                  */
} SMIF_t;


/**
 *  @}
 */ /* End of group SMIF_Private_typedef */
/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_functions
 *  @{
 */
/** 
 *  @}
 */ /* End of group SMIF_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Exported_functions
 *  @{
 */
TXZ_Result smif_init(SMIF_t *p_obj);                                                            /*!< SMIF init.         */
TXZ_Result smif_finalize(void);                                                                 /*!< SMIF finalize.     */
TXZ_Result smif_erase_chip(uint8_t ch);                                                         /*!< SMIF erase chip.   */
TXZ_Result smif_erase_sector(uint8_t *p_address, uint8_t ch);                                   /*!< SMIF erase sector. */
TXZ_Result smif_write_data(uint8_t *p_dst, uint8_t *p_src, uint32_t length, uint8_t ch);        /*!< SMIF write data.   */
TXZ_Result smif_sflash_read_id(uint8_t *id, uint8_t ch);                                        /*!< SMIF read ID.      */
TXZ_Result smif_bank_read(uint8_t *bank, uint8_t ch);                                           /*!< SMIF back read.    */
TXZ_Result smif_bank_write(uint8_t bank, uint8_t ch);                                           /*!< SMIF bank write.   */
/** 
 *  @}
 */ /* End of group SMIF_Exported_functions */

/**
 *  @}
 */ /* End of group SMIF */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __SMIF_H */
