/** 
 *******************************************************************************
 * @file    txz_smif_include.h
 * @brief   This file provides internal common definition.
 * @version V1.0.0.0
 * $Date:: 2017-10-25 10:43:01 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __SMIF_INCLUDE_H
#define __SMIF_INCLUDE_H    1

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup SMIF
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_define
 *  @{
 */

/** 
 *  @defgroup SMIF_NullPointer  Null Pointer
 *  @brief    Null Pointer.
 *  @{
 */
#define SMIF_NULL        ((void *)0)       /*!< Null Pointer. */
/**
 *  @}
 */ /* End of group SMIF_NullPointer */

/** 
 *  @defgroup SMIF_Timer_Interval  Timer Interval
 *  @brief    Timer Interval.
 *  @{
 */
#define SMIF_TIMER_INTERVAL       (100)       /*!< Timer Interval(us). */
/**
 *  @}
 */ /* End of group SMIF_Timer_Interval */

/** 
 *  @defgroup SMIF_ParameterResult  Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define SMIF_PARAM_OK             ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define SMIF_PARAM_NG             ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group SMIF_ParameterResult */

/** 
 *  @defgroup SMIF_Check_Result  SMIF Check Result
 *  @brief    SMIF Check Result
 *  @{
 */
#define SMIF_E_SFLASH_OK          ((int32_t)0)              /*!< No Error.         */
#define SMIF_E_SFLASH_BUSY        ((int32_t)-1)             /*!< Busy.             */
#define SMIF_E_SFLASH_ABORT       ((int32_t)-2)             /*!< Abort.            */
#define SMIF_E_SFLASH_TIMEOUT     ((int32_t)-3)             /*!< Timeout.          */
#define SMIF_E_SFLASH_ERROR       ((int32_t)-4)             /*!< Error.            */
/**
 *  @}
 */ /* End of group SMIF_Check_Result */

/** 
 *  @defgroup SMIF_Polling_Time  SMIF Polling Time
 *  @brief    SMIF Polling Time
 *  @{
 */
#define SMIF_SFLASH_POLLING_NO_TIME         (0)     /*!<    0us */
#define SMIF_SFLASH_POLLING_TIME_QUAD     (100)     /*!<  100us */
#define SMIF_SFLASH_POLLING_TIME_WRITE    (100)     /*!<  100us */
#define SMIF_SFLASH_POLLING_TIME_ERASE   (5000)     /*!< 5000us */
/**
 *  @}
 */ /* End of group SMIF_Polling_Time */

/** 
 *  @defgroup SMIF_Busy_Time_Out        SMIF Busy Time Out
 *  @brief    SMIF Busy Time Out
 *  @{
 */
#define SMIF_SFLASH_TIMEOUT_US             (300)    /*!<     300us */
#define SMIF_SFLASH_TIMEOUT_QUAD          (3000)    /*!<    3000ms */
#define SMIF_SFLASH_TIMEOUT_WRITE            (3)    /*!<       3ms */
#define SMIF_SFLASH_TIMEOUT_ERASE         (3000)    /*!<   3,000ms */
#define SMIF_SFLASH_TIMEOUT_CHIP_ERASE  (500000)    /*!< 500,000ms */
/**
 *  @}
 */ /* End of group SMIF_Busy_Time_Out */


/** 
 *  @}
 */ /* End of group SMIF_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group SMIF_Private_typedef */

/** 
 *  @addtogroup SMIF_Private_functions
 *  @{
 */
extern TXZ_Result smif_timer_initialize(void);              /*!< SMIF timer init.       */
extern TXZ_Result smif_timer_finalize(void);                /*!< SMIF timer finalize.   */
extern TXZ_Result smif_timer_start(void);                   /*!< SMIF timer start.      */
extern TXZ_Result smif_timer_stop(void);                    /*!< SMIF timer stop.       */
extern uint32_t SysOpe_GetTime(void);                       /*!< SMIF timer get time.       */
extern uint32_t SysOpe_GetElapseTime(uint32_t baseTime);    /*!< SMIF timer get elapse time */
extern void BusyCheckTimer(void);                           /*!< SMIF busy check time.      */

/**
 *  @}
 */ /* End of group SMIF_Private_functions */

/**
 *  @}
 */ /* End of group SMIF */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __SMIF_INCLUDE_H */


