/**
 *******************************************************************************
 * @file    txz_cec.c
 * @brief   This file provides API functions for FUART driver.
 * @version V1.0.0.0
 * $Date:: 2017-09-20 10:43:01 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_cec_include.h"
#include "txz_cec.h"

#if defined(__CEC_H)
/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup CEC
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Private_define CEC Private Define
 *  @{
 */


/** 
 *  @}
 */ /* End of group CEC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Private_define CEC Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group CEC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Private_define CEC Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group CEC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Private_typedef CEC Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group CEC_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Private_fuctions CEC Private Fuctions
 *  @{
 */
#ifdef DEBUG
    __STATIC_INLINE int32_t check_param_clk(uint32_t param);
    __STATIC_INLINE int32_t check_param_logical_address(uint32_t param);
    __STATIC_INLINE int32_t check_param_ack_disable(uint32_t param);
    __STATIC_INLINE int32_t check_param_high_noise_cancel(uint32_t param);
    __STATIC_INLINE int32_t check_param_low_noise_cancel(uint32_t param);
    __STATIC_INLINE int32_t check_param_interval_min(uint32_t param);
    __STATIC_INLINE int32_t check_param_interval_max(uint32_t param);
    __STATIC_INLINE int32_t check_param_data_point(uint32_t param);
    __STATIC_INLINE int32_t check_param_time_out(uint32_t param);
    __STATIC_INLINE int32_t check_param_receive_error_hold(uint32_t param);
    __STATIC_INLINE int32_t check_param_mismach_receive(uint32_t param);
    __STATIC_INLINE int32_t check_param_start_bit_fall_max(uint32_t param);
    __STATIC_INLINE int32_t check_param_start_bit_fall_min(uint32_t param);
    __STATIC_INLINE int32_t check_param_start_bit_rise_max(uint32_t param);
    __STATIC_INLINE int32_t check_param_start_bit_rise_min(uint32_t param);
    __STATIC_INLINE int32_t check_param_data_0_rise_max(uint32_t param);
    __STATIC_INLINE int32_t check_param_data_0_rise_min(uint32_t param);
    __STATIC_INLINE int32_t check_param_data_1_rise_max(uint32_t param);
    __STATIC_INLINE int32_t check_param_data_1_rise_min(uint32_t param);
    __STATIC_INLINE int32_t check_param_start_bit_enable(uint32_t param);
    __STATIC_INLINE int32_t check_param_wav_error_enable(uint32_t param);
    __STATIC_INLINE int32_t check_param_start_bit_rise_timing(uint32_t param);
    __STATIC_INLINE int32_t check_param_start_bit_interval_timing(uint32_t param);
    __STATIC_INLINE int32_t check_param_data_bit_rise_timing(uint32_t param);
    __STATIC_INLINE int32_t check_param_data_bit_interval_timing(uint32_t param);
    __STATIC_INLINE int32_t check_param_broadcast(uint32_t param);
    __STATIC_INLINE int32_t check_param_bus_free_wait(uint32_t param);
#endif /* #ifdef DEBUG */

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Check the clk's parameter.
  * @param  param    :clk's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_ACKDIS
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_clk(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxFSSEL_CECCLK_FS:
    case CECxFSSEL_CECCLK_TBXOUT:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the logical_address's parameter.
  * @param  param    :logical_address's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CECxADD
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_logical_address(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    if( ((param & 0xFFFF0000) == 0x00) && ((param & 0x0000FFFF) != 0) )
    {
        result = CEC_PARAM_OK;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the ack_disable's parameter.
  * @param  param    :ack_disable's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_ACKDIS
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ack_disable(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECACKDIS_ACK_ON:
    case CECxRCR1_CECACKDIS_ACK_OFF:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the high_noise_cancel's parameter.
  * @param  param    :high_noise_cancel's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_HNC
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_high_noise_cancel(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECHNC_NONE:
    case CECxRCR1_CECHNC_1_FS:
    case CECxRCR1_CECHNC_2_FS:
    case CECxRCR1_CECHNC_3_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the low_noise_cancel's parameter.
  * @param  param    :low_noise_cancel's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_LNC
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_low_noise_cancel(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECLNC_NONE:
    case CECxRCR1_CECLNC_1_FS:
    case CECxRCR1_CECLNC_2_FS:
    case CECxRCR1_CECLNC_3_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the interval_min's parameter.
  * @param  param    :interval_min's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_MIN
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interval_min(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECMIN_NONE:
    case CECxRCR1_CECMIN_P_1_FS:
    case CECxRCR1_CECMIN_P_2_FS:
    case CECxRCR1_CECMIN_P_3_FS:
    case CECxRCR1_CECMIN_M_1_FS:
    case CECxRCR1_CECMIN_M_2_FS:
    case CECxRCR1_CECMIN_M_3_FS:
    case CECxRCR1_CECMIN_M_4_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the interval_max's parameter.
  * @param  param    :interval_max's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_MAX
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interval_max(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECMAX_NONE:
    case CECxRCR1_CECMAX_P_1_FS:
    case CECxRCR1_CECMAX_P_2_FS:
    case CECxRCR1_CECMAX_P_3_FS:
    case CECxRCR1_CECMAX_M_1_FS:
    case CECxRCR1_CECMAX_M_2_FS:
    case CECxRCR1_CECMAX_M_3_FS:
    case CECxRCR1_CECMAX_M_4_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the data_point's parameter.
  * @param  param    :data_point's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_DAT
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_data_point(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECDAT_NONE:
    case CECxRCR1_CECDAT_P_2_FS:
    case CECxRCR1_CECDAT_P_4_FS:
    case CECxRCR1_CECDAT_P_6_FS:
    case CECxRCR1_CECDAT_M_2_FS:
    case CECxRCR1_CECDAT_M_4_FS:
    case CECxRCR1_CECDAT_M_6_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the time_out's parameter.
  * @param  param    :time_out's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_TOUT
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_time_out(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECTOUT_1BIT:
    case CECxRCR1_CECTOUT_2BIT:
    case CECxRCR1_CECTOUT_3BIT:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the receive_error_hold's parameter.
  * @param  param    :receive_error_hold's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_RIHLD
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_receive_error_hold(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECRIHLD_OFF:
    case CECxRCR1_CECRIHLD_ON:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the mismach_receive's parameter.
  * @param  param    :mismach_receive's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_OTH
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_mismach_receive(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR1_CECOTH_OFF:
    case CECxRCR1_CECOTH_ON:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the start_bit_fall_max's parameter.
  * @param  param    :start_bit_fall_max's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_START_FMAX
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_start_bit_fall_max(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR2_CECSWAV3_NONE:
    case CECxRCR2_CECSWAV3_1_FS:
    case CECxRCR2_CECSWAV3_2_FS:
    case CECxRCR2_CECSWAV3_3_FS:
    case CECxRCR2_CECSWAV3_4_FS:
    case CECxRCR2_CECSWAV3_5_FS:
    case CECxRCR2_CECSWAV3_6_FS:
    case CECxRCR2_CECSWAV3_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the start_bit_fall_min's parameter.
  * @param  param    :start_bit_fall_min's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_START_FMIN
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_start_bit_fall_min(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR2_CECSWAV2_NONE:
    case CECxRCR2_CECSWAV2_1_FS:
    case CECxRCR2_CECSWAV2_2_FS:
    case CECxRCR2_CECSWAV2_3_FS:
    case CECxRCR2_CECSWAV2_4_FS:
    case CECxRCR2_CECSWAV2_5_FS:
    case CECxRCR2_CECSWAV2_6_FS:
    case CECxRCR2_CECSWAV2_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the start_bit_rise_max's parameter.
  * @param  param    :start_bit_rise_max's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_START_RMAX
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_start_bit_rise_max(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR2_CECSWAV1_NONE:
    case CECxRCR2_CECSWAV1_1_FS:
    case CECxRCR2_CECSWAV1_2_FS:
    case CECxRCR2_CECSWAV1_3_FS:
    case CECxRCR2_CECSWAV1_4_FS:
    case CECxRCR2_CECSWAV1_5_FS:
    case CECxRCR2_CECSWAV1_6_FS:
    case CECxRCR2_CECSWAV1_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the start_bit_rise_min's parameter.
  * @param  param    :start_bit_rise_min's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_START_RMIN
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_start_bit_rise_min(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR2_CECSWAV0_NONE:
    case CECxRCR2_CECSWAV0_1_FS:
    case CECxRCR2_CECSWAV0_2_FS:
    case CECxRCR2_CECSWAV0_3_FS:
    case CECxRCR2_CECSWAV0_4_FS:
    case CECxRCR2_CECSWAV0_5_FS:
    case CECxRCR2_CECSWAV0_6_FS:
    case CECxRCR2_CECSWAV0_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the data_0_rise_max's parameter.
  * @param  param    :data_0_rise_max's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_DATA0_RMAX
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_data_0_rise_max(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR3_CECWAV3_NONE:
    case CECxRCR3_CECWAV3_1_FS:
    case CECxRCR3_CECWAV3_2_FS:
    case CECxRCR3_CECWAV3_3_FS:
    case CECxRCR3_CECWAV3_4_FS:
    case CECxRCR3_CECWAV3_5_FS:
    case CECxRCR3_CECWAV3_6_FS:
    case CECxRCR3_CECWAV3_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the data_0_rise_min's parameter.
  * @param  param    :data_0_rise_min's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_DATA0_RMIN
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_data_0_rise_min(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR3_CECWAV2_NONE:
    case CECxRCR3_CECWAV2_1_FS:
    case CECxRCR3_CECWAV2_2_FS:
    case CECxRCR3_CECWAV2_3_FS:
    case CECxRCR3_CECWAV2_4_FS:
    case CECxRCR3_CECWAV2_5_FS:
    case CECxRCR3_CECWAV2_6_FS:
    case CECxRCR3_CECWAV2_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the data_1_rise_max's parameter.
  * @param  param    :data_1_rise_max's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_DATA1_RMAX
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_data_1_rise_max(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR3_CECWAV1_NONE:
    case CECxRCR3_CECWAV1_1_FS:
    case CECxRCR3_CECWAV1_2_FS:
    case CECxRCR3_CECWAV1_3_FS:
    case CECxRCR3_CECWAV1_4_FS:
    case CECxRCR3_CECWAV1_5_FS:
    case CECxRCR3_CECWAV1_6_FS:
    case CECxRCR3_CECWAV1_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the data_1_rise_min's parameter.
  * @param  param    :data_1_rise_min's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_DATA1_RMIN
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_data_1_rise_min(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR3_CECWAV0_NONE:
    case CECxRCR3_CECWAV0_1_FS:
    case CECxRCR3_CECWAV0_2_FS:
    case CECxRCR3_CECWAV0_3_FS:
    case CECxRCR3_CECWAV0_4_FS:
    case CECxRCR3_CECWAV0_5_FS:
    case CECxRCR3_CECWAV0_6_FS:
    case CECxRCR3_CECWAV0_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the start_bit_enable's parameter.
  * @param  param    :start_bit_enable's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_RSTAEN
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_start_bit_enable(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR3_CECRSTAEN_DISABLE:
    case CECxRCR3_CECRSTAEN_ENABLE:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the wav_error_enable's parameter.
  * @param  param    :wav_error_enable's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_WAVEN
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wav_error_enable(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxRCR3_CECWAVEN_DISABLE:
    case CECxRCR3_CECWAVEN_ENABLE:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the start_bit_rise_timing's parameter.
  * @param  param    :start_bit_rise_timing's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_STRS
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_start_bit_rise_timing(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxTCR_CECSTRS_NONE:
    case CECxTCR_CECSTRS_1_FS:
    case CECxTCR_CECSTRS_2_FS:
    case CECxTCR_CECSTRS_3_FS:
    case CECxTCR_CECSTRS_4_FS:
    case CECxTCR_CECSTRS_5_FS:
    case CECxTCR_CECSTRS_6_FS:
    case CECxTCR_CECSTRS_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the start_bit_interval_timing's parameter.
  * @param  param    :start_bit_interval_timing's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_SPRD
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_start_bit_interval_timing(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxTCR_CECSPRD_NONE:
    case CECxTCR_CECSPRD_1_FS:
    case CECxTCR_CECSPRD_2_FS:
    case CECxTCR_CECSPRD_3_FS:
    case CECxTCR_CECSPRD_4_FS:
    case CECxTCR_CECSPRD_5_FS:
    case CECxTCR_CECSPRD_6_FS:
    case CECxTCR_CECSPRD_7_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the data_bit_rise_timing's parameter.
  * @param  param    :data_bit_rise_timing's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_DTR2
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_data_bit_rise_timing(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxTCR_CECDTRS_NONE:
    case CECxTCR_CECDTRS_1_FS:
    case CECxTCR_CECDTRS_2_FS:
    case CECxTCR_CECDTRS_3_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the data_bit_interval_timing's parameter.
  * @param  param    :data_bit_interval_timing's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_DPRD
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_data_bit_interval_timing(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxTCR_CECDPRD_NONE:
    case CECxTCR_CECDPRD_1_FS:
    case CECxTCR_CECDPRD_2_FS:
    case CECxTCR_CECDPRD_3_FS:
    case CECxTCR_CECDPRD_4_FS:
    case CECxTCR_CECDPRD_5_FS:
    case CECxTCR_CECDPRD_6_FS:
    case CECxTCR_CECDPRD_7_FS:
    case CECxTCR_CECDPRD_8_FS:
    case CECxTCR_CECDPRD_9_FS:
    case CECxTCR_CECDPRD_10_FS:
    case CECxTCR_CECDPRD_11_FS:
    case CECxTCR_CECDPRD_12_FS:
    case CECxTCR_CECDPRD_13_FS:
    case CECxTCR_CECDPRD_14_FS:
    case CECxTCR_CECDPRD_15_FS:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the broadcast's parameter.
  * @param  param    :broadcast's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_BRD
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_broadcast(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxTCR_CECBRD_OFF:
    case CECxTCR_CECBRD_ON:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the bus_free_wait's parameter.
  * @param  param    :bus_free_wait's parameter
  * @retval CEC_PARAM_OK :Valid
  * @retval CEC_PARAM_NG :Invalid
  * @note   Macro definition is @ref CEC_FREE
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_bus_free_wait(uint32_t param)
{
    int32_t result = CEC_PARAM_NG;

    switch (param) 
    {
    case CECxTCR_CECFREE_NONE:
    case CECxTCR_CECFREE_1_BIT:
    case CECxTCR_CECFREE_2_BIT:
    case CECxTCR_CECFREE_3_BIT:
    case CECxTCR_CECFREE_4_BIT:
    case CECxTCR_CECFREE_5_BIT:
    case CECxTCR_CECFREE_6_BIT:
    case CECxTCR_CECFREE_7_BIT:
    case CECxTCR_CECFREE_8_BIT:
    case CECxTCR_CECFREE_9_BIT:
    case CECxTCR_CECFREE_10_BIT:
    case CECxTCR_CECFREE_11_BIT:
    case CECxTCR_CECFREE_12_BIT:
    case CECxTCR_CECFREE_13_BIT:
    case CECxTCR_CECFREE_14_BIT:
    case CECxTCR_CECFREE_15_BIT:
        result = CEC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
#endif /* #ifdef DEBUG */

/**
 *  @}
 */ /* End of group CEC_Private_functions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CEC_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the CEC object.
  * @param     p_obj   :CEC object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result cec_init(cec_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_clk(p_obj->init.clk));
    assert_param(check_param_logical_address(p_obj->init.logical_address));
    assert_param(check_param_ack_disable(p_obj->init.ack_disable));
    assert_param(check_param_high_noise_cancel(p_obj->init.high_noise_cancel));
    assert_param(check_param_low_noise_cancel(p_obj->init.low_noise_cancel));
    assert_param(check_param_interval_min(p_obj->init.interval_min));
    assert_param(check_param_interval_max(p_obj->init.interval_max));
    assert_param(check_param_data_point(p_obj->init.data_point));
    assert_param(check_param_time_out(p_obj->init.time_out));
    assert_param(check_param_receive_error_hold(p_obj->init.receive_error_hold));
    assert_param(check_param_mismach_receive(p_obj->init.mismach_receive));
    assert_param(check_param_start_bit_fall_max(p_obj->init.start_bit_fall_max));
    assert_param(check_param_start_bit_fall_min(p_obj->init.start_bit_fall_min));
    assert_param(check_param_start_bit_rise_max(p_obj->init.start_bit_rise_max));
    assert_param(check_param_start_bit_rise_min(p_obj->init.start_bit_rise_min));
    assert_param(check_param_data_0_rise_max(p_obj->init.data_0_rise_max));
    assert_param(check_param_data_0_rise_min(p_obj->init.data_0_rise_min));
    assert_param(check_param_data_1_rise_max(p_obj->init.data_1_rise_max));
    assert_param(check_param_data_1_rise_min(p_obj->init.data_1_rise_min));
    assert_param(check_param_start_bit_enable(p_obj->init.start_bit_enable));
    assert_param(check_param_wav_error_enable(p_obj->init.wav_error_enable));
    assert_param(check_param_start_bit_rise_timing(p_obj->init.start_bit_rise_timing));
    assert_param(check_param_start_bit_interval_timing(p_obj->init.start_bit_interval_timing));
    assert_param(check_param_data_bit_rise_timing(p_obj->init.data_bit_rise_timing));
    assert_param(check_param_data_bit_interval_timing(p_obj->init.data_bit_interval_timing));
    assert_param(check_param_bus_free_wait(p_obj->init.bus_free_wait));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Enable CEC                */
    /*------------------------------*/
    p_obj->p_instance->EN = CECxEN_CECEN_ENABLE;
    /*------------------------------*/
    /*  CEC Soft Reset              */
    /*------------------------------*/
    p_obj->p_instance->RESET = CECxRESET_CECRESET_ENABLE;
    p_obj->p_instance->REN = CECxREN_CECREN_DISABLE;
    p_obj->p_instance->TEN = CECxTEN_CECTEN_DISABLE;
    /*------------------------------*/
    /*  CLK                         */
    /*------------------------------*/
    p_obj->p_instance->FSSEL = p_obj->init.clk;
    /*------------------------------*/
    /*  Logical Address             */
    /*------------------------------*/
    p_obj->p_instance->ADD = p_obj->init.logical_address;
    /*------------------------------*/
    /*  Receive Control             */
    /*------------------------------*/
    p_obj->p_instance->RCR1 = p_obj->init.ack_disable 
                            | p_obj->init.high_noise_cancel
                            | p_obj->init.low_noise_cancel
                            | p_obj->init.interval_min
                            | p_obj->init.interval_max
                            | p_obj->init.data_point
                            | p_obj->init.time_out
                            | p_obj->init.receive_error_hold
                            | p_obj->init.mismach_receive;
                            
    p_obj->p_instance->RCR2 = p_obj->init.start_bit_fall_max
                            | p_obj->init.start_bit_fall_min
                            | p_obj->init.start_bit_rise_max
                            | p_obj->init.start_bit_rise_min;
                            
    p_obj->p_instance->RCR3 = p_obj->init.data_0_rise_max
                            | p_obj->init.data_0_rise_min
                            | p_obj->init.data_1_rise_max
                            | p_obj->init.data_1_rise_min
                            | p_obj->init.start_bit_enable
                            | p_obj->init.wav_error_enable;
    /*------------------------------*/
    /*  Transfer Control            */
    /*------------------------------*/
    p_obj->p_instance->TCR  = p_obj->init.start_bit_rise_timing
                            | p_obj->init.start_bit_interval_timing
                            | p_obj->init.data_bit_rise_timing
                            | p_obj->init.data_bit_interval_timing
							| CECxTCR_CECBRD_OFF
                            | p_obj->init.bus_free_wait;
    /*------------------------------*/
    /*  Interrupt Clear              */
    /*------------------------------*/
    
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Release the CEC object.
  * @param     p_obj   :CEC object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result cec_deinit(cec_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  CEC Soft Reset              */
    /*------------------------------*/
    p_obj->p_instance->RESET = CECxRESET_CECRESET_ENABLE;
    p_obj->p_instance->REN = CECxREN_CECREN_DISABLE;
    p_obj->p_instance->TEN = CECxTEN_CECTEN_DISABLE;
    /*------------------------------*/
    /*  Disable CEC                */
    /*------------------------------*/
    p_obj->p_instance->EN = CECxEN_CECEN_DISABLE;
    /*------------------------------*/
    /*  Interrupt Disable           */
    /*------------------------------*/
    /*------------------------------*/
    /*  Interrupt Clear              */
    /*------------------------------*/
    /*------------------------------*/
    /*  FIFO Disable                */
    /*------------------------------*/
    
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Discard transmit.
  * @param     p_obj   :CEC object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      This function clears transmit's fifo, end flag and error info.
  * @attention This function is not available in interrupt.
  * @attention Use after interrupt is disabled.
  */
/*--------------------------------------------------*/
TXZ_Result cec_discard_transmit(cec_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Disable Transfer            */
    /*------------------------------*/
    /* Write to TEN(=0).               */
    p_obj->p_instance->TEN = CECxTEN_CECTEN_DISABLE;
    /*------------------------------*/
    /*  Interrupt Clear              */
    /*------------------------------*/
    
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Discard receive.
  * @param     p_obj   :CEC object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      This function clears receive's fifo, end flag and error info.
  * @attention This function is not available in interrupt.
  * @attention Use after interrupt is disabled.
  */
/*--------------------------------------------------*/
TXZ_Result cec_discard_receive(cec_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Disable Transfer            */
    /*------------------------------*/
    /* Write to REN(=0).               */
    p_obj->p_instance->REN = CECxREN_CECREN_DISABLE;
    /*------------------------------*/
    /*  Interrupt Clear              */
    /*------------------------------*/

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Transmit data. Non-Blocking Communication.
  * @param  p_obj   :CEC object.
  * @param  p_info  :The information of transmit data.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Asynchronous Processing.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result cec_transmitIt(cec_t *p_obj, cec_transmit_t *p_info)
{
    TXZ_Result result  = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_info));
    /* Check the parameter of transmit. */
    assert_param(IS_POINTER_NOT_NULL(p_info->p_data));
    assert_param(check_param_broadcast(p_info->broadcast));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Disable Transmit            */
    /*------------------------------*/
    /* Write to TEN(=0).               */
    p_obj->p_instance->TEN = CECxTEN_CECTEN_DISABLE;
    {
        uint32_t    count = 0;
        volatile uint32_t    monitor;
        while(++count != 0)
        {
            monitor = p_obj->p_instance->TEN;
            if((monitor & CECxTEN_CECTRANS_MASK) == CECxTEN_CECTRANS_STOP)
            {
                break;
            }
        }
    }
    /*------------------------------*/
    /*  Information Setting         */
    /*------------------------------*/
    p_obj->transmit.rp = 0;
    p_obj->transmit.info.p_data  = p_info->p_data;
    p_obj->transmit.info.num     = p_info->num;
    if(p_info->broadcast == CECxTCR_CECBRD_ON)
    {
        p_obj->p_instance->TCR  |= CECxTCR_CECBRD_ON;
    }
    else
    {
        p_obj->p_instance->TCR  &= ~CECxTCR_CECBRD_ON;
    }
    /*------------------------------*/
    /*  Data Setting                */
    /*------------------------------*/
    {
        /*--- CECxDR ---*/
        if (p_obj->transmit.info.num > p_obj->transmit.rp)
        {
            if(p_obj->transmit.info.num == (p_obj->transmit.rp + 1))
            {
                p_obj->p_instance->TBUF  = ((*(p_obj->transmit.info.p_data + p_obj->transmit.rp) & CECxRBUF_CECTBUF_MASK ) | CECxTBUF_CECTEOM_ON);
                p_obj->transmit.rp    += 1;
            }
            else
            {
                p_obj->p_instance->TBUF  = (*(p_obj->transmit.info.p_data + p_obj->transmit.rp) & CECxRBUF_CECTBUF_MASK );
                p_obj->transmit.rp    += 1;
            }
        }
    }
    /*------------------------------*/
    /*  Enable Transmit             */
    /*------------------------------*/
    /* Write to TEN(=1).               */
    p_obj->p_instance->TEN = CECxTEN_CECTEN_ENABLE;

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Receive data. Non-Blocking Communication.
  * @param  p_obj   :CEC object.
  * @param  p_info  :The information of receive buffer.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Asynchronous Processing.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result cec_receiveIt(cec_t *p_obj, cec_receive_t *p_info)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_info));
    /* Check the parameter of transmit. */
    assert_param(IS_POINTER_NOT_NULL(p_info->p_data));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Disable Receive             */
    /*------------------------------*/
    /* Write to REN(=0).               */
    p_obj->p_instance->REN = CECxREN_CECREN_DISABLE;
    
    {
        uint32_t    count = 0;
        volatile uint32_t    monitor;
        while(++count != 0)
        {
            monitor = p_obj->p_instance->REN;
            if((monitor & CECxREN_CECREN_MASK) == CECxREN_CECREN_STOP)
            {
                break;
            }
        }
    }
    /*------------------------------*/
    /*  Information Setting         */
    /*------------------------------*/
    p_obj->receive.wp = 0;
    p_obj->receive.info.p_data  = p_info->p_data;
    p_obj->receive.info.num     = p_info->num;
    /*------------------------------*/
    /*  Enable Receive              */
    /*------------------------------*/
    /* Write to REN(=0).               */
    p_obj->p_instance->REN = CECxREN_CECREN_ENABLE;

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  IRQ Handler for transmit.
  * @param  p_obj  :CEC object.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void cec_transmit_irq_handler(cec_t *p_obj)
{
    uint32_t tx_status;
    TXZ_Result result = TXZ_SUCCESS;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef DEBUG */
    tx_status = p_obj->p_instance->TSTAT;
    if (p_obj->transmit.info.num <= p_obj->transmit.rp)
    {
        if((tx_status & CECxTSTAT_CECTIEND_MASK) == CECxTSTAT_CECTIEND_SET)  /* all message transfer end */
        {
            /*=== Transmit Done!! ===*/
            /*------------------------------*/
            /* Call Handler                 */
            /*------------------------------*/
            if (p_obj->transmit.handler != CEC_NULL)
            {
                /* Call the transmit handler with TXZ_SUCCESS. */
                p_obj->transmit.handler(p_obj->init.id, TXZ_SUCCESS, tx_status);
            }
        }
        else
        {
            if (p_obj->transmit.handler != CEC_NULL)
            {
                p_obj->transmit.handler(p_obj->init.id, TXZ_ERROR, tx_status);
            }
        }
    }
    else
    {
        if((tx_status & CECxTSTAT_CECTIUR_MASK) == CECxTSTAT_CECTIUR_SET)  /* transfer buffer under run */
        {
            result = TXZ_ERROR;
        }
        if((tx_status & CECxTSTAT_CECTIACK_MASK) == CECxTSTAT_CECTIACK_SET)  /* ack error */
        {
            result = TXZ_ERROR;
        }
        if((tx_status & CECxTSTAT_CECTIAL_MASK) == CECxTSTAT_CECTIAL_SET)  /* arbitration lost error */
        {
            result = TXZ_ERROR;
        }
        if((tx_status & CECxTSTAT_CECTIEND_MASK) == CECxTSTAT_CECTIEND_SET)  /* all message transfer end */
        {
            result = TXZ_ERROR;
        }
        if((tx_status & CECxTSTAT_CECTISTA_MASK) == CECxTSTAT_CECTISTA_SET)  /* trasfer start */
        {
            /* no process */
        }
        if( result == TXZ_SUCCESS)
        {
            /*=== Transmit Continue ===*/
            /*------------------------------*/
            /*  Data Setting                */
            /*------------------------------*/
            if(p_obj->transmit.info.num == (p_obj->transmit.rp + 1))
            {
                p_obj->p_instance->TBUF  = ((*(p_obj->transmit.info.p_data + p_obj->transmit.rp) & CECxRBUF_CECTBUF_MASK ) | CECxTBUF_CECTEOM_ON);
                p_obj->transmit.rp    += 1;
            }
            else
            {
                p_obj->p_instance->TBUF  = (*(p_obj->transmit.info.p_data + p_obj->transmit.rp) & CECxRBUF_CECTBUF_MASK );
                p_obj->transmit.rp    += 1;
            }
        }
        else
        {
            if (p_obj->transmit.handler != CEC_NULL)
            {
                p_obj->transmit.handler(p_obj->init.id, TXZ_ERROR, tx_status);
            }
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief  IRQ Handler for receive.
  * @param  p_obj  :CEC object.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void cec_receive_irq_handler(cec_t *p_obj)
{
    uint32_t ren_reg;
    uint32_t rbuf_reg;
    uint32_t rx_status;
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Trans Registar              */
    /*------------------------------*/
    /* Read current CECxREN */
    rbuf_reg = p_obj->p_instance->RBUF;
    rx_status = p_obj->p_instance->RSTAT;
    ren_reg = p_obj->p_instance->REN;
    /*------------------------------*/
    /*  Receive Status Check        */
    /*------------------------------*/
    if (ren_reg  == CECxREN_CECREN_ENABLE)
    {
        if((rx_status & CECxRSTAT_CECRIWAV_MASK) == CECxRSTAT_CECRIWAV_SET)     /* wav error */
        {
            result = TXZ_ERROR;
        }
        if((rx_status & CECxRSTAT_CECRIOR_MASK) == CECxRSTAT_CECRIOR_SET)  /* receive buffer over run error */
        {
            result = TXZ_ERROR;
        }
        if((rx_status & CECxRSTAT_CECRIACK_MASK) == CECxRSTAT_CECRIACK_SET)  /* ack error */
        {
            result = TXZ_ERROR;
        }
        if((rx_status & CECxRSTAT_CECRIMIN_MASK) == CECxRSTAT_CECRIMIN_SET)  /* min interval error */
        {
            result = TXZ_ERROR;
        }
        if((rx_status & CECxRSTAT_CECRIMAX_MASK) == CECxRSTAT_CECRIMAX_SET)  /* max ivterval error */
        {
            result = TXZ_ERROR;
        }
        if((rx_status & CECxRSTAT_CECRISTA_MASK) == CECxRSTAT_CECRISTA_SET)  /* start bit detect */
        {
            /* no process */
        }
        if( result == TXZ_SUCCESS)
        {
            if((rx_status & CECxRSTAT_CECRIEND_MASK) == CECxRSTAT_CECRIEND_SET)  /* 1byte receive complete */
            {
                if(p_obj->receive.wp  > p_obj->receive.info.num)
                {
                    if(p_obj->receive.handler != CEC_NULL)
                    {
                        cec_receive_t param;
                        
                        rx_status |= CECxRSTAT_CECRIBFOV_SET;
                        param.p_data  = p_obj->receive.info.p_data;
                        param.num     = p_obj->receive.wp;
                        p_obj->receive.wp = 0;
                        /* Call the receive handler with TXZ_SUCCESS. */
                        p_obj->receive.handler(p_obj->init.id, TXZ_ERROR, &param, rx_status);
                    }
                }
                else
                {
                    *(p_obj->receive.info.p_data + p_obj->receive.wp)  =  (uint8_t)(rbuf_reg  & CECxRBUF_CECRBUF_MASK);
                    p_obj->receive.wp += 1;
                    if((rbuf_reg  & CECxRBUF_CECEOM_MASK) == CECxRBUF_CECEOM_SET)
                    {
                        /*------------------------------*/
                        /* Call Handler                 */
                        /*------------------------------*/
                        if(p_obj->receive.handler != CEC_NULL)
                        {
                            cec_receive_t param;
                            param.p_data  = p_obj->receive.info.p_data;
                            param.num     = p_obj->receive.wp;
                            p_obj->receive.wp = 0;
                            /* Call the receive handler with TXZ_SUCCESS. */
                            p_obj->receive.handler(p_obj->init.id, TXZ_SUCCESS, &param, rx_status);
                        }
                    }
                }
            }
        }
        else{
            p_obj->receive.handler(p_obj->init.id, TXZ_ERROR, CEC_NULL, rx_status);
        }
    }
}


/*--------------------------------------------------*/
/**
  * @brief  Get TX status.
  * @details  Status bits.
  * | Bit   | Bit Symbol | Function                         |
  * | 31-5  | -          | -                                |
  * | 4     | CECTIUR    | trasfer buffer under run flag.   |
  * | 3     | CECTIACK   | ACK Error flag.                  |
  * | 2     | CECTIAL    | Arbitration lost flag.           |
  * | 1     | CECTEND    | ALL Message transfer end flag.   |
  * | 0     | CECTISTA   | transfer start flag.             |
  * 
  * @param  p_obj    :CEC object.
  * @param  p_tx_status :Save area for status.
  * @retval TXZ_SUCCESS  :Success.
  * @retval TXZ_ERROR    :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result cec_get_tx_status(cec_t *p_obj, uint32_t *p_tx_status)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_tx_status));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Status Read                 */
    /*------------------------------*/
    /*--- CECxFR ---*/
    /* Read current CECxFR. */
    *p_tx_status = p_obj->p_instance->TSTAT;

    return (result);
}


/*--------------------------------------------------*/
/**
  * @brief  Get RX status.
  * @details  RX Status bits.
  * | Bit   | Bit Symbol | Function                     |
  * | 31-7  | -          | -                            |
  * | 6     | CECRIWAV   | WAV Error Interrupt flag.    |
  * | 5     | CECRIOR    | Over run flag.               |
  * | 4     | CECRIACK   | ACK Error flag.              |
  * | 3     | CECRIMIN   | Min. interval Error flag.    |
  * | 2     | CECRIMAX   | Max. interval Error flag.    |
  * | 1     | CECRISTA   | Start bit detect flag.       |
  * | 0     | CECRIEND   | Receive complete flag.       |
  * 
  * @param  p_obj    :CEC object.
  * @param  p_rx_status :Save area for status.
  * @retval TXZ_SUCCESS  :Success.
  * @retval TXZ_ERROR    :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result cec_get_rx_status(cec_t *p_obj, uint32_t *p_rx_status)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the CEC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_rx_status));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Status Read                 */
    /*------------------------------*/
    /*--- CECxFR ---*/
    /* Read current CECxFR. */
    *p_rx_status = p_obj->p_instance->RSTAT;

    return (result);
}


/**
 *  @}
 */ /* End of group CEC_Exported_functions */

/**
 *  @} 
 */ /* End of group CEC */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__CEC_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

