/**
 *******************************************************************************
 * @file    txz_smif.c
 * @brief   This file provides API functions for SMIF Class.
 * @version V1.0.0.0
 * $Date:: 2017-10-24 00:00:00 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_smif.h"
#include "txz_smif_include.h"
#if defined(__SMIF_H)

/**
 *  @addtogroup Periph_Driver
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_define 
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_define 
 *  @{
 */

#define     SFLASH_PAGE_SIZE        (256)   /*!< FLASH WRITE PGAE SIZE */

/** 
 *  @defgroup Device_Status_Result  Device_Status_Result
 *  @brief    Device Status Result
 *  @{
 */
#define     DEVICE_IDLE             (0)     /*!< DEVICE IDLE */
#define     DEVICE_BUSY             (1)     /*!< DEVICE BUSY */
#define     DEVICE_ERR              (-1)    /*!< DEVICE ERROR */
/**
 *  @}
 */ /* End of group Device_Status_Result */

/** 
 *  @defgroup Command_State Command State
 *  @brief    Command State
 *  @{
 */
#define    STATE_IDLE           (0)         /*!< STATE IDLE */
#define    STATE_CMD_BUSY       (1)         /*!< STATE COMMAMD BUSY */
/**
 *  @}
 */ /* End of group Command State */

/** 
 *  @defgroup BUSY_Check_State  BUSY Check State
 *  @brief    BUSY Check State
 *  @{
 */
#define    SMIF_BUSY_CHECK_OFF   (0)        /*!< busy check off */
#define    SMIF_BUSY_CHECK_ON    (1)        /*!< busy check on */
/**
 *  @}
 */ /* End of group Command State */

#define    SFLASH_INT_TIMEOUT_COUNT (100000)        /*!< command transfer time out */

#define     CMD_WRITE_PAGE          (0x02)  /*!< Instruction: Write Page */
#define     CMD_ERASE_CHIP          (0xC7)  /*!< Instruction: Erase Full Memory Array */
#define     CMD_ERASE_SECTOR        (0xD8)  /*!< Instruction: Erase Sector */
#define     CMD_STATUS_READ         (0x05)  /*!< Instruction: Read Status Register */
#define     CMD_CONFIG_READ         (0x35)  /*!< Instruction: Read Configraion Register */
#define     CMD_STATUS_WRITE        (0x01)  /*!< Instruction: Write Status Register */
#define     CMD_WRITE_ENABLE        (0x06)  /*!< Instruction: Write Enable */
#define     CMD_WRITE_DISABLE       (0x04)  /*!< Instruction: Write Disable */
#define     CMD_READ_ID             (0x9F)  /*!< Instruction: Read ID */
#define     CMD_BANK_READ           (0x16)  /*!< Instruction: Bank Register Read */
#define     CMD_BANK_WRITE          (0x17)  /*!< Instruction: Bank Register Write */

#define    CMD_WRITE_PAGE_LEN       4       /*!< Write Page Length */
#define    CMD_ERASE_CHIP_LEN       1       /*!< Erase Chip Length */
#define    CMD_ERASE_SECTOR_LEN     4       /*!< Erase Sector Length */
#define    CMD_WRITE_ENABLE_LEN     1       /*!< Write Enable Legth */
#define    CMD_WRITE_DISABLE_LEN    1       /*!< Write Disable Length */
#define    CMD_READ_ID_LEN          4       /*!< Read ID Length */
#define    CMD_STATUS_WRITE_LEN     3       /*!< Staus Write Length */
#define    CMD_BANK_READ_LEN        2       /*!< Bank Register Read Length */
#define    CMD_BANK_WRITE_LEN       2       /*!< Bank Register Write Length */

#define    SF_PBufEn   (0x10U)              /*!< Primary Buffer Enable Flag */
#define    SF_SBufEn   (0x20U)              /*!< Secondary Buffer Enable Flag */
#define    SF_CSNum_CS0 (0x00U)              /*!< CSNum SMIxCS0_N */
#define    SF_CSNum_CS1 (0x02U)              /*!< CSNum SMIxCS1_N */
#define    SF_CycGo    (0x01U)              /*!< Go Flag */

#define    SF_CycProg  (0x02U)              /*!< Program Flag */
#define    SF_CycDone  (0x01U)              /*!< Done Flag */

#define    STATUS_BUSY_BIT        0x01U     /* Staus[0] : 0=ready, 1=busy */
#define    STATUS_QUAD_BIT        0x0200U   /* Congfig[1] : 0=Single/dual, 1=quad */
/** 
 *  @}
 */ /* End of group SMIF_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_typedef
 *  @{
 */

typedef struct
{
    int                 sf_state;                           /*!< state */
    uint32_t            busy_check;   /*!< busy check on/off */
    uint32_t            busy_count;                     /*!< busy check counter */
    uint32_t            busy_polling;    /*!< busy check polling time */
    int32_t             busy_state;      /*!< busy check state */
} busy_setting_t;

/** 
 *  @}
 */ /* End of group SMIF_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_variables
 *  @{
 */

static  TSB_SMI_TypeDef     *p_reg;                             /*!< Registers base address.           */
static  busy_setting_t      busy_ck[SMIF_MAX_CHANNEL+1];
/**
 *  @}
 */ /* End of group SMIF_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Const Table                                                                 */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_const
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group SMIF_Private_const */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Private_functions
 *  @{
 */
static int sflash_WaitTransComp(void);
static int sflash_ReadStatus(uint16_t *pData, int statusLen, uint8_t ch);
static int sflash_IsDeviceBusy(uint8_t ch);
static int sflash_WriteEnable(uint8_t ch);
static int sflash_WriteDisable(uint8_t ch);
static int sflash_WaitBusyOff(uint32_t pollingTime, uint32_t timeout, uint8_t ch);
static int sflash_quad_enable(uint8_t ch);
static int sflash_quad_disable(uint8_t ch);

#ifdef DEBUG
__STATIC_INLINE int32_t check_param_bank(uint8_t param);
__STATIC_INLINE int32_t check_param_map_base(uint32_t param);
__STATIC_INLINE int32_t check_param_map_enable(uint32_t param);
__STATIC_INLINE int32_t check_param_capacity(uint32_t param);
__STATIC_INLINE int32_t check_param_trans_ckock(uint32_t param);
__STATIC_INLINE int32_t check_param_deasert_time(uint32_t param);
__STATIC_INLINE int32_t check_param_poll_wip(uint32_t param);
__STATIC_INLINE int32_t check_param_cmd_op(uint32_t param);
#endif
/*--------------------------------------------------*/
/** 
  * @brief  Wait Command Transfer Complete.
  * @param  -
  * @return result
  * @retval SMIF_E_SFLASH_OK
  * @retval SMIF_E_SFLASH_TIMEOUT
  * @note   -
  */
/*--------------------------------------------------*/
static int sflash_WaitTransComp(void)
{
#ifndef TEST_DEBUG
    uint32_t count;

    count = SFLASH_INT_TIMEOUT_COUNT;
    while ((p_reg->STAT & SF_CycDone) == 0)
    {
        count--;
        if (count == 0)
        {
            p_reg->STAT &= ~SF_CycDone;
            return SMIF_E_SFLASH_TIMEOUT;
        }
    }
    p_reg->STAT &= ~SF_CycDone;
#endif
    return SMIF_E_SFLASH_OK;
}


/*--------------------------------------------------*/
/** 
  * @brief  Read Status Command.
  * @param  pData
  * @param  statusLen
  * @param  ch
  * @return result
  * @retval SMIF_E_SFLASH_OK
  * @retval SMIF_E_SFLASH_TIMEOUT
  * @note   -
  */
/*--------------------------------------------------*/
static int sflash_ReadStatus(uint16_t *pData, int statusLen, uint8_t ch)
{
    int ret;
    uint32_t        cs_num;
    
    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return SMIF_E_SFLASH_ERROR;
    }

    p_reg->PBUF0 = (0xFFUL << 8) + CMD_STATUS_READ;
    p_reg->RACR1 = (1UL << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;
    ret = sflash_WaitTransComp();
    if (ret == SMIF_E_SFLASH_OK)
    {
#ifndef TEST_DEBUG
        *pData = (uint16_t)((uint8_t)(p_reg->PBUF0 >> 8));
#else
        *pData = 0x00;
#endif
    }
    else
    {
        return ret;
    }

    if (statusLen == 2)
    {
        p_reg->PBUF0 = (0xFFUL << 8) + CMD_CONFIG_READ;
        p_reg->RACR1 = (1UL << 16) + SF_PBufEn + cs_num;
        p_reg->RACR1 |= SF_CycGo;
        ret = sflash_WaitTransComp();
        if (ret == SMIF_E_SFLASH_OK)
        {
    #ifndef TEST_DEBUG
            *pData = *pData | (uint16_t)(p_reg->PBUF0 & 0xff00);
    #else
            *pData = 0x00;
    #endif
        } else {
            return ret;
        }
    }
    return SMIF_E_SFLASH_OK;
}

/*--------------------------------------------------*/
/** 
  * @brief  Is Device Busy.
  * @param  ch
  * @return result
  * @retval SMIF_E_SFLASH_OK
  * @retval SMIF_E_SFLASH_TIMEOUT
  * @note   -
  */
/*--------------------------------------------------*/
static int sflash_IsDeviceBusy(uint8_t ch)
{
    uint16_t status;
    int ret;

    ret = sflash_ReadStatus(&status, 1, ch);
    if (ret == SMIF_E_SFLASH_OK)
    {
        if ((status & STATUS_BUSY_BIT) == STATUS_BUSY_BIT)
        {
            return DEVICE_BUSY;
        }
        else
        {
            return DEVICE_IDLE;
        }
    }
    else
    {
        return DEVICE_ERR;
    }
}


/*--------------------------------------------------*/
/** 
  * @brief  Write Enable Command
  * @param  ch
  * @return result
  * @retval SMIF_E_SFLASH_OK
  * @retval SMIF_E_SFLASH_TIMEOUT
  * @note   -
  */
/*--------------------------------------------------*/
static int sflash_WriteEnable(uint8_t ch)
{
    int ret;
    uint32_t        cs_num;

    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return SMIF_E_SFLASH_ERROR;
    }
    p_reg->PBUF0 = CMD_WRITE_ENABLE;
    p_reg->RACR1 = ((CMD_WRITE_ENABLE_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;

    ret = sflash_WaitTransComp();
    return ret;
}


/*--------------------------------------------------*/
/** 
  * @brief  Wait Disable Command
  * @param  ch
  * @return result
  * @retval SMIF_E_SFLASH_OK
  * @retval SMIF_E_SFLASH_TIMEOUT
  * @note   -
  */
/*--------------------------------------------------*/
static int sflash_WriteDisable(uint8_t ch)
{
    int ret;
    uint32_t        cs_num;

    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return SMIF_E_SFLASH_ERROR;
    }
    p_reg->PBUF0 = CMD_WRITE_DISABLE;
    p_reg->RACR1 = ((CMD_WRITE_DISABLE_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;

    ret = sflash_WaitTransComp();
    return ret;
}

/*--------------------------------------------------*/
/** 
  * @brief  sflash_WaitBusyOff.
  * @param  pollingTime :(us)
  * @param  timeout :(ms)
  * @param  ch
  * @return Busy_Check_Result
  * @retval SMIF_E_SFLASH_OK
  * @retval SMIF_E_SFLASH_BUSY
  * @retval SMIF_E_SFLASH_ABORT
  * @retval SMIF_E_SFLASH_TIMEOUT
  * @note   -
  */
/*--------------------------------------------------*/
static int sflash_WaitBusyOff(uint32_t pollingTime, uint32_t timeout, uint8_t ch)
{
    int retVal = SMIF_E_SFLASH_OK;
    uint32_t baseTime = SysOpe_GetTime();

    if(ch > SMIF_MAX_CHANNEL)
    {
        retVal = SMIF_E_SFLASH_ERROR;
        return retVal;
    }
    if(busy_ck[ch].sf_state != STATE_CMD_BUSY )
    {
        retVal = SMIF_E_SFLASH_ABORT;
        return retVal;
    }
    if(pollingTime == SMIF_SFLASH_POLLING_NO_TIME)
    {
        uint32_t time;

        for (;;)
        {
            if (sflash_IsDeviceBusy(ch) == DEVICE_IDLE)
            {
                retVal = SMIF_E_SFLASH_OK;
                break;
            }
            time = SysOpe_GetElapseTime(baseTime);
            if ((time * SMIF_TIMER_INTERVAL)  >= timeout)
            {
                retVal = SMIF_E_SFLASH_TIMEOUT;
                break;
            }
        }
    }
    else
    {
        uint32_t time;
        
        busy_ck[ch].busy_check = SMIF_BUSY_CHECK_OFF;
        busy_ck[ch].busy_count = 0;
        busy_ck[ch].busy_polling = pollingTime / SMIF_TIMER_INTERVAL;
        busy_ck[ch].busy_state = SMIF_E_SFLASH_BUSY;
        busy_ck[ch].busy_check = SMIF_BUSY_CHECK_ON;
        
        for (;;)
        {
            if(busy_ck[ch].busy_state !=SMIF_E_SFLASH_BUSY)
            {
                busy_ck[ch].busy_check = SMIF_BUSY_CHECK_OFF;
                retVal = busy_ck[ch].busy_state;
                break;
            }
            time = SysOpe_GetElapseTime(baseTime);
            if ( (time * SMIF_TIMER_INTERVAL / 1000) >= timeout)
            {
                busy_ck[ch].busy_check = SMIF_BUSY_CHECK_OFF;
                retVal = SMIF_E_SFLASH_TIMEOUT;
                break;
            }
        }
    }
    return retVal;
}
/*--------------------------------------------------*/
/** 
  * @brief  Quad Enable.
  * @param  ch
  * @return result
  * @retval SMIF_E_SFLASH_OK
  * @retval SMIF_E_SFLASH_BUSY
  * @retval SMIF_E_SFLASH_ABORT
  * @retval SMIF_E_SFLASH_TIMEOUT
  * @retval SMIF_E_SFLASH_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
static int sflash_quad_enable(uint8_t ch)
{
    uint16_t status = 0;
    int ret = SMIF_E_SFLASH_OK;
    uint32_t        cs_num;

    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return SMIF_E_SFLASH_ERROR;
    }
    busy_ck[ch].sf_state = STATE_CMD_BUSY;
    ret = sflash_ReadStatus(&status, 2, ch);
    if (ret == SMIF_E_SFLASH_OK)
    {
        if ((status & STATUS_BUSY_BIT) == STATUS_BUSY_BIT)
        {
            busy_ck[ch].sf_state = STATE_IDLE;
            ret = SMIF_E_SFLASH_BUSY;
            return(ret);
        }
    }
    else
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        return(ret);
    }
    
    status |= STATUS_QUAD_BIT;
    ret = sflash_WriteEnable(ch);
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        return(ret);
    }
    p_reg->PBUF0 = ((uint32_t)status << 8) + CMD_STATUS_WRITE;
    p_reg->RACR1 = ((CMD_STATUS_WRITE_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;

    ret = sflash_WaitTransComp();
    if (ret == SMIF_E_SFLASH_OK)
    {
        ret = sflash_WaitBusyOff(SMIF_SFLASH_POLLING_TIME_QUAD, SMIF_SFLASH_TIMEOUT_QUAD, ch);
        if (ret != SMIF_E_SFLASH_OK)
        {
            busy_ck[ch].sf_state = STATE_IDLE;
            return(ret);
        }
        ret = sflash_ReadStatus(&status, 2, ch);
        if (ret == SMIF_E_SFLASH_OK)
        {
        #ifndef TEST_DEBUG
            if ((status & STATUS_QUAD_BIT) == STATUS_QUAD_BIT)
        #else
            if (1)
        #endif
            {
                ret = SMIF_E_SFLASH_OK;
            }
            else
            {
                ret = SMIF_E_SFLASH_ERROR;
            }
        }
    }
    busy_ck[ch].sf_state = STATE_IDLE;
    return(ret);
}

/*--------------------------------------------------*/
/** 
  * @brief  Quad Disable.
  * @param  ch
  * @return result
  * @retval SMIF_E_SFLASH_OK
  * @retval SMIF_E_SFLASH_BUSY
  * @retval SMIF_E_SFLASH_ABORT
  * @retval SMIF_E_SFLASH_TIMEOUT
  * @retval SMIF_E_SFLASH_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
static int sflash_quad_disable(uint8_t ch)
{
    uint16_t status = 0;
    int ret = SMIF_E_SFLASH_OK;
    uint32_t        cs_num;

    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return SMIF_E_SFLASH_ERROR;
    }
    busy_ck[ch].sf_state = STATE_CMD_BUSY;
    ret = sflash_ReadStatus(&status, 2, ch);
    if (ret == SMIF_E_SFLASH_OK)
    {
        if ((status & STATUS_BUSY_BIT) == STATUS_BUSY_BIT)
        {
            busy_ck[ch].sf_state = STATE_IDLE;
            ret = SMIF_E_SFLASH_BUSY;
            return(ret);
        }
    }
    else
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        return(ret);
    }
    
    status &= ~STATUS_QUAD_BIT;
    ret = sflash_WriteEnable(ch);
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        return(ret);
    }
    p_reg->PBUF0 = ((uint32_t)status << 8) + CMD_STATUS_WRITE;
    p_reg->RACR1 = ((CMD_STATUS_WRITE_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;

    ret = sflash_WaitTransComp();
    if (ret == SMIF_E_SFLASH_OK)
    {
        ret = sflash_WaitBusyOff(SMIF_SFLASH_POLLING_TIME_QUAD, SMIF_SFLASH_TIMEOUT_QUAD, ch);
        if (ret != SMIF_E_SFLASH_OK)
        {
            busy_ck[ch].sf_state = STATE_IDLE;
            return(ret);
        }
        ret = sflash_ReadStatus(&status, 2, ch);
        if (ret == SMIF_E_SFLASH_OK)
        {
            if ((status & STATUS_QUAD_BIT) == 0)
            {
                ret = SMIF_E_SFLASH_OK;
            }
            else
            {
                ret = SMIF_E_SFLASH_ERROR;
            }
        }
    }
    busy_ck[ch].sf_state = STATE_IDLE;
    return(ret);
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Check the bank parameter.
  * @param  param    :bank parameter
  * @retval SMIF_PARAM_OK :Valid
  * @retval SMIF_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_bank(uint8_t param)
{
    int32_t result = SMIF_PARAM_NG;
    
    switch (param) 
    {
    case 0:
    case 1:
    case 2:
    case 3:
        result = SMIF_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the map basec parameter.
  * @param  param    :map base parameter
  * @retval SMIF_PARAM_OK :Valid
  * @retval SMIF_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_map_base(uint32_t param)
{
    int32_t result = SMIF_PARAM_NG;
    
    if(param <= SMIF_MAP_BASE_MAX)
    {
        result = SMIF_PARAM_OK;
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the capacity parameter.
  * @param  param    :capacity parameter
  * @retval SMIF_PARAM_OK :Valid
  * @retval SMIF_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_capacity(uint32_t param)
{
    int32_t result = SMIF_PARAM_NG;
    
    switch (param) 
    {
    case SMIF_FDEN_64KB:
    case SMIF_FDEN_128KB:
    case SMIF_FDEN_256KB:
    case SMIF_FDEN_512KB:
    case SMIF_FDEN_1MB:
    case SMIF_FDEN_2MB:
    case SMIF_FDEN_4MB:
    case SMIF_FDEN_8MB:
    case SMIF_FDEN_16MB:
        result = SMIF_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the map enable parameter.
  * @param  param    :map enable parameter
  * @retval SMIF_PARAM_OK :Valid
  * @retval SMIF_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_map_enable(uint32_t param)
{
    int32_t result = SMIF_PARAM_NG;
    
    if(param <= SMIF_MAX_CHANNEL)
    {
        result = SMIF_PARAM_OK;
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the trans clock.
  * @param  param    :trans clock parameter
  * @retval SMIF_PARAM_OK :Valid
  * @retval SMIF_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_trans_ckock(uint32_t param)
{
    int32_t result = SMIF_PARAM_NG;
    
    SystemCoreClockUpdate();
    if((SystemCoreClock > 140000000) && (SystemCoreClock <= 160000000))
    {
        if((param >= 7) && (param <= SMIF_TRANS_CLOCK_MAX))
        {
            result = SMIF_PARAM_OK;
        }
    }
    else if((SystemCoreClock > 120000000) && (SystemCoreClock <= 140000000))
    {
        if((param >= 6) && (param <= SMIF_TRANS_CLOCK_MAX))
        {
            result = SMIF_PARAM_OK;
        }
    }
    else if((SystemCoreClock > 100000000) && (SystemCoreClock <= 120000000))
    {
        if((param >= 5) && (param <= SMIF_TRANS_CLOCK_MAX))
        {
            result = SMIF_PARAM_OK;
        }
    }
    else if((SystemCoreClock >  80000000) && (SystemCoreClock <= 100000000))
    {
        if((param >= 4) && (param <= SMIF_TRANS_CLOCK_MAX))
        {
            result = SMIF_PARAM_OK;
        }
    }
    else if((SystemCoreClock >  60000000) && (SystemCoreClock <=  8000000))
    {
        if((param >= 3) && (param <= SMIF_TRANS_CLOCK_MAX))
        {
            result = SMIF_PARAM_OK;
        }
    }
    else if((SystemCoreClock >  40000000) && (SystemCoreClock <=  6000000))
    {
        if((param >= 2) && (param <= SMIF_TRANS_CLOCK_MAX))
        {
            result = SMIF_PARAM_OK;
        }
    }
    else if(SystemCoreClock <=  40000000)
    {
        if((param >= SMIF_TRANS_CLOCK_MIN) && (param <= SMIF_TRANS_CLOCK_MAX))
        {
            result = SMIF_PARAM_OK;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the deasert time parameter.
  * @param  param    :deasert time parameter
  * @retval SMIF_PARAM_OK :Valid
  * @retval SMIF_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_deasert_time(uint32_t param)
{
    int32_t result = SMIF_PARAM_NG;
    
    if(param<=SMIF_DEASERT_TIME_MAX)
    {
        result = SMIF_PARAM_OK;
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the WIP Polling parameter.
  * @param  param    :WIP Polling parameter
  * @retval SMIF_PARAM_OK :Valid
  * @retval SMIF_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_poll_wip(uint32_t param)
{
    int32_t result = SMIF_PARAM_NG;
    
    switch (param) 
    {
    case SMIF_WIP_POLLING_DISABLE:
    case SMIF_WIP_POLLING_ENABLE:
        result = SMIF_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Direct Command OpCode parameter.
  * @param  param    :Direct Command OpCode parameter
  * @retval SMIF_PARAM_OK :Valid
  * @retval SMIF_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_cmd_op(uint32_t param)
{
    int32_t result = SMIF_PARAM_NG;
    
    switch (param) 
    {
    case SMIF_CMD_FAST_READ:
    case SMIF_CMD_DUAL_OUTPUT:
    case SMIF_CMD_DUAL_I_O:
    case SMIF_CMD_QUAD_OUTPUT:
    case SMIF_CMD_QUAD_I_O:
        result = SMIF_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

#endif
/**
 *  @}
 */ /* End of group SMIF_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Initialize.
  * @param  p_obj :instance
  * @return result
  * @retval TXZ_SUCCESS
  * @retval TXZ_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_init(SMIF_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    int ret = SMIF_E_SFLASH_OK;
    
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_map_base(p_obj->init.map[0].base));
    assert_param(check_param_capacity(p_obj->init.map[0].capacity));
    assert_param(check_param_map_enable(p_obj->init.map[0].enable));
    assert_param(check_param_map_base(p_obj->init.map[1].base));
    assert_param(check_param_capacity(p_obj->init.map[1].capacity));
    assert_param(check_param_map_enable(p_obj->init.map[1].enable));
    assert_param(check_param_trans_ckock(p_obj->init.clock.trans_clock));
    assert_param(check_param_deasert_time(p_obj->init.clock.deasert_time));
    assert_param(check_param_poll_wip(p_obj->init.clock.poll_wip));
    assert_param(check_param_cmd_op(p_obj->init.direct[0].CmdOP));
    assert_param(check_param_cmd_op(p_obj->init.direct[1].CmdOP));
    
#endif /* #ifdef DEBUG */
    
    busy_ck[0].busy_check = SMIF_BUSY_CHECK_OFF;
    busy_ck[0].busy_count = 0;
    busy_ck[0].busy_polling = SMIF_SFLASH_POLLING_TIME_ERASE / SMIF_TIMER_INTERVAL;
    busy_ck[0].busy_state = SMIF_E_SFLASH_OK;
    
    busy_ck[1].busy_check = SMIF_BUSY_CHECK_OFF;
    busy_ck[1].busy_count = 0;
    busy_ck[1].busy_polling = SMIF_SFLASH_POLLING_TIME_ERASE / SMIF_TIMER_INTERVAL;
    busy_ck[1].busy_state = SMIF_E_SFLASH_OK;
    
    result = smif_timer_initialize();
    if(result != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    result = smif_timer_start();
    if(result != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    
    p_reg = p_obj->p_instance;
    p_reg->MAP0 = (p_obj->init.map[0].base & 0x00FF0000) | (p_obj->init.map[0].capacity << 2) | p_obj->init.map[0].enable;
    p_reg->MAP1 = (p_obj->init.map[1].base & 0x00FF0000) | (p_obj->init.map[1].capacity << 2) | p_obj->init.map[1].enable;
    p_reg->DACR0 = (p_obj->init.clock.trans_clock << 16) | (p_obj->init.clock.deasert_time << 8) | (p_obj->init.clock.poll_wip << 6) | 0x00000010;
    p_reg->DACR1 = (p_obj->init.clock.trans_clock << 16) | (p_obj->init.clock.deasert_time << 8) | (p_obj->init.clock.poll_wip << 6) | 0x00000010;
    p_reg->RACR0 = (p_obj->init.clock.trans_clock << 16) | (p_obj->init.clock.deasert_time << 8) | 0x00000010;
    if(p_obj->init.map[0].enable == SMIF_MAP_ENABLE)
    {
        if(p_obj->init.direct[0].CmdOP == SMIF_CMD_FAST_READ)
        {
            ret = sflash_quad_disable(0);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR0 = 0x0B001000;
        }
        else if(p_obj->init.direct[0].CmdOP == SMIF_CMD_DUAL_OUTPUT)
        {
            ret = sflash_quad_disable(0);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR0 = 0x3B001040;
        }
        else if(p_obj->init.direct[0].CmdOP == SMIF_CMD_DUAL_I_O)
        {
            ret = sflash_quad_disable(0);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR0 = 0xBB001054;
        }
        else if(p_obj->init.direct[0].CmdOP == SMIF_CMD_QUAD_OUTPUT)
        {
            ret = sflash_quad_enable(0);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR0 = 0x6B001080;
        }
        else if(p_obj->init.direct[0].CmdOP == SMIF_CMD_QUAD_I_O)
        {
            ret = sflash_quad_enable(0);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR0 = 0xEB0030A8;
        }
    }
    if(p_obj->init.map[1].enable == SMIF_MAP_ENABLE)
    {
        if(p_obj->init.direct[1].CmdOP == SMIF_CMD_FAST_READ)
        {
            ret = sflash_quad_disable(1);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR1 = 0x0B001000;
        }
        else if(p_obj->init.direct[1].CmdOP == SMIF_CMD_DUAL_OUTPUT)
        {
            ret = sflash_quad_disable(1);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR1 = 0x3B001040;
        }
        else if(p_obj->init.direct[1].CmdOP == SMIF_CMD_DUAL_I_O)
        {
            ret = sflash_quad_disable(1);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR1 = 0xBB001054;
        }
        else if(p_obj->init.direct[1].CmdOP == SMIF_CMD_QUAD_OUTPUT)
        {
            ret = sflash_quad_enable(1);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR1 = 0x6B001080;
        }
        else if(p_obj->init.direct[1].CmdOP == SMIF_CMD_QUAD_I_O)
        {
            ret = sflash_quad_enable(1);
            if(ret != SMIF_E_SFLASH_OK)
            {
                result = TXZ_ERROR;
                return(result);
            }
            p_reg->DRCR1 = 0xEB0030A8;
        }
    }
    return(result);
}
/*--------------------------------------------------*/
/** 
  * @brief  finalize
  * @param  -
  * @return result
  * @retval TXZ_SUCCESS
  * @retval TXZ_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_finalize(void)
{
    TXZ_Result result = TXZ_ERROR;

    if(sflash_WriteDisable(0) != SMIF_E_SFLASH_OK)
    {
        /* no process */
    }
    if(sflash_WriteDisable(1) != SMIF_E_SFLASH_OK)
    { 
        /* no process */
    }
    result = smif_timer_stop();
    if(result != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    result = smif_timer_finalize();
    if(result != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    result = TXZ_SUCCESS;
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  erase chip
  * @param  ch
  * @return result
  * @retval TXZ_SUCCESS
  * @retval TXZ_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_erase_chip(uint8_t ch)
{
    TXZ_Result result = TXZ_ERROR;
    int ret = 0;
    uint32_t        cs_num;
    
    
    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return(result);
    }
    busy_ck[ch].sf_state = STATE_CMD_BUSY;
    
    if (sflash_IsDeviceBusy(ch) != DEVICE_IDLE)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }
    
    ret = sflash_WriteEnable(ch);
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }
    
    p_reg->PBUF0 = CMD_ERASE_CHIP;
    p_reg->RACR1 = ((CMD_ERASE_CHIP_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;
    
    ret = sflash_WaitTransComp();
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }
    
    ret = sflash_WaitBusyOff(SMIF_SFLASH_POLLING_TIME_ERASE, SMIF_SFLASH_TIMEOUT_CHIP_ERASE, ch);
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }
    busy_ck[ch].sf_state = STATE_IDLE;
    result = TXZ_SUCCESS;

EXIT:
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  erase sector
  * @param  p_address
  * @param  ch
  * @return result
  * @retval TXZ_SUCCESS
  * @retval TXZ_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_erase_sector(uint8_t *p_address, uint8_t ch)
{
    TXZ_Result result = TXZ_ERROR;
    int ret = 0;
    uint32_t addr = (uint32_t)(p_address) & 0x03FFFFFF;
    uint32_t        cs_num;
    
    
    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return(result);
    }
    busy_ck[ch].sf_state = STATE_CMD_BUSY;

    if (sflash_IsDeviceBusy(ch) != DEVICE_IDLE)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }

    ret = sflash_WriteEnable(ch);
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }

    p_reg->PBUF0 = ((addr << 24) & 0xff000000) + ((addr << 8) & 0x00ff0000) + ((addr >> 8) & 0x0000ff00) + CMD_ERASE_SECTOR;
    p_reg->RACR1 = ((CMD_ERASE_SECTOR_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;

    ret = sflash_WaitTransComp();
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }

    ret = sflash_WaitBusyOff(SMIF_SFLASH_POLLING_TIME_ERASE, SMIF_SFLASH_TIMEOUT_ERASE, ch);
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }
    busy_ck[ch].sf_state = STATE_IDLE;
    result = TXZ_SUCCESS;

EXIT:
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Write Data.
  * @param  p_dst: destination
  * @param  p_src: source
  * @param  length
  * @param  ch
  * @return result
  * @retval TXZ_SUCCESS
  * @retval TXZ_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_write_data(uint8_t *p_dst, uint8_t *p_src, uint32_t length, uint8_t ch)
{
    TXZ_Result result = TXZ_ERROR;
    uint32_t i;
    int ret = 0;
    uint32_t writeLen = 0;
    uint32_t count = 0;
    uint32_t temp = 0;
    uint32_t addr = (uint32_t)(p_dst) & 0x03FFFFFF;
    uint32_t        cs_num;
    
    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return(result);
    }
    busy_ck[ch].sf_state = STATE_CMD_BUSY;

    if (sflash_IsDeviceBusy(ch) != DEVICE_IDLE)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }

    writeLen = 0;
    while (length > writeLen)
    {
        ret = sflash_WriteEnable(ch);
        if (ret != 0)
        {
            busy_ck[ch].sf_state = STATE_IDLE;
            goto EXIT;
        }

        temp = addr & (uint32_t)(SFLASH_PAGE_SIZE - 1);
        if (temp == 0)
        {    /* page limit */
            if ((length - writeLen) > SFLASH_PAGE_SIZE)
            {
                count = SFLASH_PAGE_SIZE;
            }
            else
            {
                count = length - writeLen;
            }
        } else {        /* not page limit */
            if ((temp + length) > SFLASH_PAGE_SIZE)
            {
                count = SFLASH_PAGE_SIZE - temp;
            }
            else
            {
                count = length;
            }
        }

        p_reg->PBUF0 = ((addr << 24) & 0xff000000) + ((addr << 8) & 0x00ff0000) + ((addr >> 8) & 0x0000ff00) + CMD_WRITE_PAGE;
        {
            volatile uint8_t *ptr = (volatile uint8_t *)&p_reg->SBUF00;
            for(i=0;i<count;++i)
            {
                *ptr++ = *p_src++;
            }
        }
        p_reg->RACR1 = ((count - 1) << 24) + ((CMD_WRITE_PAGE_LEN - 1) << 16) + SF_SBufEn + SF_PBufEn + cs_num;
        p_reg->RACR1 |= SF_CycGo;

        ret = sflash_WaitTransComp();
        if (ret != SMIF_E_SFLASH_OK)
        {
            busy_ck[ch].sf_state = STATE_IDLE;
            goto EXIT;
        }

        ret = sflash_WaitBusyOff(SMIF_SFLASH_POLLING_TIME_WRITE, SMIF_SFLASH_TIMEOUT_WRITE, ch);
        if (ret != SMIF_E_SFLASH_OK)
        {
            busy_ck[ch].sf_state = STATE_IDLE;
            goto EXIT;
        }
        addr += count;
        writeLen += count;
    }
    busy_ck[ch].sf_state = STATE_IDLE;
    result = TXZ_SUCCESS;

EXIT:
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Read ID.
  * @param  id address.
  * @param  ch
  * @return result
  * @retval TXZ_SUCCESS
  * @retval TXZ_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_sflash_read_id(uint8_t *id, uint8_t ch)
{
    TXZ_Result result = TXZ_ERROR;
    int ret = 0;
    uint32_t        cs_num;

    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return(result);
    }
    p_reg->PBUF0 = CMD_READ_ID;
    p_reg->RACR1 = ((CMD_READ_ID_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;

    ret = sflash_WaitTransComp();
    if (ret == SMIF_E_SFLASH_OK)
    {
        id[0] = (uint8_t)(p_reg->PBUF0 >> 8);
        id[1] = (uint8_t)(p_reg->PBUF0 >> 16);
        id[2] = (uint8_t)(p_reg->PBUF0 >> 24);
        result = TXZ_SUCCESS;
    }
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Busy Check Timer.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void BusyCheckTimer(void)
{
    if(busy_ck[0].busy_check == SMIF_BUSY_CHECK_ON)
    {
        if(++busy_ck[0].busy_count > busy_ck[0].busy_polling)
        {
            busy_ck[0].busy_count = 0;
            if (sflash_IsDeviceBusy(0) == DEVICE_IDLE)
            {
                busy_ck[0].busy_state = SMIF_E_SFLASH_OK;
            }
        }
    }
    if(busy_ck[1].busy_check == SMIF_BUSY_CHECK_ON)
    {
        if(++busy_ck[1].busy_count > busy_ck[1].busy_polling)
        {
            busy_ck[1].busy_count = 0;
            if (sflash_IsDeviceBusy(1) == DEVICE_IDLE)
            {
                busy_ck[1].busy_state = SMIF_E_SFLASH_OK;
            }
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Bank Register Read.
  * @param  bank address.
  * @param  ch
  * @return result
  * @retval TXZ_SUCCESS
  * @retval TXZ_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_bank_read(uint8_t *bank, uint8_t ch)
{
    TXZ_Result result = TXZ_ERROR;
    int ret = 0;
    uint32_t        cs_num;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return(result);
    }
    p_reg->PBUF0 = CMD_BANK_READ;
    p_reg->RACR1 = ((CMD_BANK_READ_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;

    ret = sflash_WaitTransComp();
    if (ret == SMIF_E_SFLASH_OK)
    {
        *bank = (uint8_t)(p_reg->PBUF0 >> 8);
        result = TXZ_SUCCESS;
    }
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Bank register write
  * @param  bank
  * @param  ch
  * @return result
  * @retval TXZ_SUCCESS
  * @retval TXZ_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_bank_write(uint8_t bank, uint8_t ch)
{
    TXZ_Result result = TXZ_ERROR;
    int ret = 0;
    uint32_t        cs_num;
    
    
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(check_param_bank(bank));
#endif /* #ifdef DEBUG */

    if(ch == 0)
    {
        cs_num = SF_CSNum_CS0;
    }
    else if(ch == 1)
    {
        cs_num = SF_CSNum_CS1;
    }
    else{
        return(result);
    }
    busy_ck[ch].sf_state = STATE_CMD_BUSY;

    if (sflash_IsDeviceBusy(ch) != DEVICE_IDLE)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }
    p_reg->PBUF0 = ((uint32_t)bank << 8) + CMD_BANK_WRITE;
    p_reg->RACR1 = ((CMD_BANK_WRITE_LEN - 1) << 16) + SF_PBufEn + cs_num;
    p_reg->RACR1 |= SF_CycGo;

    ret = sflash_WaitTransComp();
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }
    ret = sflash_WaitBusyOff(SMIF_SFLASH_POLLING_NO_TIME, SMIF_SFLASH_TIMEOUT_US, ch);
    if (ret != SMIF_E_SFLASH_OK)
    {
        busy_ck[ch].sf_state = STATE_IDLE;
        goto EXIT;
    }
    busy_ck[ch].sf_state = STATE_IDLE;
    result = TXZ_SUCCESS;

EXIT:
    return(result);
}

/**
 *  @}
 */ /* End of group SMIF_Exported_functions */


/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__SMIF_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

