/** 
 *******************************************************************************
 * @file    txz_smif_timer.c
 * @brief   TIMER
 * @version V1.0.0.0
 * $Date:: 2017-10-25 08:26:38 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_smif_include.h"
#include "txz_cg.h"
#include "txz_t32a.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup SMIF_UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_UTILITIES_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_UTILITIES_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_UTILITIES_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_UTILITIES_Private_define
 *  @{
 */

#define TIMER_NULL        ((void *)0)  /*!< Null Pointer */

/** 
 *  @}
 */ /* End of group SMIF_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_UTILITIES_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_UTILITIES_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group SMIF_UTILITIES_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_UTILITIES_Private_variables
 *  @{
 */

static  t32a_t      t32a_timer;                 /*!< Limit timer :T32A. */
static  uint32_t    free_count = 0;             /*!< free run counter.  */
uint32_t    sec_count = 0;              /*!< 1sec counter.      */

/**
 *  @}
 */ /* End of group SMIF_UTILITIES_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_UTILITIES_Private_fuctions
 *  @{
 */

static void timer_driver_match_handler(uint32_t id, uint32_t status, TXZ_Result result);
/*--------------------------------------------------*/
/** 
  * @brief  Timer Driver Compare Match Detection Handler.
  * @param  id        :User ID
  * @param  status    :Status
  * @param  result    :Result
  * @return -
  * @retval -
  * @note   Call by T32A Driver
  */
/*--------------------------------------------------*/
static void timer_driver_match_handler(uint32_t id, uint32_t status, TXZ_Result result)
{
    if (result == TXZ_SUCCESS)
    {
        /* Check INTA1 */
        if ((status & T32A_INTx1_FLG_MASK) == T32A_INTx1_FLG_MASK)
        {
            ++free_count;
            if((free_count % 10000) == 0)
            {
                ++sec_count;
            }
            BusyCheckTimer();
        }
    }
}

/**
 *  @}
 */ /* End of group SMIF_UTILITIES_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SMIF_UTILITIES_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_timer_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    t32a_t *p_t32a = &t32a_timer;
    t32a_initial_setting_t *p_init = &t32a_timer.init[T32A_TIMERA];
    cg_t paramCG;
    uint32_t    interval = SMIF_TIMER_INTERVAL;   /* Interval Time (us).           */

    p_t32a->p_instance = TSB_T32A5;
    p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
    p_t32a->init_mode.mode.mode = T32A_MODE_16;
    if(t32a_mode_init(p_t32a) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /* CG Driver Register Address Allocation. */
    paramCG.p_instance = TSB_CG;
    /* Calculate timer counter. */
    {
        TXZ_WorkState loopBreak = TXZ_BUSY;
        uint32_t i;
        
        for (i=0; (i < 8) && (loopBreak == TXZ_BUSY); i++)
        {
            if (i == 0)
            {
                p_init->crx.prscl = T32A_PRSCLx_1;
                if (t32a_Calculator(
                        &p_init->rgx1.rgx1, 
                        interval, 
                        cg_get_mphyt0(&paramCG), 
                        p_init->crx.prscl
                        ) == TXZ_SUCCESS)
                {
                    loopBreak = TXZ_DONE;
                }
            }
            else
            {
                p_init->crx.prscl = (uint32_t)(i << 28);
                if (t32a_Calculator(
                        &p_init->rgx1.rgx1, 
                        interval, 
                        cg_get_mphyt0(&paramCG), 
                        p_init->crx.prscl
                        ) == TXZ_SUCCESS)
                {
                    loopBreak = TXZ_DONE;
                }
            }
        }
        if (loopBreak == TXZ_BUSY)
        {
            result = TXZ_ERROR;
            return(result);
        }
    }
    /* Run Control      */
    p_init->runx.sftstp        = T32A_COUNT_STOP;
    p_init->runx.sftsta        = T32A_COUNT_DONT_START;
    p_init->runx.run           = T32A_RUN_DISABLE;
    /* Counter Control  */
    p_init->crx.clk            = T32A_CLKx_PRSCLx;
    p_init->crx.wbf            = T32A_WBF_DISABLE;
    p_init->crx.updn           = T32A_COUNT_UP;
    p_init->crx.reld           = T32A_RELOAD_TREGx;
    p_init->crx.stop           = T32A_STOP_NON;
    p_init->crx.start          = T32A_START_NON;
    /* Output Control 0 */
    p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
    /* Output Control 1 */
    p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
    p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
    p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_DISABLE;
    p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_DISABLE;
    /* Timer Counter 1  */
    p_init->rgx0.rgx0          = 0;
    /* Counter Reload */
    p_init->reldx.reld         = 0;
    /* Counter Capture  */
    p_init->tmrx.tmrx          = 0;
    /* Counter Capture Control A0 */
    p_init->capx0.capx0        = 0;
    /* Counter Capture Control A1 */
    p_init->capx1.capx1        = 0;
    /* Interrupt mask register Set */
    p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
    p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
    p_init->imx.imx1           = T32A_IMx1_MASK_NOREQ;
    p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
    /* DMA Request      */
    p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
    p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
    p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
    /* User ID          */
    p_init->id                 = 0;     /* No Use */
    /* Handler          */
    p_init->handler_T = timer_driver_match_handler;
    if (t32a_timer_init(p_t32a, T32A_TIMERA) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    return(result);
}

TXZ_Result smif_timer_finalize(void)
{
    t32a_t *p_t32a = &t32a_timer;
    
    TXZ_Result result = TXZ_SUCCESS;
    if (t32a_deinit(p_t32a, T32A_TIMERA) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Start.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_timer_start(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    t32a_t *p_t32a = &t32a_timer;

    /*--- Start       ---*/
    if (t32a_timer_startIT(p_t32a, T32A_TIMERA) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    if (t32a_SWcounter_start(p_t32a, T32A_TIMERA) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    /*--- NVIC Enable ---*/
    NVIC_EnableIRQ(INTT32A05_A_CT_IRQn);
    return(result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Stop.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result smif_timer_stop(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    t32a_t *p_t32a = &t32a_timer;

    /*--- NVIC Disable ---*/
    NVIC_DisableIRQ(INTT32A05_A_CT_IRQn);
    /*--- Stop         ---*/
    if (t32a_timer_stopIT(p_t32a, T32A_TIMERA) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    if (t32a_SWcounter_stop(p_t32a, T32A_TIMERA) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get base time.
  * @param  -
  * @retval now time.
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t SysOpe_GetTime(void)
{
    return(free_count);
}
/*--------------------------------------------------*/
/** 
  * @brief  Get Elapse time.
  * @param  baseTime
  * @retval elapsed time.
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t SysOpe_GetElapseTime(uint32_t baseTime)
{
    return(free_count - baseTime);
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ Handler.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void smif_timer_handle(void)
{
    t32a_t *p_t32a = &t32a_timer;

    t32a_timer_IRQHandler(p_t32a);
}

/**
 *  @}
 */ /* End of group SMIF_UTILITIES_Exported_functions */


/**
 *  @}
 */ /* End of group SMIF_UTILITIES */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
