/** 
 *******************************************************************************
 * @file    bsp_adc.c
 * @brief   This file provides API functions for BSP ADC driver.
 * @version V1.0.0.0
 * $Date:: 2018-02-27 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */

#include "bsp_adc.h"
#include "txz_cg.h"
#include "txz_adc.h"

#if defined(__BSP_ADC_H)

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Private_macro */
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_functions
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group  UTILITIES_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_define
 *  @{
 */

/**
 *  @name  ADxCLK Macro Definition.
 *  @brief ADxCLK Register Macro Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol |
 *  | 31:12 | :---       |
 *  | 11:8  | EXAZ1      |
 *  | 7     | -          |
 *  | 6:3   | EXAZ0      |
 *  | 2:0   | VACLK      |
 *  @{
 */
/* VACLK[2:0] */
#define ADC_ADCLK_1_DIV	 ((uint32_t)0x00000001)  /*'< ADC SCLK ADCLK division value */
#define ADC_ADCLK_2_DIV	 ((uint32_t)0x00000002)  /*'< ADC SCLK ADCLK/2 division value */
#define ADC_ADCLK_4_DIV	 ((uint32_t)0x00000004)  /*'< ADC SCLK ADCLK/4 division value */
#define ADC_ADCLK_8_DIV	 ((uint32_t)0x00000008)  /*'< ADC SCLK ADCLK/8 division value */
#define ADC_ADCLK_16_DIV ((uint32_t)0x00000010)  /*'< ADC SCLK ADCLK/16 division value */
/**
 *  @}
 */ /* End of name ADxCLK Macro Definition */


/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_mode1_setting(void)
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @pre       Conversion has stoped.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
adc_mod1_t adc_get_mode1(adc_t *p_adc)
{
	adc_mod1_t result = ADC_MOD1_SCLK_3;
	uint32_t clock;

    /* System core clock update */
	SystemCoreClockUpdate();
	clock = CoreClockInput;

	switch (p_adc->init.clk.vadcld)
	{
		case ADC_SCLK_1:
			clock /= ADC_ADCLK_1_DIV;
			break;
		case ADC_SCLK_2:
			clock /= ADC_ADCLK_2_DIV;
			break;
		case ADC_SCLK_4:
			clock /=  ADC_ADCLK_4_DIV;
			break;
		case ADC_SCLK_8:
			clock /=  ADC_ADCLK_8_DIV;
			break;
		case ADC_SCLK_16:
			clock /= ADC_ADCLK_16_DIV;
			break;
		default:
			clock = 0;
			break;
	}
	if(clock <= 40000000){
		result = ADC_MOD1_SCLK_1;
	}else if((40000000 < clock) && (clock <= 50000000)){
		result = ADC_MOD1_SCLK_2;
	}else if((50000000 < clock) && (clock <= 60000000)){
		result = ADC_MOD1_SCLK_3;
	}else if((60000000 < clock) && (clock <= 80000000)){
		result = ADC_MOD1_SCLK_4;
	}else{
		/* nothing@*/
	}
	return (result);
}

#endif /* defined(__BSP_ADC_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */
