/** 
 *******************************************************************************
 * @file    bsp_fuart_io_mdma.c
 * @brief   FUART input/output
 * @version V1.0.0.0
 * $Date:: 2017-07-13 10:52:31 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdarg.h>
#include "bsp_fuart_io_mdma.h"

#if defined(__BSP_FUART_IO_MDMA_H)
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup BSP_UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuartion                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_define
 *  @{
 */
/*----------------------*/
/*  for Debug           */
/*----------------------*/
#ifdef DEBUG
    #if 1
        #define FUART_IO_CFG_DEBUG
    #endif
#endif
/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_define
 *  @{
 */

#define FUART_IO_NULL           ((void *)0)  /*!< Null Pointer */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_define
 *  @{
 */
/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_typedef
 *  @{
 */
/**
 *  @}
 */ /* End of group BSP_UTILITIES_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_variables
 *  @{
 */
static fuart_io_mdma_t    instance;   /*!< USB FUART Instance. */
/**
 *  @}
 */ /* End of group BSP_UTILITIES_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_fuctions
 *  @{
 */
static void fuart_io_error(void);
static int put_char(int c);
static int get_char(void);
static void transmit_handler(uint32_t id, TXZ_Result result);
static void receive_handler(uint32_t id, TXZ_Result result, fuart_receive_t *p_info);

/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void fuart_io_error(void)
{
#ifdef FUART_IO_CFG_DEBUG
    while(1)
    {
        __nop();
    }
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  The character is shown to an output stream.
  * @param  c :Character data.
  * @return Output Character data.
  * @retval (value < 0) :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static int put_char(int c)
{
    int ret = -1;
    fuart_io_mdma_private_tx_t *p_tx = &instance.info.tx;

    if (p_tx->state == FUART_IO_MDMA_SEND_IDLE)
    {
        fuart_transmit_t param;
        volatile TXZ_WorkState loopBreak = TXZ_BUSY;
        uint32_t status;

        /*------------------------------*/
        /*  Transmit                    */
        /*------------------------------*/
        p_tx->buff[0] = (uint8_t)c;
        p_tx->state   = FUART_IO_MDMA_SEND_RUN;
        param.p_data    = &p_tx->buff[0];
        param.num       = 1;
        if (fuart_mdma_transmitIt(&instance.info.fuart, &param) != TXZ_SUCCESS)
        {
            fuart_io_error();
        }
        /*------------------------------*/
        /*  Wait to transmit            */
        /*------------------------------*/
        loopBreak = TXZ_BUSY;
        while(loopBreak == TXZ_BUSY)
        {
            switch (p_tx->state)
            {
            case FUART_IO_MDMA_SEND_RUN:
                /* no process */
                break;
            case FUART_IO_MDMA_SEND_SUCCESS:
                if (fuart_get_status(&instance.info.fuart.super, &status) == TXZ_SUCCESS)
                {
                    if ((status & FUARTxFR_BUSY_MASK) != FUARTxFR_BUSY_FLAG_SET)
                    {
                        loopBreak = TXZ_DONE;
                    }
                }
                break;
            default:
                loopBreak = TXZ_DONE;
                break;
            }
        }
        /*------------------------------*/
        /*  Check Error                 */
        /*------------------------------*/
        if (p_tx->state == FUART_IO_MDMA_SEND_FAILURE)
        {
            fuart_io_error();
        }
        /* Clear status. */
        p_tx->state = FUART_IO_MDMA_SEND_IDLE;
        /* Set result. */
        ret = c;
    }

    return (ret);
}

/*--------------------------------------------------*/
/** 
  * @brief  The character is acquired from an input stream.
  * @param  -
  * @return Input Character data.
  * @retval (value < 0) :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static int get_char(void)
{
    volatile int c = -1;
    fuart_io_mdma_private_rx_t *p_rx = &instance.info.rx;

    if (p_rx->state == FUART_IO_MDMA_RECV_IDLE)
    {
        fuart_receive_t param;

        /*------------------------------*/
        /*  Receive                     */
        /*------------------------------*/
        p_rx->buff[0] = 0;
        p_rx->state   = FUART_IO_MDMA_RECV_RUN;
        param.p_data    = &p_rx->buff[0];
        param.num       = 1;
        if (fuart_mdma_receiveIt(&instance.info.fuart, &param) != TXZ_SUCCESS)
        {
            fuart_io_error();
        }
        /*------------------------------*/
        /*  Wait to receive             */
        /*------------------------------*/
        while(p_rx->state == FUART_IO_MDMA_RECV_RUN)
        {
            /* no processing */
        }
        /*------------------------------*/
        /*  Check Error                 */
        /*------------------------------*/
        if (p_rx->state == FUART_IO_MDMA_RECV_FAILURE)
        {
            fuart_io_error();
        }
        /* Clear status. */
        p_rx->state = FUART_IO_MDMA_RECV_IDLE;
        /* Set result. */
        c = p_rx->buff[0];
    }

    return (c);
}


/*--------------------------------------------------*/
/** 
  * @brief      The character is shown to an output stream.
  * @param[in]  s       :Source address.
  * @retval     Suceess :Num of output data(bytes).
  * @retval     Failure :-1
  * @pre        Have finished termination of a designated character string by NULL character.
  * @note       -
  */
/*--------------------------------------------------*/
int put_s(const char *s)
{
    int ret = -1;
    fuart_io_mdma_private_tx_t *p_tx = &instance.info.tx;

    if (p_tx->state == FUART_IO_MDMA_SEND_IDLE)
    {
        volatile TXZ_WorkState loopBreak = TXZ_BUSY;
        uint32_t wp = 0;

        /*------------------------------*/
        /*  Check Data                  */
        /*------------------------------*/
        while (loopBreak == TXZ_BUSY)
        {
            if (wp > (FUART_IO_MDMA_SEND_MAX-1))
            {
                p_tx->buff[wp] = '\0';
                loopBreak = TXZ_DONE;
            }
            else
            {
                if (*s != '\0')
                {
                    p_tx->buff[wp] = (uint8_t)*s;
                    wp++;
                    s++;
                }
                else
                {
                    loopBreak = TXZ_DONE;
                }
            }
        }
        if (wp != 0)
        {
            fuart_transmit_t param;

            /*------------------------------*/
            /*  Transmit                    */
            /*------------------------------*/
            param.p_data    = &p_tx->buff[0];
            param.num       = wp;
            p_tx->state   = FUART_IO_MDMA_SEND_RUN;
            if (fuart_mdma_transmitIt(&instance.info.fuart, &param) != TXZ_SUCCESS)
            {
                fuart_io_error();
            }
            /*------------------------------*/
            /*  Wait to transmit            */
            /*------------------------------*/
            loopBreak = TXZ_BUSY;
            while(loopBreak == TXZ_BUSY)
            {
                switch (p_tx->state)
                {
                case FUART_IO_MDMA_SEND_RUN:
                    /* no process */
                    break;
                case FUART_IO_MDMA_SEND_SUCCESS:
                    {
                        uint32_t status;
                        if (fuart_get_status(&instance.info.fuart.super, &status) != TXZ_SUCCESS)
                        {
                            fuart_io_error();
                        }
                        if ((status & FUARTxFR_BUSY_MASK) != FUARTxFR_BUSY_FLAG_SET)
                        {
                            loopBreak = TXZ_DONE;
                        }
                    }
                    break;
                default:
                    fuart_io_error();
                    break;
                }
            }
            /*------------------------------*/
            /*  Check Error                 */
            /*------------------------------*/
            if (p_tx->state == FUART_IO_MDMA_SEND_FAILURE)
            {
                fuart_io_error();
            }
            /* Clear status. */
            p_tx->state = FUART_IO_MDMA_SEND_IDLE;
            ret = (int)wp;
        }
    }

    return (ret);
}

/*--------------------------------------------------*/
/** 
  * @brief  The character is acquired from an input stream.
  * @param  s
  * @param  num : Number of Input Caracter
  * @return Input Character data.
  * @retval (value < 0) :Failure
  * @note   -
  */
/*--------------------------------------------------*/
int get_s(char *s, uint8_t num)
{
    int ret = 0;
    uint8_t i = 0;
    fuart_io_mdma_private_rx_t *p_rx = &instance.info.rx;
    
    if( num > (FUART_IO_MDMA_RECEIVE_MAX-2)){
        num = FUART_IO_MDMA_RECEIVE_MAX-2;
    }
    /* recive start */
    if (p_rx->state == FUART_IO_MDMA_RECV_IDLE)
    {
        {
            fuart_receive_t param;

            param.p_data    = &p_rx->buff[0];
            param.num       = num;
            p_rx->state   = FUART_IO_MDMA_RECV_RUN;
            if (fuart_mdma_receiveIt(&instance.info.fuart, &param) != TXZ_SUCCESS)
            {
                fuart_io_error();
            }
        }
        /* Message transfer */
        {
            char s_tmp[4] = {0};
            s_tmp[0] = (char)(0x30 + (num / 100) % 10);
            s_tmp[1] = (char)(0x30 + (num / 10) % 10);
            s_tmp[2] = (char)(0x30 + num % 10);
            s_tmp[3] = '\0';
            if (put_s("Please Input ") == -1)
            {
                fuart_io_error();
            }
            if (put_s((const char *)&s_tmp[0]) == -1)
            {
                fuart_io_error();
            }
            if (put_s(" characters = ") == -1)
            {
                fuart_io_error();
            }
        }
        /* wait receive */
        {
            volatile TXZ_WorkState loopBreak = TXZ_BUSY;

            while (loopBreak == TXZ_BUSY)
            {
                switch (p_rx->state)
                {
                case FUART_IO_MDMA_RECV_RUN:
                    /* no process */
                    break;
                case FUART_IO_MDMA_RECV_SUCCESS:
                    loopBreak = TXZ_DONE;
                    break;
                default:
                    fuart_io_error();
                    break;
                }
            }
            p_rx->state = FUART_IO_MDMA_RECV_IDLE;
        }
        /* "Echo" tresfer */
        if (put_s("\n Echo = ") == -1)
        {
            fuart_io_error();
        }
        /* dsta trasnfer */
        p_rx->buff[num] = '\0';
        for(i=0; i<=num; ++i)
        {
            *s++ = (char)p_rx->buff[i];
        }
        if (put_s((const char *)&p_rx->buff[0]) == -1)
        {
            fuart_io_error();
        }
        if (put_s(" \n") == -1)
        {
            fuart_io_error();
        }
    }
    ret = num;
    return(ret);
}

/*--------------------------------------------------*/
/** 
  * @brief  Transmit handler.
  * @param  id      :User ID.
  * @param  result  :Result.
  * @return -
  * @retval -
  * @note   Called by fuart driver.
  */
/*--------------------------------------------------*/
static void transmit_handler(uint32_t id, TXZ_Result result)
{
    fuart_io_mdma_t *p_instance = (fuart_io_mdma_t *)id;

    if (p_instance != FUART_IO_NULL)
    {
        fuart_io_mdma_private_tx_t *p_tx = &p_instance->info.tx;

        if (p_tx->state != FUART_IO_MDMA_SEND_IDLE)
        {
            /*------------------------------*/
            /*  Result Check                */
            /*------------------------------*/
            if (result == TXZ_SUCCESS)
            {
                p_tx->state = FUART_IO_MDMA_SEND_SUCCESS;
            }
            else
            {
                p_tx->state = FUART_IO_MDMA_SEND_FAILURE;
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Receive handler.
  * @param  id      :User ID.
  * @param  result  :Result.
  * @param  p_info  :Received information Source Address.
  * @return -
  * @retval -
  * @note   Called by fuart driver.
  */
/*--------------------------------------------------*/
static void receive_handler(uint32_t id, TXZ_Result result, fuart_receive_t *p_info)
{
    fuart_io_mdma_t *p_instance = (fuart_io_mdma_t *)id;

    if ((p_instance != FUART_IO_NULL) && (p_info != FUART_IO_NULL))
    {
        fuart_io_mdma_private_rx_t *p_rx = &p_instance->info.rx;


        if (p_rx->state == FUART_IO_MDMA_RECV_RUN)
        {
            /*------------------------------*/
            /*  Result Check                */
            /*------------------------------*/
            if (result == TXZ_SUCCESS)
            {
                p_rx->state = FUART_IO_MDMA_RECV_SUCCESS;
            }
            else
            {
                p_rx->state = FUART_IO_MDMA_RECV_FAILURE;
            }
        }
    }
}
/**
 *  @}
 */ /* End of group BSP_UTILITIES_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Initialize
  * @param  p_param :Initial Setting Information Source Address.
  * @return Instance Address.
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
fuart_io_mdma_t *fuart_io_mdma_initialize(fuart_io_mdma_initial_setting_t *p_param)
{
    /*------------------------------*/
    /*  Initialize Variable         */
    /*------------------------------*/
    instance.init.id           = p_param->id;
    instance.init.p_reg        = p_param->p_reg;
    instance.init.boudrate     = p_param->boudrate;
    instance.init.mdma.p_mdma  = p_param->mdma.p_mdma;
    instance.info.tx.state     = FUART_IO_MDMA_SEND_IDLE;
    instance.info.rx.state     = FUART_IO_MDMA_RECV_IDLE;
    /*------------------------------*/
    /*  FUart Setting                */
    /*------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    instance.info.fuart.super.p_instance = instance.init.p_reg;
    /*--- Driver Initialize ---*/
    instance.info.fuart.super.init.id            = (uint32_t)&instance;  /* User ID: Any Value (Example...UART driver instance address). */
    {
        cg_t paramCG;

        paramCG.p_instance = TSB_CG;
        if (fuart_get_boudrate_setting(
                cg_get_mphyt0(&paramCG), 
                instance.init.boudrate, 
                &instance.info.fuart.super.init.boudrate
                ) != TXZ_SUCCESS)
        {
            fuart_io_error();
        }
    }
    /* FUart     */
    instance.info.fuart.super.init.tx_int.inttx  = FUART_TX_INT_DISABLE;
    instance.info.fuart.super.init.tx_int.level  = FUART_FIFO_LEVEL_4;
    instance.info.fuart.super.init.rx_int.intrx  = FUART_RX_INT_DISABLE;
    instance.info.fuart.super.init.rx_int.level  = FUART_FIFO_LEVEL_4;
    instance.info.fuart.super.init.interr        = FUART_OV_ERR_INT_DISABLE | FUART_BK_ERR_INT_DISABLE | FUART_PA_ERR_INT_DISABLE | FUART_FR_ERR_INT_DISABLE;
    instance.info.fuart.super.init.fifo          = FUART_FIFO_DISABLE;
    instance.info.fuart.super.init.ctse          = FUART_CTS_DISABLE;
    instance.info.fuart.super.init.rtse          = FUART_RTS_DISABLE;
    instance.info.fuart.super.init.sblen         = FUART_STOP_BIT_1;
    instance.info.fuart.super.init.even          = FUART_PARITY_BIT_ODD;
    instance.info.fuart.super.init.pe            = FUART_PARITY_DISABLE;
    instance.info.fuart.super.init.sm            = FUART_DATA_LENGTH_8;
    instance.info.fuart.super.init.stpa          = FUART_STATIC_PARITY_DISABLE;
    instance.info.fuart.super.transmit.handler   = transmit_handler;
    instance.info.fuart.super.receive.handler    = receive_handler;
    /* FUart MDMA */
    instance.info.fuart.init.p_obj    = instance.init.mdma.p_mdma;
    instance.info.fuart.init.tx.type  = FUART_MDMA_TYPE_AREA_TO_FIX;
    instance.info.fuart.init.rx.type  = FUART_MDMA_TYPE_FIX_TO_AREA;
    if (fuart_mdma_init(&instance.info.fuart) != TXZ_SUCCESS)
    {
        fuart_io_error();
    }

    return(&instance);
}


/*--------------------------------------------------*/
/** 
  * @brief  Finalize
  * @param  p_instance :Instance Address.
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
void fuart_io_mdma_finalize(fuart_io_mdma_t *p_instance)
{
    /*------------------------------*/
    /*  FUart Finalize               */
    /*------------------------------*/
    /*--- Driver Finalize ---*/
    if (fuart_mdma_deinit(&p_instance->info.fuart) != TXZ_SUCCESS)
    {
        fuart_io_error();
    }
    /*--- Destruct (Register release) ---*/
    p_instance->info.fuart.super.p_instance = FUART_IO_NULL;
}

/*--------------------------------------------------*/
/** 
  * @brief  FUART IRQ Handler.
  * @param  -
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
void fuart_io_irq_handler(void)
{
}

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_functions */


/*------------------------------------------------------------------------------*/
/*  Semihosting Retarget                                                        */
/*------------------------------------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
struct __FILE { int handle; /* Add whatever you need here */ };
FILE __stdout;
FILE __stdin;
int fputc(int ch, FILE *f) {
#elif defined ( __ICCARM__ )    /*IAR Compiler */
int putchar(int ch) {
#endif
    return (put_char(ch));
}

#if defined ( __CC_ARM   )      /* RealView Compiler */
int getc(FILE * f)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
int getchar(void)
#endif
{
    return (get_char());
}

#if defined ( __CC_ARM   )      /* RealView Compiler */
int ferror(FILE *f)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
int ferror(void)
#endif
{
    return (0);
}

/**
 *  @}
 */ /* End of group BSP_UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__BSP_FUART_IO_MDMA_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */
