/** 
 *******************************************************************************
 * @file    timer.h
 * @brief   External reference declaration for Timer.
 * @version V1.0.0.0
 * $Date:: 2017-09-01 08:26:38 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __BSP_TIMER_H
#define __BSP_TIMER_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup BSP_UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/* no define */


/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup BSP_UTILITIES_Exported_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Initial setting. \n
 *         User must be set.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t    id;         /*!< User ID.                      */
    t32a_t      *p_t32a;    /*!< T32A Driver Instance Address. */
    uint32_t    type;       /*!< Type. \n 
                                    Value is @ref t32_type_t.  */
    IRQn_Type   irq;        /*!< Interrupt Number. \n          */
    uint32_t    interval;   /*!< Interval Time (us).           */
    void (*handler)(uint32_t id);   /*!< Notify :Interval Time Passed. */
} timer_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  Private Information.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t reserved;
} timer_private_info_t;

/*----------------------------------*/
/** 
 * @brief  Timer Class.
*/
/*----------------------------------*/
typedef struct
{
    timer_initial_setting_t init;      /*!< Initial setting.     */
    timer_private_info_t    info;      /*!< Private information. */
} timer_t;
/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_functions
 *  @{
 */
void timer_initialize(timer_t *p_instance);
void timer_finalize(timer_t *p_instance);
void timer_start(timer_t *p_instance);
void timer_stop(timer_t *p_instance);
void timer_irq_handler(timer_t *p_instance);


/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_functions */

/**
 *  @}
 */ /* End of group BSP_UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_TIMER_H */

