/** 
 *******************************************************************************
 * @file    bsp_uart_io_mdma.h
 * @brief   External reference declaration for UART input/output
 * @version V1.0.1.0
 * $Date:: 2017-09-01 08:26:38 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __BSP_UART_IO_MDMA_H
#define __BSP_UART_IO_MDMA_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdarg.h>
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup BSP_UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

#define UART_IO_MDMA_SEND_MAX       ((uint32_t)0x100)   /*!< Size of send max. (byte)    */
#define UART_IO_MDMA_RECEIVE_MAX    ((uint32_t)0x100)   /*!< Size of receive max. (byte) */


/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Send state.
*/
/*----------------------------------*/
typedef enum {
    UART_IO_MDMA_SEND_IDLE = 0U,     /*!< Idle                   */
    UART_IO_MDMA_SEND_RUN,           /*!< Transmiting.           */
    UART_IO_MDMA_SEND_SUCCESS,       /*!< Transmission complete. */
    UART_IO_MDMA_SEND_FAILURE        /*!< Transmission error.    */
} UartIOMDmaTxState;

/*----------------------------------*/
/** 
 * @brief  Receive state.
*/
/*----------------------------------*/
typedef enum {
    UART_IO_MDMA_RECV_IDLE = 0U,     /*!< Idle                   */
    UART_IO_MDMA_RECV_RUN,           /*!< Receiving              */
    UART_IO_MDMA_RECV_SUCCESS,       /*!< Received complete.     */
    UART_IO_MDMA_RECV_FAILURE        /*!< Received error.        */
} UartIOMDmaRxState;

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  USB UART Initial Setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t            id;         /*!< User ID.         */
    TSB_UART_TypeDef    *p_reg;     /*!< Register Bass Address.  */
    uint32_t            boudrate;   /*!< Boudrate(bps).   */
    struct
    {
        mdma_t           *p_mdma;   /*!< MDMA driver instance address.  */
    } mdma;                         /*!< MDMA dirver instance. */ 
} uart_io_mdma_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  USB UART Send information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile UartIOMDmaTxState   state;                          /*!< Status.                     */
    uint8_t                      buff[UART_IO_MDMA_SEND_MAX];    /*!< Send data.                  */
} uart_io_mdma_private_tx_t;

/*----------------------------------*/
/** 
 * @brief  USB UART Receive information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile UartIOMDmaRxState   state;  /*!< Status.                     */
    uint8_t                     buff[UART_IO_MDMA_RECEIVE_MAX];    /*!< Receive data.               */
} uart_io_mdma_private_rx_t;

/*----------------------------------*/
/** 
 * @brief  USB UART Private Member.
*/
/*----------------------------------*/
typedef struct
{
    uart_mdma_t                  uart;  /*!< Uart driver instance.       */
    uart_io_mdma_private_tx_t    tx;    /*!< Send information.           */
    uart_io_mdma_private_rx_t    rx;    /*!< Receive information.        */
} uart_io_mdma_private_info_t;

/*----------------------------------*/
/** 
 * @brief  USB UART class.
*/
/*----------------------------------*/
typedef struct
{
    uart_io_mdma_initial_setting_t init; /*!< Initial setting.     */
    uart_io_mdma_private_info_t    info; /*!< Private information. */
} uart_io_mdma_t;

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_functions
 *  @{
 */
uart_io_mdma_t *uart_io_mdma_initialize(uart_io_mdma_initial_setting_t *p_param);
void uart_io_mdma_finalize(uart_io_mdma_t *p_instance);
/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_functions */

/**
 *  @}
 */ /* End of group BSP_UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_UART_IO_MDMA_H */

