/** 
 *******************************************************************************
 * @file    bsp.c
 * @brief   Board Support Package. \n
 *          TMPM4G9 Development Board.
 * @version V1.0.0.9
 * $Date:: 2018-03-01 14:24:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "bsp.h"

#if defined(__BSP_H)
/**
 *  @addtogroup BSP
 *  @{
 */

/** 
 *  @defgroup TMPM4G9_DEVE TMPM4G9 Development Board
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4G9_DEVE_Private_macro TMPM4G9 Development Board Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4G9_DEVE_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4G9_DEVE_Private_define TMPM4G9 Development Board Private Define
 *  @{
 */

/** 
 *  @defgroup ELOSCConfiguration ELOSC Configuration
 *  @brief    ELOSC Configuration.
 *  @{
 */
#define ELOSC_CFG_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us). */
#define ELOSC_CFG_CLOCK         ((uint32_t)(32768))     /*!< Clock(hz).        */
/** 
 *  @}
 */ /* End of group ELOSCConfiguration */

/** 
 *  @}
 */ /* End of group TMPM4G9_DEVE_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4G9_DEVE_Private_define TMPM4G9 Development Board Private Define
 *  @{
 */

#define BSP_NULL        ((void *)0) /*!< Null Pointer */

/** 
 *  @defgroup  CGWUPLCR  CGWUPLCR Register
 *  @brief     LOSC warming up Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:27 | -          |
 *  | 26:12 | WUPTL      |
 *  | 11:8  | WUPTL      |
 *  | 7:2   | -          |
 *  | 1     | WULEF      |
 *  | 0     | WULON      |
 *  @{
 */
/* WUPTL  */
#define CGWUPLCR_WUPTL_HIGH_MASK        ((uint32_t)0x07FFF000)    /*!< WUPTL   :High Bit Mask.  */
/* WULEF  */
#define CGWUPLCR_WULEF_MASK             ((uint32_t)0x00000002)    /*!< WULEF   :Mask.           */
#define CGWUPLCR_WULEF_R_DONE           ((uint32_t)0x00000000)    /*!< WULEF   :[R]  :Done      */
#define CGWUPLCR_WULEF_R_RUNNING        ((uint32_t)0x00000002)    /*!< WULEF   :[R]  :Running   */
/* WULON  */
#define CGWUPLCR_WULON_MASK             ((uint32_t)0x00000001)    /*!< WULON   :Mask.           */
#define CGWUPLCR_WULON_W_ENABLE         ((uint32_t)0x00000001)    /*!< WULON   :[W]  :Enable     */
/** 
 *  @}
 */ /* End of group CGWUPLCR */

/** 
 *  @defgroup  RLMLOSCCR  RLMLOSCCR Register
 *  @brief     LOSC Control Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:1  | -          |
 *  | 0     | XTEN       |
 *  @{
 */
/* XTEN */
#define RLMLOSCCR_XTEN_RW_DISABLE       ((uint32_t)0x00000000)    /*!< XTEN :[R/W]  :Disable  */
#define RLMLOSCCR_XTEN_RW_ENABLE        ((uint32_t)0x00000003)    /*!< XTEN :[R/W]  :Enable   */
/** 
 *  @}
 */ /* End of group RLMLOSCCR */

/** 
 *  @}
 */ /* End of group TMPM4G9_DEVE_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4G9_DEVE_Private_define TMPM4G9 Development Board Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4G9_DEVE_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4G9_DEVE_Private_typedef TMPM4G9 Development Board Private Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief   Timer Information Table.
 * @detail  
 * | [][0] | [][1] | [][2] |
 * | :---  | :---  | :---  |
 * | CH    | Type  | IRQ   |
*/
/*----------------------------------*/
typedef struct
{
    uint32_t  ch;       /*!< channel  */
    uint32_t  type;     /*!< type     */
    IRQn_Type irq;      /*!< irq Type */
} bsp_timer_info_t;

/*----------------------------------*/
/** 
 * @brief   Output Pulse Information Table.
 * @detail  
 * | [][0] | [][1] | [][2] |
 * | :---  | :---  | :---  |
 * | CH    | Type  | IRQ   |
*/
/*----------------------------------*/
typedef struct
{
    uint32_t  ch;       /*!< channel  */
    uint32_t  type;     /*!< type     */
    IRQn_Type irq;      /*!< irq Type */
} bsp_output_pulse_info_t;

/**
 *  @}
 */ /* End of group TMPM4G9_DEVE_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4G9_DEVE_Private_variables TMPM4G9 Development Board Private Variables
 *  @{
 */
static gpio_t port;     /*!< Port driver instance. */
static BSPExInt exi;
/*-----------------------------------------*/
/* Name    | Pin    | Port |  CN           */
/*-----------------------------------------*/
/* USB_TXD | 17     | PA1  |  5: 10        */
/*-----------------------------------------*/
/*----------------------------------*/
/** 
 * @brief   USB UART TX Information Table.
 * @detail  
 * | [][0] | [][1] | [][2] | [][3]    | [][4]         |
 * | :---  | :---  | :---  | :---     | :---          |
 * | Group | Num   | CH    | MDMAC CH | Baudrate(bps) |
*/
/*----------------------------------*/
static const uint32_t usbUartTxTbl[BSP_USB_UART_MAX][5] =
{
    {(uint32_t)GPIO_PORT_E, (uint32_t)GPIO_PORT_2, (uint32_t)(0), (uint32_t)(9), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_H, (uint32_t)GPIO_PORT_0, (uint32_t)(1), (uint32_t)(13), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_0, (uint32_t)(2), (uint32_t)(21), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_V, (uint32_t)GPIO_PORT_0, (uint32_t)(3), (uint32_t)(29), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_M, (uint32_t)GPIO_PORT_0, (uint32_t)(4), (uint32_t)(1), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_0, (uint32_t)(5), (uint32_t)(31), (uint32_t)(115200)},
};

/*-----------------------------------------*/
/* Name        | Pin    | Port |  CN       */
/*-----------------------------------------*/
/* USB_RXD     | 16     | PA2  |  5: 12    */
/*-----------------------------------------*/
/*----------------------------------*/
/** 
 * @brief   USB UART RX Information Table.
 * @detail  
 * | [][0] | [][1] | [][2] | [][3]    | [][4]         | 
 * | :---  | :---  | :---  | :---     | :---          |
 * | Group | Num   | CH    | MDMAC CH | Baudrate(bps) |
*/
/*----------------------------------*/
static const uint32_t usbUartRxTbl[BSP_USB_UART_MAX][5] =
{
    {(uint32_t)GPIO_PORT_E, (uint32_t)GPIO_PORT_3, (uint32_t)(0), (uint32_t)(7), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_H, (uint32_t)GPIO_PORT_1, (uint32_t)(1), (uint32_t)(11), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_1, (uint32_t)(2), (uint32_t)(19), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_V, (uint32_t)GPIO_PORT_1, (uint32_t)(3), (uint32_t)(28), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_M, (uint32_t)GPIO_PORT_1, (uint32_t)(4), (uint32_t)(0), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_1, (uint32_t)(5), (uint32_t)(30), (uint32_t)(115200)},
};

/*-----------------------------------------*/
/* Name        | Pin    | Port |  CN       */
/*-----------------------------------------*/
/* PORT_LED[0] | 30     | PB4  |  5: 28    */
/* PORT_LED[1] | 31     | PB5  |  5: 30    */
/* PORT_LED[2] | 32     | PB6  |  5: 32    */
/* PORT_LED[3] | 33     | PB7  |  5: 34    */
/*-----------------------------------------*/
/*----------------------------------*/
/** 
 * @brief   Port LED Information Table.
 * @detail  
 * | [][0] | [][1] |
 * | :---  | :---  |
 * | Group | Num   |
*/
/*----------------------------------*/
static const uint32_t ledTbl[BSP_LED_MAX][2] =
{
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_0},
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_1},
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_2},
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_3}
};

/*-----------------------------------------*/
/* Name        | Pin    | Port |  CN       */
/*-----------------------------------------*/
/* PORT_PSW[3] | 69     | PN1  |  9: 26    */
/* PORT_PSW[2] | 68     | PN2  |  9: 24    */
/* PORT_PSW[1] | 67     | PN3  |  9: 22    */
/* PORT_PSW[0] | 66     | PN4  |  9: 20    */
/*-----------------------------------------*/
/*----------------------------------*/
/** 
 * @brief   Push SW Information Table.
 * @detail  
 * | [][0] | [][1] |
 * | :---  | :---  |
 * | Group | Num   |
*/
/*----------------------------------*/
static const uint32_t pswTbl[BSP_PSW_MAX][2] =
{
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_6},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_7},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_6},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_7}
};

/*----------------------------------*/
/** 
 * @brief   Timer Information Table.
*/
/*----------------------------------*/
static const bsp_timer_info_t timerTbl[BSP_TIMER_MAX] =
{
    {(uint32_t)0, (uint32_t)T32A_TIMERA, INTT32A00_A_CT_IRQn},
    {(uint32_t)2, (uint32_t)T32A_TIMERA, INTT32A02_A_CT_IRQn}
};

/*----------------------------------*/
/** 
 * @brief   Output Pulse Information Table.
*/
/*----------------------------------*/
static const bsp_output_pulse_info_t pulseOutTbl[BSP_OUT_PULSE_SAMPLE_MAX] =
{
    {(uint32_t)3, (uint32_t)T32A_TIMERA, INTT32A03_A_CT_IRQn},
    {(uint32_t)3, (uint32_t)T32A_TIMERB, INTT32A03_B_C01_CPC_IRQn}
};
/*----------------------------------*/
/** 
 * @brief   TSPI TX Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2] | [][3]  | 
 * | :---  | :---  | :---  | :---   |
 * | Group | Num   | CH    | DMA CH |
*/
/*----------------------------------*/
static const uint32_t tspiTxTbl[BSP_SFLASH_MAX][4] =
{
    {(uint32_t)GPIO_PORT_A, (uint32_t)GPIO_PORT_0, (uint32_t)(0), (uint32_t)(1)},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_3, (uint32_t)(1), (uint32_t)(1)},
    {(uint32_t)GPIO_PORT_A, (uint32_t)GPIO_PORT_4, (uint32_t)(2), (uint32_t)(3)},
    {(uint32_t)GPIO_PORT_K, (uint32_t)GPIO_PORT_4, (uint32_t)(3), (uint32_t)(3)},
    {(uint32_t)GPIO_PORT_D, (uint32_t)GPIO_PORT_3, (uint32_t)(4), (uint32_t)(5)}
};
/*----------------------------------*/
/** 
 * @brief   TSPI CLK Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2] |
 * | :---  | :---  | :---  |
 * | Group | Num   | CH    |
*/
/*----------------------------------*/
static const uint32_t tspiCLKTbl[BSP_SFLASH_MAX][3] =
{
    {(uint32_t)GPIO_PORT_A, (uint32_t)GPIO_PORT_1, (uint32_t)(0)},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_1, (uint32_t)(1)},
    {(uint32_t)GPIO_PORT_A, (uint32_t)GPIO_PORT_6, (uint32_t)(2)},
    {(uint32_t)GPIO_PORT_K, (uint32_t)GPIO_PORT_6, (uint32_t)(3)},
    {(uint32_t)GPIO_PORT_D, (uint32_t)GPIO_PORT_1, (uint32_t)(4)}
};

/*-----------------------------------------*/
/* Name        | Pin    | Port |  CN       */
/*-----------------------------------------*/
/* USB_RXD     | 16     | PK0  |  5: 12    */
/*-----------------------------------------*/
/*----------------------------------*/
/** 
 * @brief   TSPI RX Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2] | [][3]  | 
 * | :---  | :---  | :---  | :---   |
 * | Group | Num   | CH    | DMA CH |
*/
/*----------------------------------*/
static const uint32_t tspiRxTbl[BSP_SFLASH_MAX][4] =
{
    {(uint32_t)GPIO_PORT_A, (uint32_t)GPIO_PORT_2, (uint32_t)(0), (uint32_t)(0)},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_2, (uint32_t)(1), (uint32_t)(0)},
    {(uint32_t)GPIO_PORT_A, (uint32_t)GPIO_PORT_5, (uint32_t)(2), (uint32_t)(2)},
    {(uint32_t)GPIO_PORT_K, (uint32_t)GPIO_PORT_5, (uint32_t)(3), (uint32_t)(2)},
    {(uint32_t)GPIO_PORT_D, (uint32_t)GPIO_PORT_2, (uint32_t)(4), (uint32_t)(4)}
};

/*-----------------------------------------*/
/* Name        | Pin    | Port |  CN       */
/*-----------------------------------------*/
/* PORT_AIN[2] | 2      | PD2  |  9: 20    */
/*-----------------------------------------*/
/*----------------------------------*/
/** 
 * @brief   Thermistors Information Table.
 * @detail  
 * | [][0] | [][1] | [][2]    | [][3]  | [][4]  |
 * | :---  | :---  | :---     | :---   | :---   |
 * | Group | Num   | ADIN CH  | ADC CH | DMA CH |
*/
/*----------------------------------*/
static const uint32_t thermistorsTbl[BSP_THERMISTORS_MAX][5] =
{
    {(uint32_t)GPIO_PORT_N, (uint32_t)GPIO_PORT_0, (uint32_t)(0),  (uint32_t)(0), (uint32_t)(19) },
    {(uint32_t)GPIO_PORT_N, (uint32_t)GPIO_PORT_1, (uint32_t)(1),  (uint32_t)(1), (uint32_t)(19) }
};

/*----------------------------------*/
/** 
 * @brief   Thermistors AD Value Table.
 * @detail   V   = AVCC*(TH1/TH1+R60)
 * @        AD   = V/(AVCC/0xFFF)
 * @               AVCC :5(v)
 * @               R60  :10(k-ohm)
 * @               TH1  :Refer thermistors temperature characteristics.
 * @        Resolution  :12 bit(=0xFFF).

*/
/*----------------------------------*/
static const int32_t NCP15XV103J03RC[][4] = 
{
    /*----------------------------------------*/
    /* TEMP. | AD-low   | AD-center| AD-high  */
    /* deg.C | -        | -        | -        */
    /*----------------------------------------*/
    { -40, 0xF72, 0xF86, 0xF96 },
    { -39, 0xF69, 0xF7E, 0xF8F },
    { -38, 0xF60, 0xF75, 0xF88 },
    { -37, 0xF56, 0xF6C, 0xF80 },
    { -36, 0xF4B, 0xF63, 0xF77 },
    { -35, 0xF40, 0xF59, 0xF6E },
    { -34, 0xF35, 0xF4E, 0xF65 },
    { -33, 0xF28, 0xF43, 0xF5B },
    { -32, 0xF1C, 0xF38, 0xF50 },
    { -31, 0xF0E, 0xF2C, 0xF45 },
    { -30, 0xF00, 0xF1F, 0xF3A },
    { -29, 0xEF2, 0xF12, 0xF2D },
    { -28, 0xEE2, 0xF04, 0xF21 },
    { -27, 0xED2, 0xEF5, 0xF13 },
    { -26, 0xEC2, 0xEE5, 0xF05 },
    { -25, 0xEB0, 0xED5, 0xEF6 },
    { -24, 0xE9E, 0xEC4, 0xEE6 },
    { -23, 0xE8B, 0xEB3, 0xED6 },
    { -22, 0xE77, 0xEA1, 0xEC5 },
    { -21, 0xE63, 0xE8E, 0xEB3 },
    { -20, 0xE4E, 0xE7A, 0xEA1 },
    { -19, 0xE38, 0xE65, 0xE8D },
    { -18, 0xE21, 0xE4F, 0xE79 },
    { -17, 0xE09, 0xE39, 0xE64 },
    { -16, 0xDF0, 0xE22, 0xE4E },
    { -15, 0xDD7, 0xE0A, 0xE37 },
    { -14, 0xDBD, 0xDF1, 0xE1F },
    { -13, 0xDA2, 0xDD7, 0xE07 },
    { -12, 0xD86, 0xDBC, 0xDED },
    { -11, 0xD69, 0xDA1, 0xDD3 },
    { -10, 0xD4C, 0xD84, 0xDB8 },
    {  -9, 0xD2D, 0xD67, 0xD9C },
    {  -8, 0xD0E, 0xD48, 0xD7E },
    {  -7, 0xCED, 0xD29, 0xD60 },
    {  -6, 0xCCC, 0xD09, 0xD41 },
    {  -5, 0xCAA, 0xCE8, 0xD21 },
    {  -4, 0xC88, 0xCC6, 0xD00 },
    {  -3, 0xC64, 0xCA4, 0xCDE },
    {  -2, 0xC40, 0xC80, 0xCBC },
    {  -1, 0xC1B, 0xC5C, 0xC98 },
    {   0, 0xBF6, 0xC37, 0xC74 },
    {   1, 0xBCF, 0xC11, 0xC4F },
    {   2, 0xBA8, 0xBEB, 0xC28 },
    {   3, 0xB81, 0xBC3, 0xC02 },
    {   4, 0xB59, 0xB9C, 0xBDA },
    {   5, 0xB30, 0xB73, 0xBB2 },
    {   6, 0xB07, 0xB4A, 0xB89 },
    {   7, 0xADD, 0xB20, 0xB5F },
    {   8, 0xAB3, 0xAF6, 0xB35 },
    {   9, 0xA88, 0xACB, 0xB0A },
    {  10, 0xA5D, 0xAA0, 0xADE },
    {  11, 0xA32, 0xA74, 0xAB2 },
    {  12, 0xA07, 0xA48, 0xA86 },
    {  13, 0x9DB, 0xA1C, 0xA59 },
    {  14, 0x9AF, 0x9EF, 0xA2C },
    {  15, 0x983, 0x9C2, 0x9FE },
    {  16, 0x956, 0x995, 0x9D0 },
    {  17, 0x92A, 0x968, 0x9A3 },
    {  18, 0x8FE, 0x93B, 0x974 },
    {  19, 0x8D1, 0x90D, 0x946 },
    {  20, 0x8A5, 0x8E0, 0x918 },
    {  21, 0x879, 0x8B3, 0x8E9 },
    {  22, 0x84D, 0x885, 0x8BB },
    {  23, 0x821, 0x858, 0x88D },
    {  24, 0x7F6, 0x82C, 0x85F },
    {  25, 0x7CA, 0x7FF, 0x831 },
    {  26, 0x79D, 0x7D3, 0x806 },
    {  27, 0x770, 0x7A7, 0x7DB },
    {  28, 0x743, 0x77B, 0x7B1 },
    {  29, 0x716, 0x750, 0x786 },
    {  30, 0x6EA, 0x725, 0x75D },
    {  31, 0x6BF, 0x6FA, 0x733 },
    {  32, 0x694, 0x6D0, 0x70A },
    {  33, 0x66A, 0x6A7, 0x6E1 },
    {  34, 0x640, 0x67E, 0x6B9 },
    {  35, 0x617, 0x655, 0x692 },
    {  36, 0x5EE, 0x62D, 0x66A },
    {  37, 0x5C7, 0x606, 0x644 },
    {  38, 0x5A0, 0x5DF, 0x61E },
    {  39, 0x579, 0x5B9, 0x5F8 },
    {  40, 0x553, 0x594, 0x5D3 },
    {  41, 0x52E, 0x56F, 0x5AF },
    {  42, 0x50A, 0x54B, 0x58B },
    {  43, 0x4E7, 0x528, 0x568 },
    {  44, 0x4C4, 0x505, 0x545 },
    {  45, 0x4A2, 0x4E3, 0x524 },
    {  46, 0x481, 0x4C2, 0x502 },
    {  47, 0x460, 0x4A1, 0x4E2 },
    {  48, 0x440, 0x481, 0x4C2 },
    {  49, 0x421, 0x462, 0x4A2 },
    {  50, 0x403, 0x443, 0x484 },
    {  51, 0x3E6, 0x425, 0x466 },
    {  52, 0x3C9, 0x408, 0x448 },
    {  53, 0x3AD, 0x3EC, 0x42C },
    {  54, 0x392, 0x3D0, 0x410 },
    {  55, 0x377, 0x3B5, 0x3F4 },
    {  56, 0x35D, 0x39B, 0x3DA },
    {  57, 0x344, 0x381, 0x3BF },
    {  58, 0x32C, 0x368, 0x3A6 },
    {  59, 0x314, 0x350, 0x38D },
    {  60, 0x2FD, 0x338, 0x375 },
    {  61, 0x2E6, 0x321, 0x35D },
    {  62, 0x2D0, 0x30A, 0x346 },
    {  63, 0x2BB, 0x2F4, 0x330 },
    {  64, 0x2A6, 0x2DF, 0x31A },
    {  65, 0x292, 0x2CA, 0x304 },
    {  66, 0x27F, 0x2B6, 0x2F0 },
    {  67, 0x26C, 0x2A2, 0x2DB },
    {  68, 0x25A, 0x28F, 0x2C7 },
    {  69, 0x248, 0x27D, 0x2B4 },
    {  70, 0x236, 0x26B, 0x2A1 },
    {  71, 0x226, 0x259, 0x28F },
    {  72, 0x216, 0x248, 0x27E },
    {  73, 0x206, 0x238, 0x26C },
    {  74, 0x1F7, 0x228, 0x25C },
    {  75, 0x1E8, 0x218, 0x24C },
    {  76, 0x1DA, 0x209, 0x23C },
    {  77, 0x1CC, 0x1FB, 0x22C },
    {  78, 0x1BE, 0x1ED, 0x21E },
    {  79, 0x1B1, 0x1DF, 0x20F },
    {  80, 0x1A5, 0x1D1, 0x201 },
    {  81, 0x199, 0x1C4, 0x1F3 },
    {  82, 0x18D, 0x1B8, 0x1E6 },
    {  83, 0x181, 0x1AC, 0x1D9 },
    {  84, 0x176, 0x1A0, 0x1CC },
    {  85, 0x16B, 0x194, 0x1C0 },
    {  86, 0x161, 0x189, 0x1B4 },
    {  87, 0x157, 0x17E, 0x1A9 },
    {  88, 0x14D, 0x174, 0x19E },
    {  89, 0x143, 0x169, 0x193 },
    {  90, 0x13A, 0x160, 0x188 },
    {  91, 0x131, 0x156, 0x17E },
    {  92, 0x129, 0x14D, 0x174 },
    {  93, 0x120, 0x144, 0x16A },
    {  94, 0x118, 0x13B, 0x161 },
    {  95, 0x110, 0x132, 0x158 },
    {  96, 0x109, 0x12A, 0x14F },
    {  97, 0x101, 0x122, 0x146 },
    {  98,  0xFA, 0x11A, 0x13E },
    {  99,  0xF3, 0x113, 0x136 },
    { 100,  0xED, 0x10C, 0x12E },
    { 101,  0xE6, 0x104, 0x126 },
    { 102,  0xE0,  0xFE, 0x11E },
    { 103,  0xDA,  0xF7, 0x117 },
    { 104,  0xD4,  0xF0, 0x110 },
    { 105,  0xCE,  0xEA, 0x109 },
    { 106,  0xC8,  0xE4, 0x103 },
    { 107,  0xC3,  0xDE,  0xFC },
    { 108,  0xBE,  0xD8,  0xF6 },
    { 109,  0xB9,  0xD3,  0xF0 },
    { 110,  0xB4,  0xCE,  0xEA },
    { 111,  0xAF,  0xC8,  0xE4 },
    { 112,  0xAA,  0xC3,  0xDF },
    { 113,  0xA6,  0xBE,  0xD9 },
    { 114,  0xA2,  0xB9,  0xD4 },
    { 115,  0x9D,  0xB5,  0xCF },
    { 116,  0x99,  0xB0,  0xCA },
    { 117,  0x95,  0xAC,  0xC5 },
    { 118,  0x92,  0xA8,  0xC0 },
    { 119,  0x8E,  0xA3,  0xBC },
    { 120,  0x8A,  0x9F,  0xB7 },
    { 121,  0x87,  0x9B,  0xB3 },
    { 122,  0x83,  0x98,  0xAF },
    { 123,  0x80,  0x94,  0xAB },
    { 124,  0x7D,  0x90,  0xA7 },
    { 125,  0x7A,  0x8D,  0xA3 }
};

/*----------------------------------*/
/** 
 * @brief   DNF Information Table.
 * @details   Detail.
 * | [][0] | 
 * | :---  | 
 * | Num | 
*/
/*-----------------------------------------*/
static const uint32_t dnfTbl[BSP_DNF_MAX][1] =
{
    {(uint32_t)4},
};
/**
 *  @}
 */ /* End of group TMPM4G9_DEVE_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4G9_DEVE_Private_fuctions TMPM4G9 Development Board Private Fuctions
 *  @{
 */
static void bsp_error(void);
static void fsys_all_enable(void);
static void fsys_all_disable(void);
static void adc_clock_enable(void);
static void adc_clock_disable(void);
static void trace_clock_enable(void);
static void trace_clock_disable(void);
static void int_ctr_reg_enable(void);
static void int_ctr_reg_disable(void);
static void port_initialize(void);
static void port_finalize(void);
static void external_losc_enable(void);
static void external_losc_disable(void);

/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void bsp_error(void)
{
#ifdef BSP_DEBUG
    while(1)
    {
        __nop();
    }
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  CG Fsys All Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void fsys_all_enable(void)
{
    /*--- [CGFSYSMENA] ---*/
    TSB_CG_FSYSMENA_IPMENA00 = 1;     /* MDMA      */
    TSB_CG_FSYSMENA_IPMENA01 = 1;     /* FUART 0   */
    TSB_CG_FSYSMENA_IPMENA02 = 1;     /* FUART 1   */
    TSB_CG_FSYSMENA_IPMENA03 = 1;     /* ADC   A   */
    TSB_CG_FSYSMENA_IPMENA04 = 1;     /* DAC   0   */
    TSB_CG_FSYSMENA_IPMENA05 = 1;     /* DAC   1   */
    TSB_CG_FSYSMENA_IPMENA06 = 1;     /* T32A  0   */
    TSB_CG_FSYSMENA_IPMENA07 = 1;     /* T32A  1   */
    TSB_CG_FSYSMENA_IPMENA08 = 1;     /* T32A  2   */
    TSB_CG_FSYSMENA_IPMENA09 = 1;     /* T32A  3   */
    TSB_CG_FSYSMENA_IPMENA10 = 1;     /* T32A  4   */
    TSB_CG_FSYSMENA_IPMENA11 = 1;     /* T32A  5   */
    TSB_CG_FSYSMENA_IPMENA12 = 1;     /* T32A  6   */
    TSB_CG_FSYSMENA_IPMENA13 = 1;     /* T32A  7   */
    TSB_CG_FSYSMENA_IPMENA14 = 1;     /* T32A  8   */
    TSB_CG_FSYSMENA_IPMENA15 = 1;     /* T32A  9   */
    TSB_CG_FSYSMENA_IPMENA16 = 1;     /* T32A  10  */
    TSB_CG_FSYSMENA_IPMENA17 = 1;     /* T32A  11  */
    TSB_CG_FSYSMENA_IPMENA18 = 1;     /* T32A  12  */
    TSB_CG_FSYSMENA_IPMENA19 = 1;     /* T32A  13  */
    TSB_CG_FSYSMENA_IPMENA20 = 1;     /* TSPI  6   */
    TSB_CG_FSYSMENA_IPMENA21 = 1;     /* TSPI  7   */
    TSB_CG_FSYSMENA_IPMENA22 = 1;     /* TSPI  8   */
    TSB_CG_FSYSMENA_IPMENA23 = 1;     /* UART  0   */
    TSB_CG_FSYSMENA_IPMENA24 = 1;     /* UART  1   */
    TSB_CG_FSYSMENA_IPMENA25 = 1;     /* UART  2   */
    TSB_CG_FSYSMENA_IPMENA26 = 1;     /* UART  3   */
    TSB_CG_FSYSMENA_IPMENA27 = 1;     /* UART  4   */
    TSB_CG_FSYSMENA_IPMENA28 = 1;     /* UART  5   */
    TSB_CG_FSYSMENA_IPMENA29 = 1;     /* I2C   0   */
    TSB_CG_FSYSMENA_IPMENA30 = 1;     /* I2C   1   */
    TSB_CG_FSYSMENA_IPMENA31 = 1;     /* I2C   2   */
    /*--- [CGFSYSMENB] ---*/
    TSB_CG_FSYSMENB_IPMENB00 = 1;     /* I2C   3   */
    TSB_CG_FSYSMENB_IPMENB01 = 1;     /* I2C   4   */
    TSB_CG_FSYSMENB_IPMENB02 = 1;     /* PORT  A   */
    TSB_CG_FSYSMENB_IPMENB03 = 1;     /* PORT  B   */
    TSB_CG_FSYSMENB_IPMENB04 = 1;     /* PORT  C   */
    TSB_CG_FSYSMENB_IPMENB05 = 1;     /* PORT  D   */
    TSB_CG_FSYSMENB_IPMENB06 = 1;     /* PORT  E   */
    TSB_CG_FSYSMENB_IPMENB07 = 1;     /* PORT  F   */
    TSB_CG_FSYSMENB_IPMENB08 = 1;     /* PORT  G   */
    TSB_CG_FSYSMENB_IPMENB09 = 1;     /* PORT  H   */
    TSB_CG_FSYSMENB_IPMENB10 = 1;     /* PORT  J   */
    TSB_CG_FSYSMENB_IPMENB11 = 1;     /* PORT  K   */
    TSB_CG_FSYSMENB_IPMENB12 = 1;     /* PORT  L   */
    TSB_CG_FSYSMENB_IPMENB13 = 1;     /* PORT  M   */
    TSB_CG_FSYSMENB_IPMENB14 = 1;     /* PORT  N   */
    TSB_CG_FSYSMENB_IPMENB15 = 1;     /* PORT  P   */
    TSB_CG_FSYSMENB_IPMENB16 = 1;     /* PORT  R   */
    TSB_CG_FSYSMENB_IPMENB17 = 1;     /* PORT  T   */
    TSB_CG_FSYSMENB_IPMENB18 = 1;     /* PORT  U   */
    TSB_CG_FSYSMENB_IPMENB19 = 1;     /* PORT  V   */
    TSB_CG_FSYSMENB_IPMENB20 = 1;     /* PORT  W   */
    TSB_CG_FSYSMENB_IPMENB21 = 1;     /* PORT  Y   */
    TSB_CG_FSYSMENB_IPMENB22 = 1;     /* IOSC      */
    TSB_CG_FSYSMENB_IPMENB23 = 1;     /* OFD       */
    TSB_CG_FSYSMENB_IPMENB24 = 1;     /* PMD       */
    TSB_CG_FSYSMENB_IPMENB28 = 1;     /* TRGSEL    */
    TSB_CG_FSYSMENB_IPMENB29 = 1;     /* -         */
    TSB_CG_FSYSMENB_IPMENB30 = 1;     /* NBD       */
    TSB_CG_FSYSMENB_IPMENB31 = 1;     /* WDT       */
    /*--- [CGFSYSENA] ---*/
    TSB_CG_FSYSENA_IPENA00   = 1;     /* SDMA A    */
    TSB_CG_FSYSENA_IPENA01   = 1;     /* SDMA B    */
    TSB_CG_FSYSENA_IPENA02   = 1;     /* SMIF      */
    TSB_CG_FSYSENA_IPENA03   = 1;     /* EBIF      */
    TSB_CG_FSYSENA_IPENA04   = 1;     /* TSPI 0    */
    TSB_CG_FSYSENA_IPENA05   = 1;     /* TSPI 1    */
    TSB_CG_FSYSENA_IPENA06   = 1;     /* TSPI 2    */
    TSB_CG_FSYSENA_IPENA07   = 1;     /* TSPI 3    */
    TSB_CG_FSYSENA_IPENA08   = 1;     /* TSPI 4    */
    TSB_CG_FSYSENA_IPENA09   = 1;     /* TSPI 5    */
    /*--- [CGFCEN] ---*/
    TSB_CG_FCEN_FCIPEN23 = 1;         /* OFD */
}

/*--------------------------------------------------*/
/** 
  * @brief  CG Fsys All Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void fsys_all_disable(void)
{
    /*--- [CGFSYSMENA] ---*/
    TSB_CG_FSYSMENA_IPMENA00 = 0;     /* MDMA      */
    TSB_CG_FSYSMENA_IPMENA01 = 0;     /* FUART 0   */
    TSB_CG_FSYSMENA_IPMENA02 = 0;     /* FUART 1   */
    TSB_CG_FSYSMENA_IPMENA03 = 0;     /* ADC   A   */
    TSB_CG_FSYSMENA_IPMENA04 = 0;     /* DAC   0   */
    TSB_CG_FSYSMENA_IPMENA05 = 0;     /* DAC   1   */
    TSB_CG_FSYSMENA_IPMENA06 = 0;     /* T32A  0   */
    TSB_CG_FSYSMENA_IPMENA07 = 0;     /* T32A  1   */
    TSB_CG_FSYSMENA_IPMENA08 = 0;     /* T32A  2   */
    TSB_CG_FSYSMENA_IPMENA09 = 0;     /* T32A  3   */
    TSB_CG_FSYSMENA_IPMENA10 = 0;     /* T32A  4   */
    TSB_CG_FSYSMENA_IPMENA11 = 0;     /* T32A  5   */
    TSB_CG_FSYSMENA_IPMENA12 = 0;     /* T32A  6   */
    TSB_CG_FSYSMENA_IPMENA13 = 0;     /* T32A  7   */
    TSB_CG_FSYSMENA_IPMENA14 = 0;     /* T32A  8   */
    TSB_CG_FSYSMENA_IPMENA15 = 0;     /* T32A  9   */
    TSB_CG_FSYSMENA_IPMENA16 = 0;     /* T32A  10  */
    TSB_CG_FSYSMENA_IPMENA17 = 0;     /* T32A  11  */
    TSB_CG_FSYSMENA_IPMENA18 = 0;     /* T32A  12  */
    TSB_CG_FSYSMENA_IPMENA19 = 0;     /* T32A  13  */
    TSB_CG_FSYSMENA_IPMENA20 = 0;     /* TSPI  6   */
    TSB_CG_FSYSMENA_IPMENA21 = 0;     /* TSPI  7   */
    TSB_CG_FSYSMENA_IPMENA22 = 0;     /* TSPI  8   */
    TSB_CG_FSYSMENA_IPMENA23 = 0;     /* UART  0   */
    TSB_CG_FSYSMENA_IPMENA24 = 0;     /* UART  1   */
    TSB_CG_FSYSMENA_IPMENA25 = 0;     /* UART  2   */
    TSB_CG_FSYSMENA_IPMENA26 = 0;     /* UART  3   */
    TSB_CG_FSYSMENA_IPMENA27 = 0;     /* UART  4   */
    TSB_CG_FSYSMENA_IPMENA28 = 0;     /* UART  5   */
    TSB_CG_FSYSMENA_IPMENA29 = 0;     /* I2C   0   */
    TSB_CG_FSYSMENA_IPMENA30 = 0;     /* I2C   1   */
    TSB_CG_FSYSMENA_IPMENA31 = 0;     /* I2C   2   */
    /*--- [CGFSYSMENB] ---*/
    TSB_CG_FSYSMENB_IPMENB00 = 0;     /* I2C   3   */
    TSB_CG_FSYSMENB_IPMENB01 = 0;     /* I2C   4   */
    TSB_CG_FSYSMENB_IPMENB02 = 0;     /* PORT  A   */
    TSB_CG_FSYSMENB_IPMENB03 = 0;     /* PORT  B   */
    TSB_CG_FSYSMENB_IPMENB04 = 0;     /* PORT  C   */
    TSB_CG_FSYSMENB_IPMENB05 = 0;     /* PORT  D   */
    TSB_CG_FSYSMENB_IPMENB06 = 0;     /* PORT  E   */
    TSB_CG_FSYSMENB_IPMENB07 = 0;     /* PORT  F   */
    TSB_CG_FSYSMENB_IPMENB08 = 0;     /* PORT  G   */
    TSB_CG_FSYSMENB_IPMENB09 = 0;     /* PORT  H   */
    TSB_CG_FSYSMENB_IPMENB10 = 0;     /* PORT  J   */
    TSB_CG_FSYSMENB_IPMENB11 = 0;     /* PORT  K   */
    TSB_CG_FSYSMENB_IPMENB12 = 0;     /* PORT  L   */
    TSB_CG_FSYSMENB_IPMENB13 = 0;     /* PORT  M   */
    TSB_CG_FSYSMENB_IPMENB14 = 0;     /* PORT  N   */
    TSB_CG_FSYSMENB_IPMENB15 = 0;     /* PORT  P   */
    TSB_CG_FSYSMENB_IPMENB16 = 0;     /* PORT  R   */
    TSB_CG_FSYSMENB_IPMENB17 = 0;     /* PORT  T   */
    TSB_CG_FSYSMENB_IPMENB18 = 0;     /* PORT  U   */
    TSB_CG_FSYSMENB_IPMENB19 = 0;     /* PORT  V   */
    TSB_CG_FSYSMENB_IPMENB20 = 0;     /* PORT  W   */
    TSB_CG_FSYSMENB_IPMENB21 = 0;     /* PORT  Y   */
    TSB_CG_FSYSMENB_IPMENB22 = 0;     /* IOSC      */
    TSB_CG_FSYSMENB_IPMENB23 = 0;     /* OFD       */
    TSB_CG_FSYSMENB_IPMENB24 = 0;     /* PMD       */
    TSB_CG_FSYSMENB_IPMENB28 = 0;     /* TRGSEL    */
    TSB_CG_FSYSMENB_IPMENB29 = 0;     /* -         */
    TSB_CG_FSYSMENB_IPMENB30 = 0;     /* NBD       */
    TSB_CG_FSYSMENB_IPMENB31 = 0;     /* WDT       */
    /*--- [CGFSYSENA] ---*/
    TSB_CG_FSYSENA_IPENA00   = 0;     /* SDMA A    */
    TSB_CG_FSYSENA_IPENA01   = 0;     /* SDMA B    */
    TSB_CG_FSYSENA_IPENA02   = 0;     /* SMIF      */
    TSB_CG_FSYSENA_IPENA03   = 0;     /* EBIF      */
    TSB_CG_FSYSENA_IPENA04   = 0;     /* TSPI 0    */
    TSB_CG_FSYSENA_IPENA05   = 0;     /* TSPI 1    */
    TSB_CG_FSYSENA_IPENA06   = 0;     /* TSPI 2    */
    TSB_CG_FSYSENA_IPENA07   = 0;     /* TSPI 3    */
    TSB_CG_FSYSENA_IPENA08   = 0;     /* TSPI 4    */
    TSB_CG_FSYSENA_IPENA09   = 0;     /* TSPI 5    */
    /*--- [CGFCEN] ---*/
    TSB_CG_FCEN_FCIPEN23 = 0;         /* OFD */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC Clock Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void adc_clock_enable(void)
{
    /*--- [CGSPCLKEN:ADCKEN] ---*/
    TSB_CG_SPCLKEN_ADCKEN = 1;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC Clock Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void adc_clock_disable(void)
{
    /*--- [CGSPCLKEN:ADCKEN] ---*/
    TSB_CG_SPCLKEN_ADCKEN = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  ETM Trace Clock Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void trace_clock_enable(void)
{
    /*--- [CGSPCLKEN:ADCKEN] ---*/
    TSB_CG_SPCLKEN_TRCKEN = 1;
}

/*--------------------------------------------------*/
/** 
  * @brief  ETM Trace Clock Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void trace_clock_disable(void)
{
    /*--- [CGSPCLKEN:ADCKEN] ---*/
    TSB_CG_SPCLKEN_TRCKEN = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A  INT Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void int_ctr_reg_enable(void)
{
    /*--- [IBIMCxxx : <INTEN> Enable] ---*/
    TSB_IB->IMC000 = 0x1; /*T32A ch0 TimerA Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC003 = 0x1; /*T32A ch0 TimerB Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC010 = 0x1; /*T32A ch1 TimerA Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC013 = 0x1; /*T32A ch1 TimerB Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC020 = 0x1; /*T32A ch2 TimerA Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC023 = 0x1; /*T32A ch2 TimerB Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC030 = 0x1; /*T32A ch3 TimerA Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC033 = 0x1; /*T32A ch3 TimerB Reg Match/Over Flow/Under Flow */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A  INT Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void int_ctr_reg_disable(void)
{
    /*--- [IBIMCxxx : <INTEN> Disable] ---*/
    TSB_IB->IMC000 = 0x0; /*T32A ch0 TimerA Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC003 = 0x0; /*T32A ch0 TimerB Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC010 = 0x0; /*T32A ch1 TimerA Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC013 = 0x0; /*T32A ch1 TimerB Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC020 = 0x0; /*T32A ch2 TimerA Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC023 = 0x0; /*T32A ch2 TimerB Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC030 = 0x0; /*T32A ch3 TimerA Reg Match/Over Flow/Under Flow */
    TSB_IB->IMC033 = 0x0; /*T32A ch3 TimerB Reg Match/Over Flow/Under Flow */
}
/*--------------------------------------------------*/
/** 
  * @brief  Port Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void port_initialize(void)
{
    /*================================================================================*/
    /*  [Port A]                                                                      */
    /*--------------------------------------------------------------------------------*/
    /*  Port | Function   | I/O | PADATA | PACR | PAFRn | PAOD | PAPUP | PADN | PAIE  */
    /*--------------------------------------------------------------------------------*/
    /*  PA0  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PA1  | T32A00OUTA | O   | 0      | 1    | PAFR3 | 0    | 0     | 0    | 0     */
    /*  PA2  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PA3  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PA4  | TSPI2TXD   | O   | 0      | 1    | PAFR7 | 0    | 0     | 0    | 0     */
    /*  PA5  | TSPI2RXD   | I   | 0      | 0    | PAFR7 | 0    | 0     | 0    | 1     */
    /*  PA6  | TSPI2SCK   | O   | 0      | 1    | PAFR7 | 0    | 0     | 0    | 0     */
    /*  PA7  | TSPI2CS0   | O   | 0      | 1    | PAFR7 | 0    | 0     | 0    | 0     */
    /*================================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pa_instance = TSB_PA;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_A) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PA0 -*/
    /* Use reset value.  */
    /*- PA1 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_1, (uint32_t)GPIO_PA1_T32A00OUTA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PA2 -*/
    /* Use reset value.  */
    /*- PA3 -*/
    /* Use reset value.  */
    /*- PA4 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_4, (uint32_t)GPIO_PA4_TSPI2TXD, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PA5 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_5, (uint32_t)GPIO_PA5_TSPI2RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PA6 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_6, (uint32_t)GPIO_PA6_TSPI2SCK, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PA7 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_7, (uint32_t)GPIO_PA7_TSPI2CS0, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*================================================================================*/
    /*  [Port B]                                                                      */
    /*--------------------------------------------------------------------------------*/
    /*  Port | Function   | I/O | PBDATA | PBCR | PBFRn | PBOD | PBPUP | PBDN | PBIE  */
    /*--------------------------------------------------------------------------------*/
    /*  PB0  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PB1  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PB2  | T32A02OUTA | O   | 0      | 1    | PAFR3 | 0    | 0     | 0    | 0     */
    /*  PB3  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PB4  | T32A03OUTA | O   | 0      | 1    | PAFR3 | 0    | 0     | 0    | 0     */
    /*  PB5  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PB6  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PB7  | -          | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*================================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pb_instance = TSB_PB;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_B) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PB0 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_0, (uint32_t)GPIO_PB0_INT04a, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /* Use reset value.  */
    /*- PB1 -*/
    /* Use reset value.  */
    /*- PB2 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_2, (uint32_t)GPIO_PB2_T32A02OUTA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PB3 -*/
    /* Use reset value.  */
    /*- PB4 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_4, (uint32_t)GPIO_PB4_T32A03OUTA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PB5 -*/
    /* Use reset value.  */
    /*- PB6 -*/
    /* Use reset value.  */
    /*- PB7 -*/
    /* Use reset value.  */
    
    /*===============================================================================*/
    /*  [Port C]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PCDATA | PCCR | PCFRn | PCOD | PCPUP | PCDN | PCIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PC0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PC1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PC2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PC3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PC4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PC5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PC6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  P76  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pc_instance = TSB_PC;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_C) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PC0 -*/
    /* Use reset value.  */
    /*- PC1 -*/
    /* Use reset value.  */
    /*- PC2 -*/
    /* Use reset value.  */
    /*- PC3 -*/
    /* Use reset value.  */
    /*- PC4 -*/
    /* Use reset value.  */
    /*- PC5 -*/
    /* Use reset value.  */
    /*- PC6 -*/
    /* Use reset value.  */
    /*- PC7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port D]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PDDATA | PDCR | PDFRn | PDOD | PDPUP | PDDN | PDIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PD0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PD1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PD2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PD3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PD4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PD5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PD6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PD7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pd_instance = TSB_PD;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_D) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PD0 -*/
    /* Use reset value.  */
    /*- PD1 -*/
    /* Use reset value.  */
    /*- PD2 -*/
    /* Use reset value.  */
    /*- PD3 -*/
    /* Use reset value.  */
    /*- PD4 -*/
    /* Use reset value.  */
    /*- PD5 -*/
    /* Use reset value.  */
    /*- PD6 -*/
    /* Use reset value.  */
    /*- PD7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port E]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PEDATA | PECR | PEFRn | PEOD | PEPUP | PEDN | PEIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PE0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PE1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PE2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PE3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PE4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PE5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PE6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PE7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pe_instance = TSB_PE;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_E) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PE0 -*/
    /* Use reset value.  */
    /*- PE1 -*/
    /* Use reset value.  */
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_2, (uint32_t)GPIO_PE2_UT0RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PE3 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_3, (uint32_t)GPIO_PE3_UT0TXDA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PE4 -*/
    /* Use reset value.  */
    /*- PE5 -*/
    /* Use reset value.  */
    /*- PE6 -*/
    /* Use reset value.  */
    /*- PE7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port F]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PFDATA | PFCR | PFFRn | PFOD | PFPUP | PFDN | PFIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PF0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pf_instance = TSB_PF;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_F) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PF0 -*/
    /* Use reset value.  */
    /*- PF1 -*/
    /* Use reset value.  */
    /*- PF2 -*/
    /* Use reset value.  */
    /*- PF3 -*/
    /* Use reset value.  */
    /*- PF4 -*/
    /* Use reset value.  */
    /*- PF5 -*/
    /* Use reset value.  */
    /*- PF6 -*/
    /* Use reset value.  */
    /*- PF7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port G]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PGDATA | PGCR | PGFRn | PGOD | PGPUP | PGDN | PGIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PG0  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG1  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG2  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG3  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG4  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG5  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG6  |  PSW 0    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PG7  |  PSW 1    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pg_instance = TSB_PG;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_G) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PG0 -*/
    /* Use reset value.  */
    /*- PG1 -*/
    /* Use reset value.  */
    /*- PG2 -*/
    /* Use reset value.  */
    /*- PG3 -*/
    /* Use reset value.  */
    /*- PG4 -*/
#ifdef FUART_TRANSFER_USE
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_4, (uint32_t)GPIO_PG4_FUT0TXD, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
#else
    /* Use reset value.  */
#endif
    /*- PG5 -*/
#ifdef FUART_TRANSFER_USE
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_5, (uint32_t)GPIO_PG5_FUT0RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
#else
    /* Use reset value.  */
#endif
    /*- PG6 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_6, (uint32_t)GPIO_PG6_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PG7 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_7, (uint32_t)GPIO_PG7_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*===============================================================================*/
    /*  [Port H]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PHDATA | PHCR | PHFRn | PHOD | PHPUP | PHDN | PHIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PH0  | UT1TXDA   | O   | 0      | 1    | PHFR5 | 0    | 0     | 0    | 0     */
    /*  PH1  | UT1RXD    | I   | 0      | 0    | PHFR5 | 0    | 0     | 0    | 1     */
    /*  PH2  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PH3  | TDI       | -   | 0      | 0    | PHFR1 | 0    | 1     | 0    | 1     */
    /*  PH4  | SWDIO     | -   | 0      | 1    | PHFR1 | 0    | 1     | 0    | 1     */
    /*  PH5  | TCK       | -   | 0      | 0    | PHFR1 | 0    | 0     | 1    | 1     */
    /*  PH6  | TDO       | -   | 0      | 1    | PHFR1 | 0    | 0     | 0    | 0     */
    /*  PH7  | TRST_N    | -   | 0      | 0    | PHFR1 | 0    | 1     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_ph_instance = TSB_PH;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_H) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PH0 -*/
    if (gpio_func(&port, GPIO_PORT_H, GPIO_PORT_0, (uint32_t)GPIO_PH0_UT1TXDA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PH1 -*/
    if (gpio_func(&port, GPIO_PORT_H, GPIO_PORT_1, (uint32_t)GPIO_PH1_UT1RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PH2 -*/
    /* Use reset value.  */
    /*- PH3 -*/
    /* Use reset value.  */    
    /*- PH4 -*/
    /* Use reset value.  */
    /*- PH5 -*/
    /* Use reset value.  */
    /*- PH6 -*/
    /* Use reset value.  */
    /*- PH7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port J]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PJDATA | PJCR | PJFRn | PJOD | PJPUP | PJDN | PJIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PJ0  | LED 0     | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ1  | LED 0     | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ2  | LED 0     | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ3  | LED 0     | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pj_instance = TSB_PJ;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_J) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PJ0 -*/
    if (gpio_func(&port, GPIO_PORT_J, GPIO_PORT_0, (uint32_t)GPIO_PJ0_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PJ1 -*/
    if (gpio_func(&port, GPIO_PORT_J, GPIO_PORT_1, (uint32_t)GPIO_PJ1_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PJ2 -*/
    if (gpio_func(&port, GPIO_PORT_J, GPIO_PORT_2, (uint32_t)GPIO_PJ2_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PJ3 -*/
    if (gpio_func(&port, GPIO_PORT_J, GPIO_PORT_3, (uint32_t)GPIO_PJ3_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PJ4 -*/
    /* Use reset value.  */
    /*- PJ5 -*/
    /* Use reset value.  */
    /*- PJ6 -*/
    /* Use reset value.  */
    /*- PJ7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port K]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PKDATA | PKCR | PKFRn | PKOD | PKPUP | PKDN | PKIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PK0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK4  |PORT_LED[0]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK5  |PORT_LED[1]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK6  |PORT_LED[2]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK7  |PORT_LED[3]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pk_instance = TSB_PK;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_K) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PK0 -*/
    /* Use reset value.  */
    /*- PK1 -*/
    /* Use reset value.  */
    /*- PK2 -*/
    /* Use reset value.  */
    /*- PK3 -*/
    /* Use reset value.  */
    /*- PK4 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_4, (uint32_t)GPIO_PK4_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PK5 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_5, (uint32_t)GPIO_PK5_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PK6 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_6, (uint32_t)GPIO_PK6_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PK7 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_7, (uint32_t)GPIO_PK7_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*===============================================================================*/
    /*  [Port L]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PLDATA | PLCR | PLFRn | PLOD | PLPUP | PLDN | PLIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PL0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL6  | PSW 2     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PL7  | PSW 3     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pl_instance = TSB_PL;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_L) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PL0 -*/
    /* Use reset value.  */
    /*- PL1 -*/
    /* Use reset value.  */
    /*- PL2 -*/
    /* Use reset value.  */
    /*- PL3 -*/
    /* Use reset value.  */
    /*- PL4 -*/
    /* Use reset value.  */
    /*- PL5 -*/
    /* Use reset value.  */
    /*- PL6 -*/
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_6, (uint32_t)GPIO_PL6_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PL7 -*/
#ifdef CGRST_SAMPLE
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_7, (uint32_t)GPIO_PL7_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
#else
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_7, (uint32_t)GPIO_PL7_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
#endif    
    /*===============================================================================*/
    /*  [Port M]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PMDATA | PMCR | PMFRn | PMOD | PMPUP | PMDN | PMIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PM0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pm_instance = TSB_PM;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_M) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PM0 -*/
    if (gpio_func(&port, GPIO_PORT_M, GPIO_PORT_0, (uint32_t)GPIO_PM0_UT4TXDA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PM1 -*/
    if (gpio_func(&port, GPIO_PORT_M, GPIO_PORT_1, (uint32_t)GPIO_PM1_UT4RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PM2 -*/
    /* Use reset value.  */
    /*- PM3 -*/
    /* Use reset value.  */
    /*- PM4 -*/
    /* Use reset value.  */
    /*- PM5 -*/
    /* Use reset value.  */
    /*- PM6 -*/
    /* Use reset value.  */
    /*- PM7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port N]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PNDATA | PNCR | PNFRn | PNOD | PNPUP | PNDN | PNIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PN0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pn_instance = TSB_PN;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_N) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PN0 -*/
    if (gpio_func(&port, GPIO_PORT_N, GPIO_PORT_0, (uint32_t)GPIO_PN0_AINA00, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PN1 -*/
    if (gpio_func(&port, GPIO_PORT_N, GPIO_PORT_1, (uint32_t)GPIO_PN1_AINA01, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PN2 -*/
    /* Use reset value.  */
    /*- PN3 -*/
    /* Use reset value.  */
    /*- PN4 -*/
    /* Use reset value.  */
    /*- PN5 -*/
    /* Use reset value.  */
    /*- PN6 -*/
    /* Use reset value.  */
    /*- PN7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port P]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PPDATA | PPCR | PPFRn | PPOD | PPPUP | PPDN | PPIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PP0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pp_instance = TSB_PP;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_P) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PP0 -*/
    /* Use reset value.  */
    /*- PP1 -*/
    /* Use reset value.  */
    /*- PP2 -*/
    /* Use reset value.  */
    /*- PP3 -*/
    /* Use reset value.  */
    /*- PP4 -*/
    /* Use reset value.  */
    /*- PP5 -*/
    /* Use reset value.  */
    /*- PP6 -*/
    /* Use reset value.  */
    /*- PP7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port R]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PRDATA | PRCR | PRFRn | PROD | PRPUP | PRDN | PRIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PR0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pr_instance = TSB_PR;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_R) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PR0 -*/
    /* Use reset value.  */
    /*- PR1 -*/
    /* Use reset value.  */
    /*- PR2 -*/
    /* Use reset value.  */
    /*- PR3 -*/
    /* Use reset value.  */
    /*- PR4 -*/
    /* Use reset value.  */
    /*- PR5 -*/
    /* Use reset value.  */
    /*- PR6 -*/
    /* Use reset value.  */
    /*- PR7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port T]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PTDATA | PTCR | PTFRn | PTOD | PTPUP | PTDN | PTIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PT0  | DAC0      | O   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT3  | RXIN0     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PT4  | RXIN1     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PT5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pt_instance = TSB_PT;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_T) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PT0 -*/
    if (gpio_func(&port, GPIO_PORT_T, GPIO_PORT_0, (uint32_t)GPIO_PT0_DAC0, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PT1 -*/
    if (gpio_func(&port, GPIO_PORT_T, GPIO_PORT_1, (uint32_t)GPIO_PT1_DAC1, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PT2 -*/
#ifdef CEC_USE
    if (gpio_func(&port, GPIO_PORT_T, GPIO_PORT_2, (uint32_t)GPIO_PT2_CEC0, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
#endif
    /*- PT3 -*/
    if (gpio_func(&port, GPIO_PORT_T, GPIO_PORT_3, (uint32_t)GPIO_PT3_RXIN0, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PT4 -*/
    if (gpio_func(&port, GPIO_PORT_T, GPIO_PORT_4, (uint32_t)GPIO_PT4_RXIN1, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PT5 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port U]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PUDATA | PUCR | PUFRn | PUOD | PUPUP | PUDN | PUIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PU0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pu_instance = TSB_PU;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_U) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PU0 -*/
    /* Use reset value.  */
    /*- PU1 -*/
    /* Use reset value.  */
    /*- PU2 -*/
    /* Use reset value.  */
    /*- PU3 -*/
    /* Use reset value.  */
    /*- PU4 -*/
    /* Use reset value.  */
    /*- PU5 -*/
    /* Use reset value.  */
    /*- PU6 -*/
    /* Use reset value.  */
    /*- PU7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port V]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PVDATA | PVCR | PVFRn | PVOD | PVPUP | PVDN | PVIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PV0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pv_instance = TSB_PV;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_V) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PV0 -*/
    /* Use reset value.  */
    /*- PV1 -*/
    /* Use reset value.  */
    /*- PV2 -*/
    /* Use reset value.  */
    /*- PV3 -*/
    /* Use reset value.  */
    /*- PV4 -*/
    /* Use reset value.  */
    /*- PV5 -*/
    /* Use reset value.  */
    /*- PV6 -*/
    /* Use reset value.  */
    /*- PV6 -*/
    /* Use reset value.  */
    /*- PV7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port W]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PWDATA | PWCR | PWFRn | PWOD | PWPUP | PWDN | PWIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PW0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pw_instance = TSB_PW;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_W) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PW0 -*/
    /* Use reset value.  */
    /*- PW1 -*/
    /* Use reset value.  */
    /*- PW2 -*/
    /* Use reset value.  */
    /*- PW3 -*/
    /* Use reset value.  */
    /*- PW4 -*/
    /* Use reset value.  */
    /*- PW5 -*/
    /* Use reset value.  */
    /*- PW6 -*/
    /* Use reset value.  */
    /*- PW7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port Y]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PYDATA | PYCR | PYFRn | PYOD | PYPUP | PYDN | PYIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PY0  | X1        | O   | 0      | N/A  | N/A   | N/A  | N/A   | 0    | 0     */
    /*  PY1  | X2        | I   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PY2  | XT1       | I   | 0      | N/A  | N/A   | 0    | 0     | 0    | 0     */
    /*  PY3  | XT2       | O   | 0      | 1    | N/A   | 0    | 0     | 0    | 0     */
    /*  PY4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | N/A   */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_py_instance = TSB_PY;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_Y) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PY0 -*/
    if (gpio_func(&port, GPIO_PORT_Y, GPIO_PORT_0, (uint32_t)GPIO_PY0_X1, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PY1 -*/
    if (gpio_func(&port, GPIO_PORT_Y, GPIO_PORT_1, (uint32_t)GPIO_PY1_X2, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PY2 -*/
    if (gpio_func(&port, GPIO_PORT_Y, GPIO_PORT_2, (uint32_t)GPIO_PY2_XT1, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PY3 -*/
    if (gpio_func(&port, GPIO_PORT_Y, GPIO_PORT_3, (uint32_t)GPIO_PY3_XT2, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PY4 -*/
    /* Use reset value.  */
}

/*--------------------------------------------------*/
/** 
  * @brief  Port Finalize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void port_finalize(void)
{
    /*======================*/
    /*  [Port A]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_A) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pa_instance = BSP_NULL;
    /*======================*/
    /*  [Port B]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_B) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pb_instance = BSP_NULL;
    /*======================*/
    /*  [Port C]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_C) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pc_instance = BSP_NULL;
    /*======================*/
    /*  [Port D]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_D) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pd_instance = BSP_NULL;
    /*======================*/
    /*  [Port E]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_E) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pe_instance = BSP_NULL;
    /*======================*/
    /*  [Port F]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_F) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pf_instance = BSP_NULL;
    /*======================*/
    /*  [Port G]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_G) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pg_instance = BSP_NULL;
    /*======================*/
    /*  [Port H]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_H) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_ph_instance = BSP_NULL;
    /*======================*/
    /*  [Port J]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_J) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pj_instance = BSP_NULL;
    /*======================*/
    /*  [Port K]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_K) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pk_instance = BSP_NULL;
    /*======================*/
    /*  [Port L]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_L) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pl_instance = BSP_NULL;
    /*======================*/
    /*  [Port M]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_M) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pm_instance = BSP_NULL;
    /*======================*/
    /*  [Port N]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_N) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pn_instance = BSP_NULL;
    /*======================*/
    /*  [Port P]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_P) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pp_instance = BSP_NULL;
    /*======================*/
    /*  [Port R]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_R) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pr_instance = BSP_NULL;
    /*======================*/
    /*  [Port T]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_T) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pt_instance = BSP_NULL;
    /*======================*/
    /*  [Port U]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_U) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pu_instance = BSP_NULL;
    /*======================*/
    /*  [Port V]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_V) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pv_instance = BSP_NULL;
    /*======================*/
    /*  [Port W]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_W) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pw_instance = BSP_NULL;
    /*======================*/
    /*  [Port V]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_Y) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_py_instance = BSP_NULL;
}

/*--------------------------------------------------*/
/** 
  * @brief  External LOSC Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void external_losc_enable(void)
{
    uint32_t work;

    /* [CGWUPLCR]<WUPTL> :Warm up time        */
    /*-----------------------------------------------*/
    /* "1"counter  (s) = 1 / ELOSC                   */
    /* "1"counter (us) = (10^6) / ELOSC              */
    /* "x"counter (us) = time                        */
    /*-----------------------------------------------*/
    /* x : time = 1 : (10^6) / ELOSC                 */
    /*-----------------------------------------------*/
    {
        uint64_t x = (uint64_t)((uint64_t)(ELOSC_CFG_WARM_UP_TIME) * (uint64_t)(ELOSC_CFG_CLOCK));
        x = (uint64_t)(x / (uint64_t)(1000000));
        if (x > (uint64_t)(0x7FFFF))
        {
            /* invalid value */
        }
        work = (uint32_t)x;
    }
    work  &= (uint32_t)(0xFFFFFFF0);
    work <<= 8;
    TSB_CG->WUPLCR = work;
    /* [RLMLOSCCR]<XTEN> :LOSC Enable         */
    TSB_RLM->LOSCCR = RLMLOSCCR_XTEN_RW_ENABLE;
    /* [CGWUPLCR]<WULON> :Enable              */
    work = (uint32_t)(TSB_CG->WUPLCR & CGWUPLCR_WUPTL_HIGH_MASK);
    TSB_CG->WUPLCR = (uint32_t)(work | CGWUPLCR_WULON_W_ENABLE);
    /* [CGWUPLCR]<WULEF> :Read(wait for warm-up) */
    while ((TSB_CG->WUPLCR & CGWUPLCR_WULEF_MASK) != CGWUPLCR_WULEF_R_DONE)
    {
        /* no processing */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  External LOSC Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void external_losc_disable(void)
{
    /* RLMLOSCCR:<XTEN=0>   */
    TSB_RLM->LOSCCR = RLMLOSCCR_XTEN_RW_DISABLE;
}

/**
 *  @}
 */ /* End of group TMPM4G9_DEVE_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TMPM4G9_DEVE_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_initialize(void)
{
    /*--- Fsys Enable      ---*/
    fsys_all_enable();
    /*--- ADC Clock Enable ---*/
    adc_clock_enable();
    /*--- EMT Trace Clock Enable ---*/
    trace_clock_enable();
    /*--- Port             ---*/
    port_initialize();
    /*--- INT Control Register Enable ---*/
    int_ctr_reg_enable();
    /*--- ELOSC Enable     ---*/
    external_losc_enable();
}

/*--------------------------------------------------*/
/** 
  * @brief  Finalize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_finalize(void)
{
    /*--- ELOSC Disable     ---*/
    external_losc_disable();
    /*--- Port              ---*/
    port_finalize();
    /*--- ADC Clock Disabl ---*/
    adc_clock_disable();
    /*--- EMT Trace Clock Disabl ---*/
    trace_clock_disable();
    /*--- INT Control Register Disable ---*/
    int_ctr_reg_disable();
    /*--- Fsys Disable      ---*/
    fsys_all_disable();
}

/*--------------------------------------------------*/
/** 
  * @brief  Port Driver Instance Addess Get Function.
  * @param  -
  * @return Port Driver Instance Addess.
  * @retval != Null :Valid Value.
  * @note   -
  */
/*--------------------------------------------------*/
gpio_t *bsp_get_gpio_instance_address(void)
{
    return (&port);
}

/*--------------------------------------------------*/
/** 
  * @brief  Port LED GPIO Get Function.
  * @param  led     :Port LED.
  * @param  p_group :GPIO Group Destination Address. \n
  *                 :Value is @ref gpio_gr_t.
  * @param  p_num   :GPIO Number Destination Address.\n
  *                 :Value is @ref gpio_num_t.
  * @return -
  * @retval -
  * @note   You can read the Port LED GPIO Group and GPIO Num.
  */
/*--------------------------------------------------*/
void bsp_get_gpio_led(BSPLed led, uint32_t *p_group, uint32_t *p_num)
{
    *p_group = ledTbl[(uint32_t)led][0];
    *p_num   = ledTbl[(uint32_t)led][1];
}

/*--------------------------------------------------*/
/** 
  * @brief  Port LED GPIO Group Get Function.
  * @param  led     :Port LED.
  * @return Port LED GPIO Group. \n
  *         Value is @ref gpio_gr_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_gpio_group_led(BSPLed led)
{
    return (ledTbl[(uint32_t)led][0]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Port LED GPIO Number Get Function.
  * @param  led     :Port LED.
  * @return Port LED GPIO Number. \n
  *         Value is @ref gpio_num_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_gpio_num_led(BSPLed led)
{
    return (ledTbl[(uint32_t)led][1]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Push SW GPIO Get Function.
  * @param  psw     :Push SW.
  * @param  p_group :GPIO Group Destination Address. \n
  *                 :Value is @ref gpio_gr_t.
  * @param  p_num   :GPIO Number Destination Address. \n
  *                 :Value is @ref gpio_num_t.
  * @return -
  * @retval -
  * @note   You can read the Push SW GPIO Group and GPIO Num.
  */
/*--------------------------------------------------*/
void bsp_get_gpio_psw(BSPPsw psw, uint32_t *p_group, uint32_t *p_num)
{
    *p_group = pswTbl[(uint32_t)psw][0];
    *p_num   = pswTbl[(uint32_t)psw][1];
}

/*--------------------------------------------------*/
/** 
  * @brief  Push SW GPIO Group Get Function.
  * @param  psw :Push SW.
  * @return Push SW GPIO Group. \n
  *         Value is @ref gpio_gr_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_gpio_group_psw(BSPPsw psw)
{
    return (pswTbl[(uint32_t)psw][0]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Push SW GPIO Number Get Function.
  * @param  psw :Push SW.
  * @return Push SW GPIO Number. \n
  *         Value is @ref gpio_num_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_gpio_num_psw(BSPPsw psw)
{
    return (pswTbl[(uint32_t)psw][1]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Channel Get Function.
  * @param  uart :USB UART.
  * @return UART Channel.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_tx_ch(BSPUsbUart uart)
{
    return (usbUartTxTbl[(uint32_t)uart][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Channel Get Function.
  * @param  uart :USB UART
  * @return UART Channel.
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_rx_ch(BSPUsbUart uart)
{
    return (usbUartRxTbl[(uint32_t)uart][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Interrupt Number Get Function.
  * @param  uart :USB UART.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_usb_uart_tx_nvic(BSPUsbUart uart)
{
    IRQn_Type type;
    switch(uart){
        case BSP_USB_UART_0:
            type = INTUART0TX_IRQn;
            break;
        case BSP_USB_UART_1:
            type = INTUART1TX_IRQn;
            break;
        case BSP_USB_UART_2:
            type = INTUART2TX_IRQn;
            break;
        case BSP_USB_UART_3:
            type = INTUART3TX_IRQn;
            break;
        case BSP_USB_UART_4:
            type = INTUART4TX_IRQn;
            break;
        case BSP_USB_UART_5:
            type = INTUART5TX_IRQn;
            break;
        default:
            break;
    }
    return (type);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Interrupt Number Get Function.
  * @param  uart :USB UART.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_usb_uart_rx_nvic(BSPUsbUart uart)
{
    IRQn_Type type;
    switch(uart){
        case BSP_USB_UART_0:
            type = INTUART0RX_IRQn;
            break;
        case BSP_USB_UART_1:
            type = INTUART1RX_IRQn;
            break;
        case BSP_USB_UART_2:
            type = INTUART2RX_IRQn;
            break;
        case BSP_USB_UART_3:
            type = INTUART3RX_IRQn;
            break;
        case BSP_USB_UART_4:
            type = INTUART4RX_IRQn;
            break;
        case BSP_USB_UART_5:
            type = INTUART5RX_IRQn;
            break;
        default:
            break;
    }
    return (type);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Interrupt Number Get Function.
  * @param  uart :USB UART.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Error
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_usb_uart_err_nvic(BSPUsbUart uart)
{
    IRQn_Type type;
    switch(uart){
        case BSP_USB_UART_0:
            type = INTUART0ERR_IRQn;
            break;
        case BSP_USB_UART_1:
            type = INTUART1ERR_IRQn;
            break;
        case BSP_USB_UART_2:
            type = INTUART2ERR_IRQn;
            break;
        case BSP_USB_UART_3:
            type = INTUART3ERR_IRQn;
            break;
        case BSP_USB_UART_4:
            type = INTUART4ERR_IRQn;
            break;
        case BSP_USB_UART_5:
            type = INTUART5ERR_IRQn;
            break;
        default:
            break;
    }
    return (type);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART DMA Channel Get Function.
  * @param  uart :USB UART.
  * @return DMA Channel.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_tx_dma_ch(BSPUsbUart uart)
{
    return (usbUartTxTbl[(uint32_t)uart][3]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART DMA Channel Get Function.
  * @param  uart :USB UART.
  * @return DMA Channel.
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_rx_dma_ch(BSPUsbUart uart)
{
    return (usbUartRxTbl[(uint32_t)uart][3]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Baudrate Get Function.
  * @param  uart :USB UART.
  * @return Baudrate(bps).
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_tx_baudrate(BSPUsbUart uart)
{
    return (usbUartTxTbl[(uint32_t)uart][4]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Baudrate Get Function.
  * @param  uart :USB UART.
  * @return Baudrate(bps).
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_rx_baudrate(BSPUsbUart uart)
{
    return (usbUartRxTbl[(uint32_t)uart][4]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer Get Function.
  * @param  timer :Timer.
  * @param  p_ch  :Timer Channel Destination Address.
  * @param  p_type:Timer Type Destination Address. \n
  *               :Value is @ref t32_type_t.
  * @return -
  * @retval -
  * @note   You can read the 1ms Timer Channel and Type.
  */
/*--------------------------------------------------*/
void bsp_get_timer(BSPTimer timer, uint32_t *p_ch, uint32_t *p_type)
{
    *p_ch   = timerTbl[(uint32_t)timer].ch;
    *p_type = timerTbl[(uint32_t)timer].type;
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer Channel Get Function.
  * @param  timer :Timer.
  * @return Timer Channel.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_timer_ch(BSPTimer timer)
{
    return (timerTbl[(uint32_t)timer].ch);
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer Type Get Function.
  * @param  timer :Timer.
  * @return Timer Type. \n
  *         Value is @ref t32_type_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_timer_type(BSPTimer timer)
{
    return (timerTbl[(uint32_t)timer].type);
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer Interrupt Number Get Function.
  * @param  timer :Timer.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_timer_nvic(BSPTimer timer)
{
    return (timerTbl[(uint32_t)timer].irq);
}

/*--------------------------------------------------*/
/** 
  * @brief  Output Pulse Channel Get Function.
  * @param  pulse :Output Pulse.
  * @return Output Pulse Channel.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_output_pulse_ch(BSPOutputPulse pulse)
{
    return (pulseOutTbl[(uint32_t)pulse].ch);
}

/*--------------------------------------------------*/
/** 
  * @brief  Output Pulse Type Get Function.
  * @param  pulse :Output Pulse.
  * @return Output Pulse Type. \n
  *         Value is @ref t32_type_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_output_pulse_type(BSPOutputPulse pulse)
{
    return (pulseOutTbl[(uint32_t)pulse].type);
}

/*--------------------------------------------------*/
/** 
  * @brief  Output Pulse Interrupt Number Get Function.
  * @param  pulse :Output Pulse.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_output_pulse_nvic(BSPOutputPulse pulse)
{
    return (pulseOutTbl[(uint32_t)pulse].irq);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Channel Get Function.
  * @param  tspi :TSPI.
  * @return TSPI Channel.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_tspi_tx_ch(BSPSFlash tspi)
{
    return (tspiTxTbl[(uint32_t)tspi][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI DMA Channel Get Function.
  * @param  tspi :TSPI.
  * @return TSPI Channel.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_tspi_tx_dma_ch(BSPSFlash tspi)
{
    return (tspiTxTbl[(uint32_t)tspi][3]);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Channel Get Function.
  * @param  tspi :TSPI.
  * @return TSPI Channel.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_tspi_clk_ch(BSPSFlash tspi)
{
    return (tspiCLKTbl[(uint32_t)tspi][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Channel Get Function.
  * @param  tspi :TSPI
  * @return TSPI Channel.
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
uint32_t bsp_get_tspi_rx_ch(BSPSFlash tspi)
{
    return (tspiRxTbl[(uint32_t)tspi][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Channel Get Function.
  * @param  tspi :TSPI
  * @return TSPI Channel.
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
uint32_t bsp_get_tspi_rx_dma_ch(BSPSFlash tspi)
{
    return (tspiRxTbl[(uint32_t)tspi][3]);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Interrupt Number Get Function.
  * @param  tspi :TSPI.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_tspi_tx_nvic(BSPSFlash tspi)
{
    IRQn_Type type;
    switch(tspi){
        case BSP_SFLASH_0:
            type = INTT0TX_IRQn;
            break;
        case BSP_SFLASH_1:
            type = INTT1TX_IRQn;
            break;
        case BSP_SFLASH_2:
            type = INTT2TX_IRQn;
            break;
        case BSP_SFLASH_3:
            type = INTT3TX_IRQn;
            break;
        default:
            break;
    }
    return (type);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Interrupt Number Get Function.
  * @param  tspi :TSPI.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_tspi_rx_nvic(BSPSFlash tspi)
{
    IRQn_Type type;
    switch(tspi){
        case BSP_SFLASH_0:
            type = INTT0RX_IRQn;
            break;
        case BSP_SFLASH_1:
            type = INTT1RX_IRQn;
            break;
        case BSP_SFLASH_2:
            type = INTT2RX_IRQn;
            break;
        case BSP_SFLASH_3:
            type = INTT3RX_IRQn;
            break;
        default:
            break;
    }
    return (type);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Interrupt Number Get Function.
  * @param  tspi :TSPI.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Error
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_tspi_tx_err_nvic(BSPSFlash tspi)
{
    IRQn_Type type;
    switch(tspi){
        case BSP_SFLASH_0:
            type = INTT0ERR_IRQn;
            break;
        case BSP_SFLASH_1:
            type = INTT1ERR_IRQn;
            break;
        case BSP_SFLASH_2:
            type = INTT2ERR_IRQn;
            break;
        case BSP_SFLASH_3:
            type = INTT3ERR_IRQn;
            break;
        default:
            break;
    }
    return (type);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Interrupt Number Get Function.
  * @param  tspi :TSPI.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Error
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_tspi_rx_err_nvic(BSPSFlash tspi)
{
    IRQn_Type type;
    switch(tspi){
        case BSP_SFLASH_0:
            type = INTT1ERR_IRQn;
            break;
        case BSP_SFLASH_1:
            type = INTT1ERR_IRQn;
            break;
        case BSP_SFLASH_2:
            type = INTT2ERR_IRQn;
            break;
        case BSP_SFLASH_3:
            type = INTT3ERR_IRQn;
            break;
        default:
            break;
    }
    return (type);
}

/*--------------------------------------------------*/
/** 
  * @brief  Thermistors ADIN Get Function
  * @param  thermistors :Thermistors.
  * @return Thermistors ADIN.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_thermistors_adin(BSPThermistors thermistors)
{
    return (thermistorsTbl[(uint32_t)thermistors][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Thermistors ADC Channel Get Function
  * @param  thermistors :Thermistors.
  * @return Thermistors ADC Channel.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_thermistors_ch(BSPThermistors thermistors)
{
    return (thermistorsTbl[(uint32_t)thermistors][3]);
}
/*--------------------------------------------------*/
/** 
  * @brief  Thermistors ADC DMA Channel Get Function
  * @param  thermistors :Thermistors.
  * @return Thermistors ADC DMA Channel.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_thermistors_dma_ch(BSPThermistors thermistors)
{
    return (thermistorsTbl[(uint32_t)thermistors][4]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Celsius.
  * @param  thermistors :Thermistors.
  * @param  cnt         :AD value.
  * @param  p_degC      :Celsius. Destination address.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Error
  */
/*--------------------------------------------------*/
TXZ_Result bsp_get_thermistors_degC(BSPThermistors thermistors, uint32_t cnt, int32_t *p_degC)
{
    TXZ_Result res = TXZ_ERROR;

    switch (thermistors)
    {
    case BSP_THERMISTORS_0:
    case BSP_THERMISTORS_1:
      {
            uint32_t i;
            uint32_t num = (sizeof(NCP15XV103J03RC) / sizeof(NCP15XV103J03RC[0]));

            for (i=0; (i<num) && (res==TXZ_ERROR); i++)
            {
                /* Compare to AD-low & AD-hgh. */
                if (((uint32_t)NCP15XV103J03RC[i][1] <= cnt) && (cnt <= (uint32_t)NCP15XV103J03RC[i][3]))
                {
                    *p_degC = NCP15XV103J03RC[i][0];
                    res     = TXZ_SUCCESS;
                }
            }
        }
        break;
    default:
        /* no processing */
        break;
    }

    return (res);
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ RTC Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_rtc_enable(void)
{
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 0      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 010    */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC49 = (uint8_t)0x05;
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ RTC Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_rtc_disable(void)
{
    /*--- IAIMC49 Disable ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 0      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 000    */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC49 = (uint8_t)(0x00);
    /*--- IAIMC49 INTxCLR ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 010    */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC49 = (uint8_t)(0xC4);
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ RMC Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_rmc_enable(void)
{
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC56 = (uint8_t)0x01;
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ RMC Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_rmc_disable(void)
{
    /*--- IAIMC56 Disable ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC56 = (uint8_t)0x00;
    /*--- IAIMC56 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC56 = (uint8_t)0x41;
}
/*--------------------------------------------------*/
/** 
  * @brief  IRQ External Interrupt Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_exint_enable(void)
{
    /*--- IAIMC08: Enable ---*/
    /*-------------------------------*/
    /* Bit | Bit Symbol   | Value    */
    /*-------------------------------*/
    /* 7   | INTNCLR      | 0        */
    /* 6   | INTPCLR      | 0        */
    /* 5   | NTNFLG       | -        */
    /* 4   | INTPFLG      | -        */
    /* 3:1 | INTMODE[2:0] | 010      */
    /* 0   | INTEN        | 1        */
    /*-------------------------------*/
    TSB_IA->IMC08 = (uint8_t)(0x05);
}
/*--------------------------------------------------*/
/** 
  * @brief  IRQ External Interrupt Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_exint_disable(void)
{
    /*--- IAIMC08 Disable ---*/
    /*--- IAIMC08: INTxCLR ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 000    */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC08 = (uint8_t)(0xC0);
}
/*--------------------------------------------------*/
/** 
  * @brief  External Interrupt Number Get Function.
  * @param  -
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_exint_nvic(void)
{
    return (INT04_IRQn);
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ External Interrupt (DNF) Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_dnf_enable(void)
{
    /*--- IAIMC08: Enable ---*/
    /*-------------------------------*/
    /* Bit | Bit Symbol   | Value    */
    /*-------------------------------*/
    /* 7   | INTNCLR      | 0        */
    /* 6   | INTPCLR      | 0        */
    /* 5   | NTNFLG       | -        */
    /* 4   | INTPFLG      | -        */
    /* 3:1 | INTMODE[2:0] | 010      */
    /* 0   | INTEN        | 1        */
    /*-------------------------------*/
    TSB_IA->IMC08 = (uint8_t)(0x05);
}
/*--------------------------------------------------*/
/** 
  * @brief  IRQ External Interrupt (DNF) Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_dnf_disable(void)
{
    /*--- IAIMC08 Disable ---*/
    /*--- IAIMC08: INTxCLR ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 000    */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC08 = (uint8_t)(0xC0);
}
/*--------------------------------------------------*/
/** 
  * @brief  DNF Register Get Function.
  * @param  dnf :DNF.
  * @return DNF Register.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TSB_DNF_TypeDef * bsp_get_dnf_reg(BSPDnf dnf)
{
    uint32_t int_num = dnfTbl[(uint32_t)dnf][0];
    if (int_num < 16)
    {
        return (TSB_DNFA);
    }else
    {    
        return (TSB_DNFB);
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  DNF InterruptNumber Get Function.
  * @param  dnf :DNF.
  * @return InterruptNumber Noise Filter Enable Value. \n
  *         Value is @ref t32_type_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_dnf_int_num(BSPDnf dnf)
{
    uint32_t num = 0x00000001;
    uint32_t int_num = dnfTbl[(uint32_t)dnf][0];
    if (int_num < 16)
    {
        num = (num << int_num);
    }else
    {
        int_num = int_num -16;
        num = (num << int_num);
    }        
    return (num);
}
/*--------------------------------------------------*/
/** 
  * @brief  DNF Interrupt Number Get Function.
  * @param  dnf :DNF.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_dnf_nvic(BSPDnf dnf)
{
    return (INT04_IRQn);
}

/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_systick(void)
{
    
}
/*--------------------------------------------------*/
/** 
  * @brief  Push SW State Change.
  * @param  psw :Push SW.
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_psw(BSPPsw psw)
{
    
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_usb_uart_rx(BSPUsbUart uart)
{
    
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_usb_uart_tx(BSPUsbUart uart)
{
    
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_usb_uart_err(BSPUsbUart uart)
{
    
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_timer(BSPTimer timer)
{
    
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_rtc(void)
{
    
}
/*--------------------------------------------------*/
/** 
  * @brief  RMC IRQ.
  * @param  status
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_rmc(uint32_t status)
{
    
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC SGL IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   Please overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_adc_sgl(void)
{
    
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC CNT IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   lease overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_adc_cnt(void)
{
    
}
/*--------------------------------------------------*/
/** 
  * @brief  External Interrupt  IRQ.
  * @param  exip
  * @return -
  * @retval -
  * @note   lease overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_exint(BSPExInt exip)
{
    
}
/*--------------------------------------------------*/
/** 
  * @brief  TSPI reception IRQ
  * @param  -
  * @return -
  * @retval -
  * @note   lease overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_sflash_rx(BSPSFlash sflash)
{
  
}
/*--------------------------------------------------*/
/** 
  * @brief  TSPI transmission IRQ
  * @param  -
  * @return -
  * @retval -
  * @note   lease overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_sflash_tx(BSPSFlash sflash)
{
  
}
/*--------------------------------------------------*/
/** 
  * @brief  TSPI error IRQ
  * @param  -
  * @return -
  * @retval -
  * @note   lease overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_sflash_err(BSPSFlash sflash)
{
  
}
/*--------------------------------------------------*/
/** 
  * @brief  HDMA Done IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   lease overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_dmac_done(void)
{
  
}

/*--------------------------------------------------*/
/** 
  * @brief  HDMA error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   lease overwrite in user application if you need this function.
  */
/*--------------------------------------------------*/
__weak void irq_dmac_err(void)
{
  
}

/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    irq_systick();
}

/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt pin 00a/00b
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT01 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   1: Interrupt pin 01a/00b
  */
/*--------------------------------------------------*/
void INT01_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT02 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   2: Interrupt pin 02a/00b
  */
/*--------------------------------------------------*/
void INT02_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT03 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   3: Interrupt pin 03a/03b
  */
/*--------------------------------------------------*/
void INT03_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT04 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   4: Interrupt pin 04a/04b
  */
/*--------------------------------------------------*/
void INT04_IRQHandler(void)
{
    /*--- IAIMC08 Disable ---*/
    /*--- IAIMC08: INTxCLR ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 000    */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC08 = (uint8_t)(0xC0);
    if(exi==BSP_EXINT_1){ exi =BSP_EXINT_0; }
    else { exi =BSP_EXINT_1; }
    /*--- Handler Call     ---*/
    irq_exint(exi);
}

/*--------------------------------------------------*/
/** 
  * @brief  INT05 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   5: Interrupt pin 05a/05b
  */
/*--------------------------------------------------*/
void INT05_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT06 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   6: Interrupt pin 06a/06b
  */
/*--------------------------------------------------*/
void INT06_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT07 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   7: Interrupt pin 07a/07b
  */
/*--------------------------------------------------*/
void INT07_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT08 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   8: Interrupt pin 08a/08b
  */
/*--------------------------------------------------*/
void INT08_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT09 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   9: Interrupt pin 09a/09b
  */
/*--------------------------------------------------*/
void INT09_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT10 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   10: Interrupt pin 10a/10b
  */
/*--------------------------------------------------*/
void INT10_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT11 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   11: Interrupt pin 11a/11b
  */
/*--------------------------------------------------*/
void INT11_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT12 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   12: Interrupt pin 12a/12b
  */
/*--------------------------------------------------*/
void INT12_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT13 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   13: Interrupt pin 13a/13b
  */
/*--------------------------------------------------*/
void INT13_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT14 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   14: Interrupt pin 14a/14b
  */
/*--------------------------------------------------*/
void INT14_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT15 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   15: Interrupt pin 15a/15b
  */
/*--------------------------------------------------*/
void INT15_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   16: Real time clock(XHz) interrupt
  */
/*--------------------------------------------------*/
void INTRTC_IRQHandler(void)
{
    /*--- IAIMC17: INTxCLR ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 010    */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC17 = (uint8_t)(0xC5);
    /*--- Handler Call     ---*/
    irq_rtc();
}

/*--------------------------------------------------*/
/** 
  * @brief  CEC0RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17: CEC reception interrupt (channel 0)
  */
/*--------------------------------------------------*/
void INTCEC0RX_IRQHandler(void)
{
#ifdef CEC_USE
    /*--- IAIMC51 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | -            | -      */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC50 = (uint8_t)0x41;
    /*--- Handler Call     ---*/
    irq_cec_rx();
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  CEC0TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   18: CEC transmission interrupt (channel 0)
  */
/*--------------------------------------------------*/
void INTCEC0TX_IRQHandler(void)
{
#ifdef CEC_USE
    /*--- IAIMC51 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | -            | -      */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC51 = (uint8_t)0x41;
    /*--- Handler Call     ---*/
    irq_cec_tx();
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief   ISDA IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   19: Interval Sensing Detector Interrupt (Unit A)
  */
/*--------------------------------------------------*/
void  INTISDA_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  ISDB IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   20: Interval Sensing Detector Interrupt (Unit B)
  */
/*--------------------------------------------------*/
void INTISDB_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  ISDC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   21: Interval Sensing Detector Interrupt (Unit C)
  */
/*--------------------------------------------------*/
void INTISDC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  RMC0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   22: Remote control reception interrupt 0
  */
/*--------------------------------------------------*/
void INTRMC0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  RMC1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   23: Remote control reception interrupt 1
  */
/*--------------------------------------------------*/
void INTRMC1_IRQHandler(void)
{
    uint32_t status;
    /*------------------------------*/
    /*  RMC Get Status              */
    /*------------------------------*/
    status =  TSB_RMC1->RSTAT;
    /*--- IAIMC55 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | -            | -      */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC56 = (uint8_t)0x41;
    /*--- Handler Call     ---*/
    irq_rmc(status);
}

/*--------------------------------------------------*/
/** 
  * @brief  LTTMR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   24: Long Term Timer Interrupt
  */
/*--------------------------------------------------*/
void INTLTTMR0_IRQHandler(void)
{
#ifdef LTTMR_USE
    /*--- IAIMC57 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | -            | -      */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC57 = (uint8_t)0x41;
    /*--- Handler Call     ---*/
    irq_lttmr();
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  HDMAATC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   25: HDMA Complete of transfer(Unit A)
  */
/*--------------------------------------------------*/
void INTHDMAATC_IRQHandler(void)
{
    irq_dmac_done();
}

/*--------------------------------------------------*/
/** 
  * @brief  HDMAAERR IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   26: HDMA transfer error(Unit A)
  */
/*--------------------------------------------------*/
void INTHDMAAERR_IRQHandler(void)
{
    irq_dmac_err();
}

/*--------------------------------------------------*/
/** 
  * @brief  HDMABTC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   27: HDMA end of transfer(Unit B)
  */
/*--------------------------------------------------*/
void INTHDMABTC_IRQHandler(void)
{
    /* no process */
}
/*--------------------------------------------------*/
/** 
  * @brief  HDMABERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   28: HDMA transfer error(Unit B)
  */
/*--------------------------------------------------*/
void INTHDMABERR_IRQHandler(void)
{
    /* no process */
}
/*--------------------------------------------------*/
/** 
  * @brief  MDMAATC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   29: MDMA Complete of transfer(Unit A)
  */
/*--------------------------------------------------*/
void INTMDMAATC_IRQHandler(void)
{
#ifdef UART_MDMA_TRANSFER_USE
    mdma_irq_end_handler();
#else
  #ifdef FUART_MDMA_TRANSFER_USE
    mdma_irq_end_handler();
  #else
    /* no process */
  #endif
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief   T32A00_A_CT IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   30: T32A00 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void  INTT32A00_A_CT_IRQHandler(void)
{
    /*--- IBIMC094 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | -            | -      */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IB->IMC000 = (uint8_t)0x41;
    irq_timer(BSP_TIMER_1MS);
}

/*--------------------------------------------------*/
/** 
  * @brief   T32A00_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   31: T32A00 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void  INTT32A00_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A01_A_CT IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   32: T32A01 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A01_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A01_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   33: T32A01 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A01_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A02_A_CT IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   34: T32A02 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A02_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A02_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   35: T32A02 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A02_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A03_A_CT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   36: T32A03 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A03_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A03_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   37: T32A03 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A03_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A04_A_CT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   38: T32A04 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A04_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A04_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   40: T32A04 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A04_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A05_A_CT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   40: T32A05 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A05_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A05_B_C01_CPC_IRQHandler
  * @param  -
  * @return -
  * @retval -
  * @note   41: T32A05 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A05_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A06_A_CT IRQHandler 
  * @param  -
  * @return -
  * @retval -
  * @note   42: T32A06 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A06_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A06_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   43: T32A06 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A06_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A07_A_CT IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   44: T32A07 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A07_A_CT_IRQHandler (void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A07_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   45: T32A07 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A07_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A08_A_CT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   46: T32A08 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A08_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A08_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   47: T32A08 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A08_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A09_A_CT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   48: T32A09 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A09_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A09_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   49: T32A09 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A09_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A10_A_CT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   50: T32A10 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A10_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A10_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   51: T32A10 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A10_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A11_A_CT IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   52: T32A11 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A11_A_CT_IRQHandler (void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A11_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   53: T32A11 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A11_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A12_A_CT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   54: T32A12 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A12_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A12_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   55: T32A12 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A12_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A13_A_CT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   56: T32A13 TimerA All Interrupt/Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A13_A_CT_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A13_B_C01_CPC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   57: T32A13 Timer Interrupt B/Capture C0,C1 Interrupt/ Timer Interrupt C
  */
/*--------------------------------------------------*/
void INTT32A13_B_C01_CPC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief   TEMG0 IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   58: PMD0 EMG interrupt
  */
/*--------------------------------------------------*/
void  INTEMG0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  OVV0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   59: PMD0 OVV interrupt
  */
/*--------------------------------------------------*/
void INTOVV0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  PWM0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   60: PMD0 interrupt
  */
/*--------------------------------------------------*/
void INTPWM0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T0RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   61: TSPI/SIO reception (channel 0)
  */
/*--------------------------------------------------*/
void INTT0RX_IRQHandler(void)
{
    irq_sflash_rx(BSP_SFLASH_0);
}

/*--------------------------------------------------*/
/** 
  * @brief  T0TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   62: TSPI/SIO transmit (channel 0)
  */
/*--------------------------------------------------*/
void INTT0TX_IRQHandler(void)
{
    irq_sflash_tx(BSP_SFLASH_0);
}

/*--------------------------------------------------*/
/** 
  * @brief  T0ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   63: TSPI/SIO error (channel 0)
  */
/*--------------------------------------------------*/
void INTT0ERR_IRQHandler(void)
{
    irq_sflash_err(BSP_SFLASH_0);
}

/*--------------------------------------------------*/
/** 
  * @brief  T1RX IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   64: TSPI/SIO reception (channel 1)
  */
/*--------------------------------------------------*/
void INTT1RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T1TX IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   65: TSPI/SIO transmit (channel 1)
  */
/*--------------------------------------------------*/
void INTT1TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T1ERR IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   66: TSPI/SIO error (channel 1)
  */
/*--------------------------------------------------*/
void INTT1ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T2RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   67: TSPI/SIO reception (channel 2)
  */
/*--------------------------------------------------*/
void INTT2RX_IRQHandler(void)
{
    irq_sflash_rx(BSP_SFLASH_2);
}

/*--------------------------------------------------*/
/** 
  * @brief  T2TX IRQHandler
  * @param  -
  * @return -
  * @retval -
  * @note   68: TSPI/SIO transmit (channel 2)
  */
/*--------------------------------------------------*/
void INTT2TX_IRQHandler(void)
{
    irq_sflash_tx(BSP_SFLASH_2);
}

/*--------------------------------------------------*/
/** 
  * @brief  T2ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   69: TSPI/SIO error (channel 2)
  */
/*--------------------------------------------------*/
void INTT2ERR_IRQHandler(void)
{
    irq_sflash_err(BSP_SFLASH_2);
}

/*--------------------------------------------------*/
/** 
  * @brief   T3RX IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   70: TSPI/SIO reception (channel 3)
  */
/*--------------------------------------------------*/
void INTT3RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T3TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   71: TSPI/SIO transmit (channel 3)
  */
/*--------------------------------------------------*/
void INTT3TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T3ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   72: TSPI/SIO error (channel 3)
  */
/*--------------------------------------------------*/
void INTT3ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief   T4RX IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   73: TSPI/SIO reception (channel 4)
  */
/*--------------------------------------------------*/
void INTT4RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T4TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   74: TSPI/SIO transmit (channel 4)
  */
/*--------------------------------------------------*/
void INTT4TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T4ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   75: TSPI/SIO error (channel 4)
  */
/*--------------------------------------------------*/
void INTT4ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief   T5RX IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   76: TSPI/SIO reception (channel 5)
  */
/*--------------------------------------------------*/
void INTT5RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T5TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   76: TSPI/SIO transmit (channel 5)
  */
/*--------------------------------------------------*/
void INTT5TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T5ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   78: TSPI/SIO error (channel 5)
  */
/*--------------------------------------------------*/
void INTT5ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief   T6RX IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   79: TSPI/SIO reception (channel 6)
  */
/*--------------------------------------------------*/
void INTT6RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T6TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   80: TSPI/SIO transmit (channel 6)
  */
/*--------------------------------------------------*/
void INTT6TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T6ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   81: TSPI/SIO error (channel 6)
  */
/*--------------------------------------------------*/
void INTT6ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief   T7RX IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   82: TSPI/SIO reception (channel 7)
  */
/*--------------------------------------------------*/
void INTT7RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T7TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   83: TSPI/SIO transmit (channel 7)
  */
/*--------------------------------------------------*/
void INTT7TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T7ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   84: TSPI/SIO error (channel 7)
  */
/*--------------------------------------------------*/
void INTT7ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief   T8RX IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   85: TSPI/SIO reception (channel 8)
  */
/*--------------------------------------------------*/
void INTT8RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T8TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   86: TSPI/SIO transmit (channel 8)
  */
/*--------------------------------------------------*/
void INTT8TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  T8ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   87: TSPI/SIO error (channel 8)
  */
/*--------------------------------------------------*/
void INTT8ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  SMI0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   88: Serial Memory Interface Interrupt
  */
/*--------------------------------------------------*/
void INTSMI0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART0RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   89: UART reception (channel 0)
  */
/*--------------------------------------------------*/
void INTUART0RX_IRQHandler(void)
{
    irq_usb_uart_rx(BSP_USB_UART_0);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART0TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   90: UART transmit (channel 0)
  */
/*--------------------------------------------------*/
void INTUART0TX_IRQHandler(void)
{
    irq_usb_uart_tx(BSP_USB_UART_0);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART0ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   91: UART error (channel 0)
  */
/*--------------------------------------------------*/
void INTUART0ERR_IRQHandler(void)
{
    irq_usb_uart_err(BSP_USB_UART_0);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART1RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   92: UART reception (channel 1)
  */
/*--------------------------------------------------*/
void INTUART1RX_IRQHandler(void)
{
    irq_usb_uart_rx(BSP_USB_UART_1);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART1TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   93: UART transmit (channel 1)
  */
/*--------------------------------------------------*/
void INTUART1TX_IRQHandler(void)
{
    irq_usb_uart_tx(BSP_USB_UART_1);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART1ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   94: UART error (channel 1)
  */
/*--------------------------------------------------*/
void INTUART1ERR_IRQHandler(void)
{
    irq_usb_uart_err(BSP_USB_UART_1);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART2RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   95: UART reception (channel 2)
  */
/*--------------------------------------------------*/
void INTUART2RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART2TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   96: UART transmit (channel 2)
  */
/*--------------------------------------------------*/
void INTUART2TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART2ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   97: UART error (channel 2)
  */
/*--------------------------------------------------*/
void INTUART2ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART3RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   98: UART reception (channel 3)
  */
/*--------------------------------------------------*/
void INTUART3RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART3TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   99: UART transmit (channel 3)
  */
/*--------------------------------------------------*/
void INTUART3TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART3ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  100: UART error (channel 3)
  */
/*--------------------------------------------------*/
void INTUART3ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART4RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  101: UART reception (channel 4)
  */
/*--------------------------------------------------*/
void INTUART4RX_IRQHandler(void)
{
    irq_usb_uart_rx(BSP_USB_UART_4);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART4TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  102: UART transmit (channel 4)
  */
/*--------------------------------------------------*/
void INTUART4TX_IRQHandler(void)
{
    irq_usb_uart_tx(BSP_USB_UART_4);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART4ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  103: UART error (channel 4)
  */
/*--------------------------------------------------*/
void INTUART4ERR_IRQHandler(void)
{
    irq_usb_uart_err(BSP_USB_UART_4);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART5RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  104: UART reception (channel 5)
  */
/*--------------------------------------------------*/
void INTUART5RX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART5TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  105: UART transmit (channel 5)
  */
/*--------------------------------------------------*/
void INTUART5TX_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  UART5ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  106: UART error (channel 5)
  */
/*--------------------------------------------------*/
void INTUART5ERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  FUART0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  107: FUART Interrupt(channel 0)
  */
/*--------------------------------------------------*/
void INTFUART0_IRQHandler(void)
{
#ifdef FUART_TRANSFER_USE
    fuart_io_irq_handler();
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  FUART1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  108: FUART Interrupt(channel 1)
  */
/*--------------------------------------------------*/
void INTFUART1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  109: I2C0 transmission and reception interrupt
  */
/*--------------------------------------------------*/
void INTI2C0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C0AL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  110: I2C0 arbitration lost interrupt
  */
/*--------------------------------------------------*/
void INTI2C0AL_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C0BF IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  111: I2C0 bus free interrupt
  */
/*--------------------------------------------------*/
void INTI2C0BF_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C0NACK IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  112: I2C0 no ack interrupt
  */
/*--------------------------------------------------*/
void INTI2C0NACK_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  113: I2C1 transmission and reception interrupt
  */
/*--------------------------------------------------*/
void INTI2C1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C1AL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  114: I2C1 arbitration lost interrupt
  */
/*--------------------------------------------------*/
void INTI2C1AL_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C1BF IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  115: I2C1 bus free interrupt
  */
/*--------------------------------------------------*/
void INTI2C1BF_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C1NACK IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  116: I2C1 no ack interrupt
  */
/*--------------------------------------------------*/
void INTI2C1NACK_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C2 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  117: I2C2 transmission and reception interrupt
  */
/*--------------------------------------------------*/
void INTI2C2_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C2AL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  118: I2C2 arbitration lost interrupt
  */
/*--------------------------------------------------*/
void INTI2C2AL_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C2BF IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  119: I2C2 bus free interrupt
  */
/*--------------------------------------------------*/
void INTI2C2BF_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C2NACK IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  120: I2C2 no ack interrupt
  */
/*--------------------------------------------------*/
void INTI2C2NACK_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C3 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  121: I2C3 transmission and reception interrupt
  */
/*--------------------------------------------------*/
void INTI2C3_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C3AL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  122: I2C3 arbitration lost interrupt
  */
/*--------------------------------------------------*/
void INTI2C3AL_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C3BF IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  123: I2C3 bus free interrupt
  */
/*--------------------------------------------------*/
void INTI2C3BF_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C3NACK IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  124: I2C3 no ack interrupt
  */
/*--------------------------------------------------*/
void INTI2C3NACK_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C4 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  125: I2C4 transmission and reception interrupt
  */
/*--------------------------------------------------*/
void INTI2C4_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C4AL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  126: I2C4 arbitration lost interrupt
  */
/*--------------------------------------------------*/
void INTI2C4AL_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C4BF IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  127: I2C4 bus free interrupt
  */
/*--------------------------------------------------*/
void INTI2C4BF_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C4NACK IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note  128: I2C4 no ack interrupt
  */
/*--------------------------------------------------*/
void INTI2C4NACK_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADACP0_IRQHandler
  * @param  -
  * @return -
  * @retval -
  * @note   129: ADC conversion monitoring function interrupt 0
  */
/*--------------------------------------------------*/
void INTADACP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADACP1_IRQHandler
  * @param  -
  * @return -
  * @retval -
  * @note   130: ADC conversion monitoring function interrupt 1
  */
/*--------------------------------------------------*/
void INTADACP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADATRG IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   131: ADC conversion triggered by General purpose is finished
  */
/*--------------------------------------------------*/
void INTADATRG_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADASGL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   132: ADC conversion triggered by Single program is finished1
  */
/*--------------------------------------------------*/
void INTADASGL_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADACNT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   133: ADC conversion triggered by Continuity program is finished
  */
/*--------------------------------------------------*/
void INTADACNT_IRQHandler(void)
{
#ifdef ADC_USE
    adc_continuity_irq_handler();
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  ADAHP IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   134: ADC High Priority AD conversion interrupt
  */
/*--------------------------------------------------*/
void INTADAHP_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  FLDRDY IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   135: Data FLASH Ready interrupt
  */
/*--------------------------------------------------*/
void INTFLDRDY_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  FLCRDY0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   136: Code FLASH Area0/1 Ready interrupt
  */
/*--------------------------------------------------*/
void INTFLCRDY0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  FLCRDY1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   137: Code FLASH Area2 Ready interrupt
  */
/*--------------------------------------------------*/
void INTFLCRDY1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  -
  * @param  -
  * @return -
  * @retval -
  * @note   138: -
  */
/*--------------------------------------------------*/


/*--------------------------------------------------*/
/** 
  * @brief  MDMAABERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   139: MDMA bus error(Unit A)
  */
/*--------------------------------------------------*/
void INTMDMAABERR_IRQHandler(void)
{
#ifdef UART_MDMA_TRANSFER_USE
    mdma_irq_error_handler(MDMA_ERROR_BUS_ERROR);
#else
  #ifdef FUART_MDMA_TRANSFER_USE
    mdma_irq_error_handler(MDMA_ERROR_BUS_ERROR);
  #else
    /* no process */
  #endif
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  MDMAADERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   140: MDMA descriptor error(Unit A)
  */
/*--------------------------------------------------*/
void INTMDMAADERR_IRQHandler(void)
{
#ifdef UART_MDMA_TRANSFER_USE
    mdma_irq_error_handler(MDMA_ERROR_DESCRIPTOR_ERROR);
#else
  #ifdef FUART_MDMA_TRANSFER_USE
    mdma_irq_error_handler(MDMA_ERROR_DESCRIPTOR_ERROR);
  #else
    /* no process */
  #endif
#endif
}

/**
 *  @}
 */ /* End of group TMPM4G9_DEVE_Exported_functions */

/**
 *  @}
 */ /* End of group TMPM4G9_DEVE */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__BSP_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */

