/** 
 *******************************************************************************
 * @file    main.c
 * @brief   EBIF SRAM Sample Application.
 * @version V1.0.0.0
 * $Date:: 2017-09-05 17:22:52 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "ebif_wapper.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup EBIF_SRAM EBIF_SRAM Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_macro EBIF_SRAM Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_define EBIF_SRAM Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_define EBIF_SRAM Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)        /*!< Null Pointer.            */
#define EBIF_NULL       ((void *)0)        /*!< Null Pointer.            */
#define STRING_NUM      (0x10)              /*!< String Size. */
#define SRAM_BASE       (uint32_t)(0x61000000)        /*!< Access location address. */
/*----------------------*/
/*  UART Setting        */
/*----------------------*/
/** 
 *  @defgroup UART_Setting UART Setting
 *  @{
 */
#define CFG_UART_BAUDRATE               ((uint32_t)115200)  /*!< Boudrate(bps) */
/** 
 *  @}
 */ /* End of group UART_Setting */

#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */


/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_define EBIF_SRAM Private Define
 *  @{
 */

/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_COMMAND_PARSE,     /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_ERROR              /*!< Error.                       */
} Processing;

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_typedef EBIF_SRAM Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_variables EBIF_SRAM Private Variables
 *  @{
 */


static EBIF_setting_t ebif_config;
static CS_setting_t   cs1_config;

static Processing proc;         /*!< Processing mode.        */
static input_t input;           /*!< Input data information. */
static gpio_t port;             /*!< Port driver instance. */

#if     defined ( __CC_ARM )    /* ARM Compiler */
    #pragma arm section zidata = "SRAM1"
#elif   defined ( __ICCARM__ )  /* IAR Compiler */
    #pragma location = "SRAM1"
#endif
static uint8_t write_data[STRING_NUM];  /*!< Write data information. */
/**
 *  @}
 */ /* End of group EBIF_SRAM_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_functions EBIF_SRAM Private Fuctions
 *  @{
 */
static void port_error(void);
static void clear_data(void);
static TXZ_Result application_initialize(void);
static void EBIF_port_initialize(void);
static void EBIF_interface_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void port_error(void)
{
    while(1)
    {
        __NOP();
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i = 0; i < (UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
    
}
/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_1))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = (uint32_t)CFG_UART_BAUDRATE;
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_1);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_1);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_1);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Port Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void EBIF_port_initialize(void)
{
    /*================================================================================*/
    /*  [Port A]                                                                      */
    /*--------------------------------------------------------------------------------*/
    /*  Port | Function   | I/O | PADATA | PACR | PAFRn | PAOD | PAPUP | PADN | PAIE  */
    /*--------------------------------------------------------------------------------*/
    /*  PA0  | EA00       | O   | 0      | 1    | PAFR1 | 0    | 0     | 0    | 0     */
    /*  PA1  | EA01       | O   | 0      | 1    | PAFR1 | 0    | 0     | 0    | 0     */
    /*  PA2  | EA02       | O   | 0      | 1    | PAFR1 | 0    | 0     | 0    | 0     */
    /*  PA3  | EA03       | O   | 0      | 1    | PAFR1 | 0    | 0     | 0    | 0     */
    /*  PA4  | EA04       | O   | 0      | 1    | PAFR1 | 0    | 0     | 0    | 0     */
    /*  PA5  | EA05       | O   | 0      | 1    | PAFR1 | 0    | 0     | 0    | 0     */
    /*  PA6  | EA06       | O   | 0      | 1    | PAFR1 | 0    | 0     | 0    | 0     */
    /*  PA7  | EA07       | O   | 0      | 1    | PAFR1 | 0    | 0     | 0    | 0     */
    /*================================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pa_instance = TSB_PA;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_A) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PA0 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_0, (uint32_t)GPIO_PA0_EA00, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PA1 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_1, (uint32_t)GPIO_PA1_EA01, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PA2 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_2, (uint32_t)GPIO_PA2_EA02, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PA3 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_3, (uint32_t)GPIO_PA3_EA03, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PA4 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_4, (uint32_t)GPIO_PA4_EA04, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PA5 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_5, (uint32_t)GPIO_PA5_EA05, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PA6 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_6, (uint32_t)GPIO_PA6_EA06, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PA7 -*/
    if (gpio_func(&port, GPIO_PORT_A, GPIO_PORT_7, (uint32_t)GPIO_PA7_EA07, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*================================================================================*/
    /*  [Port B]                                                                      */
    /*--------------------------------------------------------------------------------*/
    /*  Port | Function   | I/O | PBDATA | PBCR | PBFRn | PBOD | PBPUP | PBDN | PBIE  */
    /*--------------------------------------------------------------------------------*/
    /*  PB0  | EA08       | O   | 0      | 1    | PBFR1 | 0    | 0     | 0    | 0     */
    /*  PB1  | EA09       | O   | 0      | 1    | PBFR1 | 0    | 0     | 0    | 0     */
    /*  PB2  | EA10       | O   | 0      | 1    | PBFR1 | 0    | 0     | 0    | 0     */
    /*  PB3  | EA11       | O   | 0      | 1    | PBFR1 | 0    | 0     | 0    | 0     */
    /*  PB4  | EA12       | O   | 0      | 1    | PBFR1 | 0    | 0     | 0    | 0     */
    /*  PB5  | EA13       | O   | 0      | 1    | PBFR1 | 0    | 0     | 0    | 0     */
    /*  PB6  | EA14       | O   | 0      | 1    | PBFR1 | 0    | 0     | 0    | 0     */
    /*  PB7  | EA15       | O   | 0      | 1    | PBFR1 | 0    | 0     | 0    | 0     */
    /*================================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pb_instance = TSB_PB;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_B) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PB0 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_0, (uint32_t)GPIO_PB0_EA08, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PB1 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_1, (uint32_t)GPIO_PB1_EA09, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PB2 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_2, (uint32_t)GPIO_PB2_EA10, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PB3 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_3, (uint32_t)GPIO_PB3_EA11, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PB4 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_4, (uint32_t)GPIO_PB4_EA12, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PB5 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_5, (uint32_t)GPIO_PB5_EA13, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PB6 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_6, (uint32_t)GPIO_PB6_EA14, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PB7 -*/
    if (gpio_func(&port, GPIO_PORT_B, GPIO_PORT_7, (uint32_t)GPIO_PB7_EA15, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    
    /*===============================================================================*/
    /*  [Port C]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PCDATA | PCCR | PCFRn | PCOD | PCPUP | PCDN | PCIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PC0  | EA16      | O   | 0      | 1    | PCFR1 | 0    | 0     | 0    | 0     */
    /*  PC1  | EA17      | O   | 0      | 1    | PCFR1 | 0    | 0     | 0    | 0     */
    /*  PC2  | EA18      | O   | 0      | 1    | PCFR1 | 0    | 0     | 0    | 0     */
    /*  PC3  | EA19      | O   | 0      | 1    | PCFR1 | 0    | 0     | 0    | 0     */
    /*  PC4  | EA20      | O   | 0      | 1    | PCFR1 | 0    | 0     | 0    | 0     */
    /*  PC5  | EA21      | O   | 0      | 1    | PCFR1 | 0    | 0     | 0    | 0     */
    /*  PC6  | EA22      | O   | 0      | 1    | PCFR1 | 0    | 0     | 0    | 0     */
    /*  PC7  | EA23      | O   | 0      | 1    | PCFR1 | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pc_instance = TSB_PC;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_C) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PC0 -*/
    if (gpio_func(&port, GPIO_PORT_C, GPIO_PORT_0, (uint32_t)GPIO_PC0_EA16, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PC1 -*/
    if (gpio_func(&port, GPIO_PORT_C, GPIO_PORT_1, (uint32_t)GPIO_PC1_EA17, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PC2 -*/
    if (gpio_func(&port, GPIO_PORT_C, GPIO_PORT_2, (uint32_t)GPIO_PC2_EA18, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PC3 -*/
    if (gpio_func(&port, GPIO_PORT_C, GPIO_PORT_3, (uint32_t)GPIO_PC3_EA19, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PC4 -*/
    if (gpio_func(&port, GPIO_PORT_C, GPIO_PORT_4, (uint32_t)GPIO_PC4_EA20, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PC5 -*/
    if (gpio_func(&port, GPIO_PORT_C, GPIO_PORT_5, (uint32_t)GPIO_PC5_EA21, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PC6 -*/
    if (gpio_func(&port, GPIO_PORT_C, GPIO_PORT_6, (uint32_t)GPIO_PC6_EA22, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PC7 -*/
    if (gpio_func(&port, GPIO_PORT_C, GPIO_PORT_7, (uint32_t)GPIO_PC7_EA23, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port D]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PDDATA | PDCR | PDFRn | PDOD | PDPUP | PDDN | PDIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PD0  | ED00      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD1  | ED01      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD2  | ED02      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD3  | ED03      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD4  | ED04      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD5  | ED05      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD6  | ED06      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD7  | ED07      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pd_instance = TSB_PD;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_D) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PD0 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_0, (uint32_t)GPIO_PD0_ED00, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD1 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_1, (uint32_t)GPIO_PD1_ED01, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD2 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_2, (uint32_t)GPIO_PD2_ED02, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD3 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_3, (uint32_t)GPIO_PD3_ED03, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD4 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_4, (uint32_t)GPIO_PD4_ED04, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD5 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_5, (uint32_t)GPIO_PD5_ED05, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD6 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_6, (uint32_t)GPIO_PD6_ED06, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD7 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_7, (uint32_t)GPIO_PD7_ED07, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port E]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PEDATA | PECR | PEFRn | PEOD | PEPUP | PEDN | PEIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PE0  | ED08      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE1  | ED09      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE2  | ED10      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE3  | ED11      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE4  | ED12      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE5  | ED13      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE6  | ED14      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE7  | ED15      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pe_instance = TSB_PE;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_E) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PE0 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_0, (uint32_t)GPIO_PE0_ED08, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE1 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_1, (uint32_t)GPIO_PE1_ED09, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE2 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_2, (uint32_t)GPIO_PE2_ED10, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE3 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_3, (uint32_t)GPIO_PE3_ED11, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE4 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_4, (uint32_t)GPIO_PE4_ED12, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE5 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_5, (uint32_t)GPIO_PE5_ED13, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE6 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_6, (uint32_t)GPIO_PE6_ED14, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE7 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_7, (uint32_t)GPIO_PE7_ED15, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port F]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PFDATA | PFCR | PFFRn | PFOD | PFPUP | PFDN | PFIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PF0  | ERD_N     | O   | 0      | 1    | PFFR1 | 0    | 0     | 0    | 0     */
    /*  PF1  | EWR_N     | O   | 0      | 1    | PFFR1 | 0    | 0     | 0    | 0     */
    /*  PF2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF6  | EBELL_N   | O   | 0      | 1    | PFFR1 | 0    | 0     | 0    | 0     */
    /*  PF7  | EBELH_N   | O   | 0      | 1    | PFFR1 | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pf_instance = TSB_PF;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_F) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PF0 -*/
    if (gpio_func(&port, GPIO_PORT_F, GPIO_PORT_0, (uint32_t)GPIO_PF0_ERD_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PF1 -*/
    if (gpio_func(&port, GPIO_PORT_F, GPIO_PORT_1, (uint32_t)GPIO_PF1_EWR_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PF2 -*/
    /* Use reset value.  */
    /*- PF3 -*/
    /* Use reset value.  */
    /*- PF4 -*/
    /* Use reset value.  */
    /*- PF5 -*/
    /* Use reset value.  */
    /*- PF6 -*/
    if (gpio_func(&port, GPIO_PORT_F, GPIO_PORT_6, (uint32_t)GPIO_PF6_EBELL_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PF7 -*/
    if (gpio_func(&port, GPIO_PORT_F, GPIO_PORT_7, (uint32_t)GPIO_PF7_EBELH_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port G]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PGDATA | PGCR | PGFRn | PGOD | PGPUP | PGDN | PGIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PG0  |  EALE     | O   | 0      | 1    | PGFR1 | 0    | 0     | 0    | 0     */
    /*  PG1  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG2  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG3  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG4  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG5  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG6  |  PSW 0    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PG7  |  PSW 1    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pg_instance = TSB_PG;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_G) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PG0 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_0, (uint32_t)GPIO_PG0_EALE, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG1 -*/
    /* Use reset value.  */
    /*- PG2 -*/
    /* Use reset value.  */
    /*- PG3 -*/
    /* Use reset value.  */
    /*- PG4 -*/
#ifdef FUART_TRANSFER_USE
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_4, (uint32_t)GPIO_PG4_FUT0TXD, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
#else
    /* Use reset value.  */
#endif
    /*- PG5 -*/
#ifdef FUART_TRANSFER_USE
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_5, (uint32_t)GPIO_PG5_FUT0RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
#else
    /* Use reset value.  */
#endif
    /*- PG6 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_6, (uint32_t)GPIO_PG6_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG7 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_7, (uint32_t)GPIO_PG7_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port H]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PHDATA | PHCR | PHFRn | PHOD | PHPUP | PHDN | PHIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PH0  | UT1TXDA   | O   | 0      | 1    | PHFR5 | 0    | 0     | 0    | 0     */
    /*  PH1  | UT1RXD    | I   | 0      | 0    | PHFR5 | 0    | 0     | 0    | 1     */
    /*  PH2  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PH3  | TDI       | -   | 0      | 0    | PHFR1 | 0    | 1     | 0    | 1     */
    /*  PH4  | SWDIO     | -   | 0      | 1    | PHFR1 | 0    | 1     | 0    | 1     */
    /*  PH5  | TCK       | -   | 0      | 0    | PHFR1 | 0    | 0     | 1    | 1     */
    /*  PH6  | TDO       | -   | 0      | 1    | PHFR1 | 0    | 0     | 0    | 0     */
    /*  PH7  | TRST_N    | -   | 0      | 0    | PHFR1 | 0    | 1     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_ph_instance = TSB_PH;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_H) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PH0 -*/
    if (gpio_func(&port, GPIO_PORT_H, GPIO_PORT_0, (uint32_t)GPIO_PH0_UT1TXDA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PH1 -*/
    if (gpio_func(&port, GPIO_PORT_H, GPIO_PORT_1, (uint32_t)GPIO_PH1_UT1RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PH2 -*/
    /* Use reset value.  */
    /*- PH3 -*/
    /* Use reset value.  */    
    /*- PH4 -*/
    /* Use reset value.  */
    /*- PH5 -*/
    /* Use reset value.  */
    /*- PH6 -*/
    /* Use reset value.  */
    /*- PH7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port J]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PJDATA | PJCR | PJFRn | PJOD | PJPUP | PJDN | PJIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PJ0  | LED 0     | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ1  | LED 0     | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ2  | LED 0     | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ3  | LED 0     | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pj_instance = TSB_PJ;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_J) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PJ0 -*/
    if (gpio_func(&port, GPIO_PORT_J, GPIO_PORT_0, (uint32_t)GPIO_PJ0_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PJ1 -*/
    if (gpio_func(&port, GPIO_PORT_J, GPIO_PORT_1, (uint32_t)GPIO_PJ1_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PJ2 -*/
    if (gpio_func(&port, GPIO_PORT_J, GPIO_PORT_2, (uint32_t)GPIO_PJ2_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PJ3 -*/
    if (gpio_func(&port, GPIO_PORT_J, GPIO_PORT_3, (uint32_t)GPIO_PJ3_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PJ4 -*/
    /* Use reset value.  */
    /*- PJ5 -*/
    /* Use reset value.  */
    /*- PJ6 -*/
    /* Use reset value.  */
    /*- PJ7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port K]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PKDATA | PKCR | PKFRn | PKOD | PKPUP | PKDN | PKIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PK0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK2  | ECS0_N    | O   | 0      | 1    | PKFR1 | 0    | 0     | 0    | 0     */
    /*  PK3  | ECS1_N    | O   | 0      | 1    | PKFR1 | 0    | 0     | 0    | 0     */
    /*  PK4  |PORT_LED[0]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK5  |PORT_LED[1]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK6  |PORT_LED[2]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK7  |PORT_LED[3]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pk_instance = TSB_PK;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_K) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PK0 -*/
    /* Use reset value.  */
    /*- PK1 -*/
    /* Use reset value.  */
    /*- PK2 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_2, (uint32_t)GPIO_PK2_ECS0_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PK3 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_3, (uint32_t)GPIO_PK3_ECS1_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PK4 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_4, (uint32_t)GPIO_PK4_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PK5 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_5, (uint32_t)GPIO_PK5_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PK6 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_6, (uint32_t)GPIO_PK6_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PK7 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_7, (uint32_t)GPIO_PK7_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port L]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PLDATA | PLCR | PLFRn | PLOD | PLPUP | PLDN | PLIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PL0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL6  | PSW 2     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PL7  | PSW 3     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pl_instance = TSB_PL;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_L) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PL0 -*/
    /* Use reset value.  */
    /*- PL1 -*/
    /* Use reset value.  */
    /*- PL2 -*/
    /* Use reset value.  */
    /*- PL3 -*/
    /* Use reset value.  */
    /*- PL4 -*/
    /* Use reset value.  */
    /*- PL5 -*/
    /* Use reset value.  */
    /*- PL6 -*/
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_6, (uint32_t)GPIO_PL6_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PL7 -*/
#ifdef CGRST_SAMPLE
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_7, (uint32_t)GPIO_PL7_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
#else
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_7, (uint32_t)GPIO_PL7_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
#endif    
    /*===============================================================================*/
    /*  [Port M]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PMDATA | PMCR | PMFRn | PMOD | PMPUP | PMDN | PMIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PM0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pm_instance = TSB_PM;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_M) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PM0 -*/
    /* Use reset value.  */
    /*- PM1 -*/
    /* Use reset value.  */
    /*- PM2 -*/
    /* Use reset value.  */
    /*- PM3 -*/
    /* Use reset value.  */
    /*- PM4 -*/
    /* Use reset value.  */
    /*- PM5 -*/
    /* Use reset value.  */
    /*- PM6 -*/
    /* Use reset value.  */
    /*- PM7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port N]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PNDATA | PNCR | PNFRn | PNOD | PNPUP | PNDN | PNIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PN0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PN7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pn_instance = TSB_PN;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_N) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PN0 -*/
    /* Use reset value.  */
    /*- PN1 -*/
    /* Use reset value.  */
    /*- PN2 -*/
    /* Use reset value.  */
    /*- PN3 -*/
    /* Use reset value.  */
    /*- PN4 -*/
    /* Use reset value.  */
    /*- PN5 -*/
    /* Use reset value.  */
    /*- PN6 -*/
    /* Use reset value.  */
    /*- PN7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port P]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PPDATA | PPCR | PPFRn | PPOD | PPPUP | PPDN | PPIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PP0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PP7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pp_instance = TSB_PP;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_P) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PP0 -*/
    /* Use reset value.  */
    /*- PP1 -*/
    /* Use reset value.  */
    /*- PP2 -*/
    /* Use reset value.  */
    /*- PP3 -*/
    /* Use reset value.  */
    /*- PP4 -*/
    /* Use reset value.  */
    /*- PP5 -*/
    /* Use reset value.  */
    /*- PP6 -*/
    /* Use reset value.  */
    /*- PP7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port R]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PRDATA | PRCR | PRFRn | PROD | PRPUP | PRDN | PRIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PR0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PR7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pr_instance = TSB_PR;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_R) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PR0 -*/
    /* Use reset value.  */
    /*- PR1 -*/
    /* Use reset value.  */
    /*- PR2 -*/
    /* Use reset value.  */
    /*- PR3 -*/
    /* Use reset value.  */
    /*- PR4 -*/
    /* Use reset value.  */
    /*- PR5 -*/
    /* Use reset value.  */
    /*- PR6 -*/
    /* Use reset value.  */
    /*- PR7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port T]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PTDATA | PTCR | PTFRn | PTOD | PTPUP | PTDN | PTIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PT0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT3  | RXIN0     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PT4  | RXIN1     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PT5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pt_instance = TSB_PT;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_T) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PT0 -*/
    /* Use reset value.  */
    /*- PT1 -*/
    /* Use reset value.  */
    /*- PT2 -*/
    /* Use reset value.  */
    /*- PT3 -*/
    if (gpio_func(&port, GPIO_PORT_T, GPIO_PORT_3, (uint32_t)GPIO_PT3_RXIN0, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PT4 -*/
    if (gpio_func(&port, GPIO_PORT_T, GPIO_PORT_4, (uint32_t)GPIO_PT4_RXIN1, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PT5 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port U]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PUDATA | PUCR | PUFRn | PUOD | PUPUP | PUDN | PUIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PU0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PU7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pu_instance = TSB_PU;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_U) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PU0 -*/
    /* Use reset value.  */
    /*- PU1 -*/
    /* Use reset value.  */
    /*- PU2 -*/
    /* Use reset value.  */
    /*- PU3 -*/
    /* Use reset value.  */
    /*- PU4 -*/
    /* Use reset value.  */
    /*- PU5 -*/
    /* Use reset value.  */
    /*- PU6 -*/
    /* Use reset value.  */
    /*- PU7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port V]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PVDATA | PVCR | PVFRn | PVOD | PVPUP | PVDN | PVIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PV0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PV7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pv_instance = TSB_PV;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_V) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PV0 -*/
    /* Use reset value.  */
    /*- PV1 -*/
    /* Use reset value.  */
    /*- PV2 -*/
    /* Use reset value.  */
    /*- PV3 -*/
    /* Use reset value.  */
    /*- PV4 -*/
    /* Use reset value.  */
    /*- PV5 -*/
    /* Use reset value.  */
    /*- PV6 -*/
    /* Use reset value.  */
    /*- PV6 -*/
    /* Use reset value.  */
    /*- PV7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port W]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PWDATA | PWCR | PWFRn | PWOD | PWPUP | PWDN | PWIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PW0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PW7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pw_instance = TSB_PW;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_W) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PW0 -*/
    /* Use reset value.  */
    /*- PW1 -*/
    /* Use reset value.  */
    /*- PW2 -*/
    /* Use reset value.  */
    /*- PW3 -*/
    /* Use reset value.  */
    /*- PW4 -*/
    /* Use reset value.  */
    /*- PW5 -*/
    /* Use reset value.  */
    /*- PW6 -*/
    /* Use reset value.  */
    /*- PW7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port Y]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PYDATA | PYCR | PYFRn | PYOD | PYPUP | PYDN | PYIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PY0  | X1        | O   | 0      | N/A  | N/A   | N/A  | N/A   | 0    | 0     */
    /*  PY1  | X2        | I   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PY2  | XT1       | I   | 0      | N/A  | N/A   | 0    | 0     | 0    | 0     */
    /*  PY3  | XT2       | O   | 0      | 1    | N/A   | 0    | 0     | 0    | 0     */
    /*  PY4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | N/A   */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_py_instance = TSB_PY;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_Y) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PY0 -*/
    if (gpio_func(&port, GPIO_PORT_Y, GPIO_PORT_0, (uint32_t)GPIO_PY0_X1, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PY1 -*/
    if (gpio_func(&port, GPIO_PORT_Y, GPIO_PORT_1, (uint32_t)GPIO_PY1_X2, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PY2 -*/
    if (gpio_func(&port, GPIO_PORT_Y, GPIO_PORT_2, (uint32_t)GPIO_PY2_XT1, GPIO_PIN_NOTINOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PY3 -*/
    if (gpio_func(&port, GPIO_PORT_Y, GPIO_PORT_3, (uint32_t)GPIO_PY3_XT2, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PY4 -*/
    /* Use reset value.  */
}


/*--------------------------------------------------*/
/** 
  * @brief  EBIF Initialize.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/

static void EBIF_interface_initialize(void)
{
    ebif_config.exbmod_wait   = EXBMOD_WAIT_0MULTI;
    ebif_config.exbmod_select = EXBMODE_SEL_MULTIPLEXEER;
    ebif_config.cs0_reg = MAIN_NULL;
    ebif_config.cs1_reg = &cs1_config;
    ebif_config.cs0_reg = MAIN_NULL;
    
    cs1_config.exbas_start_address = SRAM_BASE;
    cs1_config.exbas_area          = ECBAS_EXAR_1M;
    
    cs1_config.exbcs_csr  = EXBCS_CSR_0CYCLE;
    cs1_config.exbcs_wrr  = EXBCS_WRR_0CYCLE;
    cs1_config.exbcs_rdr  = EXBCS_RDR_0CYCLE;
    cs1_config.exbcs_alew = EXBCS_ALEW_2CYCLE;          /* 2cycle : 2 * 1/160 * 1000 = 12.5ns */
    cs1_config.exbcs_wrs  = EXBCS_WRS_0CYCLE;
    cs1_config.exbcs_rds  = EXBCS_RDS_0CYCLE;
    cs1_config.exbcs_wsel = EXBCS_WSEL_LOW_ACTIVE;
    cs1_config.exbcs_wait = EXBCS_WAIT_INTERNAL;
    cs1_config.exbcs_csiw = EXBCS_CSIW_8WAIT;           /* 8wait = 8 * 1/160 * 1000 = 50.0ns */
    cs1_config.exbcs_csw  = EXBCS_CSW_16BIT;
    cs1_config.exbcs_csw0 = EXBCS_CSW0_ENABLE;
    
    EBIF_init(&ebif_config);
}

/**
 *  @}
 */ /* End of group EBIF_SRAM_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Exported_functions EBIF_SRAM Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    int num;
    int len = 0;
    int cnum = 0;
    /*----------------------*/
    /* Set Processing Mode  */
    /*----------------------*/
    proc = PROC_OUTPUT_MSG;
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    EBIF_port_initialize();
    EBIF_interface_initialize();
    /* Application initialization */
    if(application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    {
        int i;
        for (i = 0; i < STRING_NUM; i++)
        {
            write_data[i] = 0;
        }
    }
    
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(1)
    {
        switch(proc)
        {
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear input data.     */
                clear_data();
                /* Output first message. */
                if (printf("command > ") != -1)
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {   
                int data;
                
                data = getchar();
                if ((data != -1) && (input.wp < UART_IO_RECEIVE_MAX))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- C.dataR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        len = input.wp;
                        proc = PROC_COMMAND_PARSE;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Echo Back Data" ---*/
        case PROC_COMMAND_PARSE:
            {
                int i;
                int32_t res = 0;

                if ((strncmp((const char *)&input.data[0], "write", 5)) == 0)
                {
                    /* WRITE Command */
                    if (len >= 9)
                    {
                        if(printf("write data > ") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                        num = (len - (6 + 2));
                        for (i = 0; (i < num) && (i < (STRING_NUM-2)); i++)
                        {
                            write_data[i] = (uint8_t)(input.data[6 + i]);
                        }
                        write_data[i] = 0x00;
                       
                        cnum = i;
                        for (i = 0; (i < num) && (i < (STRING_NUM-2)); i++)
                        {
                            if(write_data[i] != (uint8_t)(input.data[6 + i]))
                            {
                                res = -1;
                                break;
                            }
                        }
                        cnum = i;
                        
                        if (res >= 0)
                        {
                            if(printf("%s\n", (const char *)&write_data[0]) < 0)
                            {
                                proc = PROC_ERROR;
                            }
                            else
                            {
                                proc = PROC_OUTPUT_MSG;
                            }
                        }
                        else
                        {
                            if(printf("\n write error !! \n") < 0)
                            {
                                /* no process */
                            }
                            proc = PROC_ERROR;
                        }
                        if(printf("\n") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                    break;
                }
                else if ((strncmp((const char *)&input.data[0], "read", 4)) == 0)
                {
                    /* READ Command */
                    if(printf("read data > ") < 0)
                    {
                        proc = PROC_ERROR;
                    }
                    if (cnum > 0)
                    {
                        if(printf("%s\n", (const char*)&write_data[0]) < 0)
                        {
                            proc = PROC_ERROR;
                        }
                        else
                        {
                            proc = PROC_OUTPUT_MSG;
                        }
                    }
                    else
                    {
                        if(printf("\n read error !! \n") < 0)
                        {
                            /* no process */
                        }
                        proc = PROC_ERROR;
                    }
                    if(printf("\n") < 0)
                    {
                        proc = PROC_ERROR;
                    }
                }
                else if ((strncmp((const char *)&input.data[0], "bw", 2)) == 0)
                {
                    /* BYTE WRITE Command */
                    if (len > 11)
                    {
                        char str[6] = {0};
                        int32_t address;
                        char pr_data[2] = {0};
                        volatile char *ptr;

                        if(printf("byte write data > ") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                        str[0]= input.data[3];
                        str[1]= input.data[4];
                        str[2]= input.data[5];
                        str[3]= input.data[6];
                        str[4]= input.data[7];
                        address = strtol(str, 0, 16);
                        if((address < 0) || (address > 0xFFFFF))
                        {
                            if(printf("\n address error !! \n") < 0)
                            {
                                /* no process */
                            }
                            proc = PROC_ERROR;
                            break;
                        }
                        pr_data[0] = input.data[9];
                        pr_data[1] = 0;
                        ptr = (char *)((uint32_t)address+SRAM_BASE);
                        *ptr = input.data[9];
                        
                        if (*ptr == input.data[9])
                        {
                            if(printf("0x%x = %s\n", address, (const char *)&pr_data[0]) < 0)
                            {
                                proc = PROC_ERROR;
                            }
                            else
                            {
                                proc = PROC_OUTPUT_MSG;
                            }
                        }
                        else
                        {
                            if(printf("\n write error !! \n") < 0)
                            {
                                /* no process  */
                            }
                            proc = PROC_ERROR;
                        }
                        if(printf("\n") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                }
                else if ((strncmp((const char *)&input.data[0], "br", 2)) == 0)
                {
                    /* BYTE READ Command */
                    if (len > 9)
                    {
                        char str[6] = {0};
                        int32_t address;
                        char pr_data[2] = {0};
                        volatile char *ptr;

                        if(printf("byte read data > ") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                        str[0]= input.data[3];
                        str[1]= input.data[4];
                        str[2]= input.data[5];
                        str[3]= input.data[6];
                        str[4]= input.data[7];
                        address = strtol(str, 0, 16);
                        if((address < 0) || (address > 0xFFFFF))
                        {
                            if(printf("\n address error !! \n") < 0)
                            {
                                /* no process */
                            }
                            proc = PROC_ERROR;
                            break;
                        }
                        ptr = (char *)((uint32_t)address+SRAM_BASE);
                        pr_data[0] = *ptr;
                        pr_data[1] = 0;
                        if (*ptr == pr_data[0])
                        {
                            if(printf("0x%x = %s\n", address, (const char*)&pr_data[0]) < 0)
                            {
                                proc = PROC_ERROR;
                            }
                            else
                            {
                                proc = PROC_OUTPUT_MSG;
                            }
                        }
                        else
                        {
                            if(printf("\n read error !! \n") < 0)
                            {
                                /* no process */
                            }
                            proc = PROC_ERROR;
                        }
                        if(printf("\n") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
                
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            if(printf("? \n") < 0)
            {
                proc = PROC_ERROR;
            }
            else
            {
                proc = PROC_OUTPUT_MSG;
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* System Error. Please Debug. */
#ifdef DEBUG
            assert_failed(MAIN_NULL, 0);
#endif
            proc = PROC_OUTPUT_MSG;
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_systick(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  Push SW State Change.
  * @param  psw :Push SW.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_psw(BSPPsw psw)
{
    /* Sample Soft don't use this function. */
}
#if defined(TMPM4K4)
/*--------------------------------------------------*/
/** 
  * @brief  External Interrupt(INT01) IRQ.
  * @param  exi : External Interrupt.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_exint(BSPExInt exi)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  Continuous Conversion Finish handler
  * @param  -
  * @retval -
  * @note   For Continuous Conversion
  */
/*--------------------------------------------------*/
void irq_potentiometers_cnt(BSPPotentiometers potentiometers)
{
    /* Sample Soft don't use this function. */
}
#endif


/*--------------------------------------------------*/
/** 
  * @brief  SPI flash reception IRQ.
  * @param  sflash :SPI Flash.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_rx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  sflash :SPI Flash.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_tx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash error IRQ.
  * @param  sflash :SPI Flash.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_err(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rtc(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  ADC SGL IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_adc_sgl(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC CNT IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_adc_cnt(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  EXINT IRQ.
  * @param  exi
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_exint(BSPExInt exi)
{
    /* Sample Soft don't use this function. */
}
#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
     /*   __nop(); */
    }
}
#endif

/**
 *  @}
 */ /* End of group EBIF_SRAM_Exported_functions */


/**
 *  @} 
 */ /* End of group EBIF_SRAM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
