/** 
 *******************************************************************************
 * @file    f2ram.c
 * @brief   Flash_Data Sample Application.
 * @version V1.0.1.0
 * $Date:: 2018-09-21 11:11:53 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_sample_def.h"
#include "fc.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup Flash_Data Flash_Data Sample Appli
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
extern uint32_t Load$$FLASH_CODE_RAM$$Base;
extern uint32_t Image$$FLASH_CODE_RAM$$Base;
extern uint32_t Load$$FLASH_CODE_RAM$$Length;

extern uint32_t Image$$RAM_DEMO_A$$Base;
extern uint32_t Load$$RAM_DEMO_A$$Length;

extern uint32_t Image$$FRAM_PAGE_1$$Base;
extern uint32_t Image$$FRAM_PAGE_2$$Base;

#elif defined ( __ICCARM__ )    /* IAR Compiler */
#pragma section = "FLASH_CODE_RAM"
#pragma section = "FLASH_CODE_ROM"
#pragma section = "FRAM_PAGE_1"
#pragma section = "FRAM_PAGE_2"
#pragma section = "DATA_A"
#endif


/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Data_Private_macro Flash_Data Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group Flash_Data_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Data_Private_define Flash_Data Private Define
 *  @{
 */
#if defined ( __CC_ARM   )      /* RealView Compiler */
//#define FLASH_API_ROM           (uint32_t *)&Load$$FLASH_CODE_ROM$$Base

//#define FLASH_API_RAM           (uint32_t *)&Image$$FLASH_CODE_ROM$$Base
//#define SIZE_FLASH_API          (uint32_t)&Load$$FLASH_CODE_RAM$$Length

#define DEMO_A_RAM              (uint32_t *)&Image$$RAM_DEMO_A$$Base
#define SIZE_DEMO_A               (uint32_t)&Load$$RAM_DEMO_A$$Length

#define PAGE_1_FRAM              (uint32_t *)&Image$$FRAM_PAGE_1$$Base
#define PAGE_2_FRAM              (uint32_t *)&Image$$FRAM_PAGE_2$$Base

#elif defined ( __ICCARM__ )    /* IAR Compiler */
#define FLASH_API_ROM           ((uint32_t *)__section_begin("FLASH_CODE_ROM"))

#define FLASH_API_RAM           ((uint32_t *)__section_begin("FLASH_CODE_RAM"))
#define SIZE_FLASH_API          ((uint32_t)__section_size("FLASH_CODE_ROM"))

#define DEMO_A_RAM              ((uint32_t *)__section_begin("DATA_A"))
#define SIZE_DEMO_A             ((uint32_t)__section_size("DATA_A"))

#define PAGE_1_FRAM             ((uint32_t *)__section_begin("FRAM_PAGE_1"))
#define PAGE_2_FRAM             ((uint32_t *)__section_begin("FRAM_PAGE_2"))

#endif

/* code start address */
#define DEMO_START_ADDR         ((uint32_t)DEMO_A_FLASH)


/** 
 *  @}
 */ /* End of group Flash_Data_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Data_Private_define Flash_Data Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group Flash_Data_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Data_Private_define Flash_Data Private Define
 *  @{
 */
/** 
 *  @defgroup Flash_Data_Private_typedef Flash_Data Private Typedef
 *  @{
 */

/**
 *  @}
 */ /* End of group Flash_Data_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Data_Private_variables Flash_Data Private Variables
 *  @{
 */
/**
 *  @}
 */ /* End of group Flash_Data_Private_variables */
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Data_Private_fuctions Flash_Data Private Fuctions
 *  @{
 */
void (* startup) (void);
uint32_t gLoadAddr;


/**
 *  @}
 */ /* End of group Flash_Data_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Data_Exported_functions Flash_Data Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  execution user program.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void exec_user_program(void)
{

        if((gLoadAddr & 0x01) == 0) {             /* Thumb check */
            gLoadAddr |= 0x01;
        }
        startup = (void(*)(void))gLoadAddr;
        startup();              /* jump to user_program */
}

/*--------------------------------------------------*/
/** 
  * @brief  veryfy flash data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result verify_fc_func(void)
{
    TXZ_Result retval = TXZ_SUCCESS;

    if(fc_verify_check(DEMO_A_RAM, PAGE_1_FRAM, SIZE_DEMO_A) == TXZ_ERROR){ 
        retval = TXZ_ERROR;
    }
    else {
        if(fc_verify_check(DEMO_A_RAM, PAGE_2_FRAM, SIZE_DEMO_A) == TXZ_ERROR){
            retval = TXZ_ERROR;
        }
        else {
                    /* do nothing */
        }
    }
    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  rewrite user program.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result rewrite_user_ram(void)
{
    TXZ_Result retval = TXZ_SUCCESS;

    if(fc_write_data_flash(DEMO_A_RAM, PAGE_1_FRAM, SIZE_DEMO_A) == TXZ_ERROR){ 
        retval = TXZ_ERROR;
    }
    else {
        if(fc_write_data_flash(DEMO_A_RAM, PAGE_2_FRAM, SIZE_DEMO_A) == TXZ_ERROR){ 
            retval = TXZ_ERROR;
        }
        else {
            /* do nothing */
        }
    }
    return (retval);

}

/**
 *  @}
 */ /* End of group Flash_Data_Exported_functions */

/**
 *  @}
 */ /* End of group Flash_Data */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
