/** 
 *******************************************************************************
 * @file    f2ram.c
 * @brief   Flash_UserInformation Sample Application.
 * @version V1.0.0.0
 * $Date:: 2018-09-21 11:28:55 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_sample_def.h"
#include "fc.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup Flash_UserInformation Flash_UserInformation Sample Appli
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
extern uint32_t Load$$FLASH_CODE_RAM$$Base;
extern uint32_t Image$$FLASH_CODE_RAM$$Base;
extern uint32_t Load$$FLASH_CODE_RAM$$Length;

#elif defined ( __ICCARM__ )    /* IAR Compiler */
#pragma section = "FLASH_CODE_RAM"
#pragma section = "FLASH_CODE_ROM"
#endif


/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_UserInformation_Private_macro Flash_UserInformation Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group Flash_UserInformation_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_UserInformation_Private_define Flash_UserInformation Private Define
 *  @{
 */
#if defined ( __CC_ARM   )      /* RealView Compiler */
#define FLASH_API_ROM           (uint32_t *)&Load$$FLASH_CODE_RAM$$Base
#define FLASH_API_RAM           (uint32_t *)&Image$$FLASH_CODE_RAM$$Base
#define SIZE_FLASH_API          (uint32_t)&Load$$FLASH_CODE_RAM$$Length



#elif defined ( __ICCARM__ )    /* IAR Compiler */
#define FLASH_API_ROM           ((uint32_t *)__section_begin("FLASH_CODE_ROM"))
#define FLASH_API_RAM           ((uint32_t *)__section_begin("FLASH_CODE_RAM"))
#define SIZE_FLASH_API          ((uint32_t)__section_size("FLASH_CODE_ROM"))

#endif

/* code start address */
#define DEMO_START_ADDR         ((uint32_t)DEMO_A_FLASH)


/** 
 *  @}
 */ /* End of group Flash_UserInformation_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_UserInformation_Private_define Flash_UserInformation Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group Flash_UserInformation_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Private_define Flash_Userboot Private Define
 *  @{
 */
/** 
 *  @defgroup Flash_Userboot_Private_typedef Flash_Userboot Private Typedef
 *  @{
 */

/**
 *  @}
 */ /* End of group Flash_UserInformation_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_UserInformation_Private_variables Flash_UserInformation Private Variables
 *  @{
 */
/**
 *  @}
 */ /* End of group Flash_UserInformation_Private_variables */
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_UserInformation_Private_fuctions Flash_UserInformation Private Fuctions
 *  @{
 */
static void Copy_Routine(uint32_t * dest, uint32_t * source, uint32_t size);
void (* startup) (void);
uint32_t gLoadAddr;


/*--------------------------------------------------*/
/** 
  * @brief  copy 32-bit data from source to dest
  * @param  the address of source and dast, the data size
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Copy_Routine(uint32_t * dest, uint32_t * source, uint32_t size)
{
    uint32_t *dest_addr, *source_addr, tmpsize;
    uint32_t i, tmps, tmpd, mask;

    dest_addr = dest;
    source_addr = source;

    tmpsize = size >> 2U;
    for (i = 0U; i < tmpsize; i++) {    /* 32bits copy */
        *dest_addr = *source_addr;
        dest_addr++;
        source_addr++;
    }
    if (size & 0x00000003U) {   /* if the last data size is not 0(maybe 1,2 or 3), copy the last data */
        mask = 0xFFFFFF00U;
        i = size & 0x00000003U;
        tmps = *source_addr;
        tmpd = *dest_addr;
        while (i - 1U) {
            mask = mask << 8U;
            i--;
        }
        tmps = tmps & (~mask);
        tmpd = tmpd & (mask);
        *dest_addr = tmps + tmpd;       /* 32bits copy, but only change the bytes need to be changed */
    } else {
        /* Do nothing */
    }
}
/**
 *  @}
 */ /* End of group Flash_UserInformation_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_UserInformation_Exported_functions Flash_UserInformation Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  execution user program.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void exec_user_program(void)
{
//		gLoadAddr = DEMO_START_ADDR;

		if((gLoadAddr & 0x01) == 0) {			/* Thumb check */
			gLoadAddr |= 0x01;
		}
        startup = (void(*)(void))gLoadAddr;
        startup();              /* jump to user_program */
}
/*--------------------------------------------------*/
/** 
  * @brief  copy flash control functions.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void copy_fc_func(void)
{
	Copy_Routine(FLASH_API_RAM, FLASH_API_ROM, SIZE_FLASH_API);     /* copy flash API to RAM */
}

/**
 *  @}
 */ /* End of group Flash_UserInformation_Exported_functions */

/**
 *  @}
 */ /* End of group Flash_UserInformation */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
