/** 
 *******************************************************************************
 * @file    main.c
 * @brief   FUART Echo MDMA Sample Application.
 * @version V1.0.0.0
 * $Date:: 2017-08-28 10:52:31 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
   
#include "txz_sample_def.h"
#include "bsp_fuart_io_mdma.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup FUART_ECHO FUART ECHO Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FUART_ECHO_Private_macro FUART ECHO Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group FUART_ECHO_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FUART_ECHO_Private_define FUART ECHO Private Define
 *  @{
 */
/** 
 *  @}
 */ /* End of group FUART_ECHO_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FUART_ECHO_Private_define FUART ECHO Private Define
 *  @{
 */

#define MAIN_NULL               ((void *)0) /*!< Null Pointer */

/** 
 *  @}
 */ /* End of group FUART_ECHO_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FUART_ECHO_Private_define FUART ECHO Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_START,             /*!< Start.                       */
    PROC_STOP,              /*!< Stop.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_OUTPUT_ECHO,       /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_STRING,            /*!< String input/output.         */
    PROC_END,               /*!< End.                         */
    PROC_ERROR              /*!< Error.                       */
} Processing;
/** 
 *  @}
 */ /* End of group FUART_ECHO_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FUART_ECHO_Private_typedef FUART ECHO Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(FUART_IO_MDMA_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint32_t wp;                                 /*!< Write pointer.                */
} input_t;
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   mdma_t        mdma;      /*!< Driver      :MDMA.               */
   fuart_io_mdma_t *p_fuart;  /*!< Application :USB FUART.          */
} instance_t;

/**
 *  @}
 */ /* End of group FUART_ECHO_Private_typedef */


extern void bsp_initialize(void);		/*!< BSP initialize.               */
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FUART_ECHO_Private_variables FUART ECHO Private Variables
 *  @{
 */

static instance_t instance;     /*!< Instance.               */
static Processing proc;         /*!< Processing mode.        */
static input_t    input;        /*!< Input data information. */
static uint32_t   loop_counter; /*!< Loop counter.           */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    __align(1024)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma data_alignment=1024
#endif
  static mdma_descriptor_table_t descriptor_base[MDMA_CH_NUM];     /*!< MDMA Descriptor Base area. */
/**
 *  @}
 */ /* End of group FUART_ECHO_Private_variables */



/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FUART_ECHO_Private_fuctions FUART ECHO Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static TXZ_Result driver_mdma_initialize(void);
static TXZ_Result driver_mdma_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void clear_input_data(void);
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.mdma.p_instance     = MAIN_NULL;
    /* Processing mode. */
    proc = PROC_START;
    
    loop_counter = 0;
    
    (void)memset(&descriptor_base[0], 0x00, sizeof(descriptor_base));
}
/*--------------------------------------------------*/
/** 
  * @brief  MDMA Driver Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_mdma_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*--- Construct (Register Allocation) ---*/
    instance.mdma.p_instance = TSB_MDMAA;
    
    /*--- Initialize ---*/
    instance.mdma.init_desc.base = (uint32_t)&descriptor_base[0];
    
    for(i=0; i<MDMA_CH_NUM; ++i)
    {
        instance.mdma.int_ch[i].handler = MAIN_NULL;
        instance.mdma.int_ch[i].priority = MDMA_PRIORITY_NORMAL;
    }
    
    if (mdma_init(&instance.mdma) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}


/*--------------------------------------------------*/
/** 
  * @brief  MDMA Driver Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_mdma_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    if (mdma_deinit() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}


/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx are same channel */
    {
        fuart_io_mdma_initial_setting_t param;
        
        param.id = (uint32_t)&instance.mdma;
        param.p_reg  = TSB_FURT0;
        param.boudrate   = 115200;
        param.mdma.p_mdma  = &instance.mdma;
        instance.p_fuart  = fuart_io_mdma_initialize(&param);
    }
    
    return (result);
}
 
/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* USB FUART             */
    /*----------------------*/
    if (instance.p_fuart != MAIN_NULL)
    {
        fuart_io_mdma_finalize(instance.p_fuart);
    }
    
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_input_data(void)
{
    uint32_t i;

    for (i=0; i < (FUART_IO_MDMA_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
}

/**
 *  @}
 */ /* End of group FUART_ECHO_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FUART_ECHO_Exported_functions FUART ECHO Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/

int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    
    /*--- driver      ---*/
    if(driver_mdma_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    
    /*--- application ---*/
    if(application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    
    while((proc != PROC_ERROR) && (proc != PROC_END))
    {
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                /* MDMA  */
                TSB_IB->IMC140 = 0x1; /* NTMDMAABERR */
                TSB_IB->IMC141 = 0x1; /* NTMDMAADERR */

                NVIC_EnableIRQ(INTMDMAATC_IRQn);
                NVIC_EnableIRQ(INTMDMAABERR_IRQn);
                NVIC_EnableIRQ(INTMDMAADERR_IRQn);
                proc = PROC_OUTPUT_MSG;
        //        proc = PROC_END;
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                /* DMA  */
                NVIC_DisableIRQ(INTMDMAATC_IRQn);
                NVIC_DisableIRQ(INTMDMAABERR_IRQn);
                NVIC_DisableIRQ(INTMDMAADERR_IRQn);
                proc = PROC_ERROR;
            }
            break;
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear input data.     */
                clear_input_data();
                /* Output first message. */
                if (printf("Input = ") != -1 )
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {
                int data;

                data = getchar();
                if ((data != -1) && (input.wp < FUART_IO_MDMA_RECEIVE_MAX))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- CR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        proc = PROC_OUTPUT_ECHO;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Echo Back Data" ---*/
        case PROC_OUTPUT_ECHO:
            {
                if (printf(" Echo = %s", (const char *)&input.data[0]) != -1 )
                {
                    if(++loop_counter > 2){
                        loop_counter = 0;
#if 0
                        proc = PROC_STRING;
#else
                        proc = PROC_OUTPUT_MSG;
#endif
                    }
                    else{
                        proc = PROC_OUTPUT_MSG;
                    }
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            {
                if (printf("\n Input Error !! \n") != -1 )
                {
                    proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- String ---*/
        case PROC_STRING:
            if(put_s("String Recive\n") < 0)
            {
                proc = PROC_ERROR;
                break;
            }
            if(get_s(&input.data[0], 24) < 0)
            {
                proc = PROC_ERROR;
                break;
            }
            proc = PROC_OUTPUT_MSG;
            break;
        /*--- Error ---*/
        case PROC_END:
            /* no processing */
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if(driver_mdma_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    while(1)
    {
    }
}
/**
 *  @}
 */ /* End of group FUART_ECHO_Exported_functions */

/**
 *  @}
 */ /* End of group FUART_ECHO */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
