/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ISD Sample Application.
 * @version V1.0.0.0
 * $Date:: 2017-11-15 11:00:46 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "bsp.h"
#include "bsp_led.h"
#include "isd.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup ISD ISD Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_macro ISD Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ISD_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_define ISD Private Define
 *  @{
 */

// #define START_STOP2

/** 
 *  @defgroup IHOSCConfiguration IHOSC Configuration
 *  @brief    IHOSC Configuration.
 *  @{
 */
#define CFG_IHOSC_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us).  Minimam warm-up time is 100(us). */
#define CFG_IHOSC_CLOCK         ((uint32_t)(10000000))  /*!< Clock(hz).        */
/** 
 *  @}
 */ /* End of group IHOSCConfiguration */

/** 
 *  @}
 */ /* End of group ISD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_define ISD Private Define
 *  @{
 */
#define COUNTMAX               (10)            /*!< retry counter max value1 */

#define MAIN_NULL       ((void *)0)     /*!< NULL. */

/** 
 *  @defgroup  SIWDxEN  SIWDxEN Register
 *  @brief     WD Enable Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:2  | -          |
 *  | 1     | WDTF       |
 *  | 0     | WDTE       |
 *  @{
 */
/* WDTE */
#define SIWDxEN_WDTE_RW_DISABLE         ((uint32_t)0x00000000)    /*!< WDTE  :[R/W] :Disable    */
#define SIWDxEN_WDTE_RW_ENABLE          ((uint32_t)0x00000001)    /*!< WDTE  :[R/W] :Enable     */
/** 
 *  @}
 */ /* End of group SIWDxEN */

/** 
 *  @defgroup  SIWDxCR  SIWDxCR Register
 *  @brief     WD Control Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:8  | -          |
 *  | 7:0   | WDCR       |
 *  @{
 */
/* WDCR */
#define SIWDxCR_WDCR_W_DISABLE          ((uint32_t)0x000000B1)    /*!< WDCR  :[W] :Disable      */
#define SIWDxCR_WDCR_W_CLEAR            ((uint32_t)0x0000004E)    /*!< WDCR  :[W] :Clear        */
/** 
 *  @}
 */ /* End of group SIWDxCR */

/** 
 *  @defgroup  CGOSCCR  CGOSCCR Register
 *  @brief     CG Control Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:20 | -          |
 *  | 19    | IHOSC2F    |
 *  | 18:17 | -          |
 *  | 16    | IHOSC1F    |
 *  | 15:10 | -          |
 *  | 9     | OSCF       |
 *  | 8     | OSCSEL     |
 *  | 7:4   | -          |
 *  | 3     | IHOSC2EN   |
 *  | 2:1   | EOSCEN     |
 *  | 0     | IHOSC1EN   |
 *  @{
 */
/* IHOSC2F  */
#define CGOSCCR_IHOSC2F_MASK            ((uint32_t)0x00080000)    /*!< IHOSC2F   :Mask                  */
#define CGOSCCR_IHOSC2F_R_STOP          ((uint32_t)0x00000000)    /*!< IHOSC2F   :[R]   :Stop           */
#define CGOSCCR_IHOSC2F_R_RUNNING       ((uint32_t)0x00080000)    /*!< IHOSC2F   :[R]   :Running        */
/* OSCF     */
#define CGOSCCR_OSCF_MASK               ((uint32_t)0x00000200)    /*!< OSCF      :Mask                  */
#define CGOSCCR_OSCF_R_IHOSC            ((uint32_t)0x00000000)    /*!< OSCF      :[R]   :Internal HOSC  */
#define CGOSCCR_OSCF_R_EHOSC            ((uint32_t)0x00000200)    /*!< OSCF      :[R]   :External HOSC  */
/* OSCSEL   */
#define CGOSCCR_OSCSEL_MASK             ((uint32_t)0x00000100)    /*!< OSCSEL    :Mask                  */
#define CGOSCCR_OSCSEL_RW_IHOSC         ((uint32_t)0x00000000)    /*!< OSCSEL    :[R/W] :Internal HOSC  */
#define CGOSCCR_OSCSEL_RW_EHOSC         ((uint32_t)0x00000100)    /*!< OSCSEL    :[R/W] :External HOSC  */
/* IHOSC2EN */
#define CGOSCCR_IHOSC2EN_MASK           ((uint32_t)0x00000008)    /*!< IHOSC2EN  :Mask                  */
#define CGOSCCR_IHOSC2EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC2EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC2EN_RW_ENABLE      ((uint32_t)0x00000008)    /*!< IHOSC2EN  :[R/W] :Enable         */
/* EOSCEN   */
#define CGOSCCR_EOSCEN_MASK             ((uint32_t)0x00000006)    /*!< EOSCEN    :Mask                  */
#define CGOSCCR_EOSCEN_RW_UNUSE         ((uint32_t)0x00000000)    /*!< EOSCEN    :[R/W] :Unuse          */
#define CGOSCCR_EOSCEN_RW_EHOSC         ((uint32_t)0x00000002)    /*!< EOSCEN    :[R/W] :External HOSC  */
#define CGOSCCR_EOSCEN_RW_ECLK          ((uint32_t)0x00000004)    /*!< EOSCEN    :[R/W] :External Clock */
/* IHOSC1EN */
#define CGOSCCR_IHOSC1EN_MASK           ((uint32_t)0x00000001)    /*!< IHOSC1EN  :Mask                  */
#define CGOSCCR_IHOSC1EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC1EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC1EN_RW_ENABLE      ((uint32_t)0x00000001)    /*!< IHOSC1EN  :[R/W] :Enable         */
/** 
 *  @}
 */ /* End of group CGOSCCR */

/** 
 *  @defgroup  CGSTBYCR  CGSTBYCR Register
 *  @brief     Standby Control Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:2  | -          |
 *  | 1:0   | STBY       |
 *  @{
 */
/* STBY */
#define CGSTBYCR_STBY_RW_IDLE           ((uint32_t)0x00000000)    /*!< STBY :[R/W] :IDLE   */
#define CGSTBYCR_STBY_RW_STOP1          ((uint32_t)0x00000001)    /*!< STBY :[R/W] :STOP1  */
#define CGSTBYCR_STBY_RW_STOP2          ((uint32_t)0x00000002)    /*!< STBY :[R/W] :STOP2  */
/** 
 *  @}
 */ /* End of group CGSTBYCR */

/** 
 *  @defgroup  CGPLL0SEL  CGPLL0SEL Register
 *  @brief     fsys PLL Select Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:8  | PLL0SET    |
 *  | 7:3   | -          |
 *  | 2     | PLL0ST     |
 *  | 1     | PLL0SEL    |
 *  | 0     | PLL0ON     |
 *  @{
 */
/* PLL0ST  */
#define CGPLL0SEL_PLL0ST_MASK           ((uint32_t)0x00000004)    /*!< PLL0ST  :Mask.           */
#define CGPLL0SEL_PLL0ST_R_FOSC         ((uint32_t)0x00000000)    /*!< PLL0ST  :[R]     :fosc   */
#define CGPLL0SEL_PLL0ST_R_FPLL         ((uint32_t)0x00000004)    /*!< PLL0ST  :[R]     :fpll   */
/* PLL0SEL */
#define CGPLL0SEL_PLL0SEL_MASK          ((uint32_t)0x00000002)    /*!< PLL0SEL :Mask.           */
#define CGPLL0SEL_PLL0SEL_RW_FOSC       ((uint32_t)0x00000000)    /*!< PLL0SEL :[R/W]   :fosc   */
#define CGPLL0SEL_PLL0SEL_RW_FPLL       ((uint32_t)0x00000002)    /*!< PLL0SEL :[R/W]   :fpll   */
/* PLL0ON  */
#define CGPLL0SEL_PLL0ON_MASK           ((uint32_t)0x00000001)    /*!< PLL0ON  :Mask.           */
#define CGPLL0SEL_PLL0ON_RW_DISABLE     ((uint32_t)0x00000000)    /*!< PLL0ON  :[R/W]   :Disable*/
#define CGPLL0SEL_PLL0ON_RW_ENABLE      ((uint32_t)0x00000001)    /*!< PLL0ON  :[R/W]   :Enable */
/** 
 *  @}
 */ /* End of group CGPLL0SEL */

/** 
 *  @defgroup  CGWUPHCR  CGWUPHCR Register
 *  @brief     HOSC warming up Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:20 | WUPT       |
 *  | 19:16 | WUPT       |
 *  | 15:9  | -          |
 *  | 8     | WUCLK      |
 *  | 7:2   | -          |
 *  | 1     | WUEF       |
 *  | 0     | WUON       |
 *  @{
 */
/* WUPT  */
#define CGWUPHCR_WUPT_HIGH_MASK         ((uint32_t)0xFFF00000)    /*!< WUPT    :High Bit Mask.  */
/* WUCLK */
#define CGWUPHCR_WUCLK_MASK             ((uint32_t)0x00000100)    /*!< WUCLK   :Mask.           */
#define CGWUPHCR_WUCLK_RW_IHOSC         ((uint32_t)0x00000000)    /*!< WUCLK   :[R/W]  :IHOSC   */
#define CGWUPHCR_WUCLK_RW_EHOSC         ((uint32_t)0x00000100)    /*!< WUCLK   :[R/W]  :EHOSC   */
/* WUEF  */
#define CGWUPHCR_WUEF_MASK              ((uint32_t)0x00000002)    /*!< WUEF    :Mask.           */
#define CGWUPHCR_WUEF_R_DONE            ((uint32_t)0x00000000)    /*!< WUEF    :[R]    :Done    */
#define CGWUPHCR_WUEF_R_RUNNING         ((uint32_t)0x00000002)    /*!< WUEF    :[R]    :Running */
/** 
 *  @}
 */ /* End of group CGWUPHCR */

/** 
 *  @defgroup  CGWUPLCR  CGWUPLCR Register
 *  @brief     LOSC warming up Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:27 | -          |
 *  | 26:12 | WUPTL      |
 *  | 11:8  | WUPTL      |
 *  | 7:2   | -          |
 *  | 1     | WULEF      |
 *  | 0     | WULON      |
 *  @{
 */
/* WUPTL  */
#define CGWUPLCR_WUPTL_HIGH_MASK        ((uint32_t)0x07FFF000)    /*!< WUPTL   :High Bit Mask.  */
/* WULEF  */
#define CGWUPLCR_WULEF_MASK             ((uint32_t)0x00000002)    /*!< WULEF   :Mask.           */
#define CGWUPLCR_WULEF_R_DONE           ((uint32_t)0x00000000)    /*!< WULEF   :[R]  :Done      */
#define CGWUPLCR_WULEF_R_RUNNING        ((uint32_t)0x00000002)    /*!< WULEF   :[R]  :Running   */
/* WULON  */
#define CGWUPLCR_WULON_MASK             ((uint32_t)0x00000001)    /*!< WULON   :Mask.           */
#define CGWUPLCR_WULON_W_ENABLE         ((uint32_t)0x00000001)    /*!< WULON   :[W]  :Enable     */
/** 
 *  @}
 */ /* End of group CGWUPLCR */

/** 
 *  @defgroup  RLMRSTFLG0  RLMRSTFLG0 Register
 *  @brief     Reset Flag Register 0.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 7:6   | -          |
 *  | 5     | LVDRSTF    |
 *  | 4     | STOP2RSTF  |
 *  | 3     | PINRSTF    |
 *  | 2:1   | -          |
 *  | 0     | PORSTF     |
 *  @{
 */
/* ALL */
#define RLMRSTFLG0_W_CLEAR              ((uint8_t)0x00)           /*!< ALL BITS  :[W]  :Clear  */
/* STOP2RSTF */
#define RLMRSTFLG0_STOP2RSTF_MASK       ((uint8_t)0x10)           /*!< STOP2RSTF :Mask.        */
#define RLMRSTFLG0_STOP2RSTF_R_RESET    ((uint8_t)0x10)           /*!< STOP2RSTF :[R]  :Reset  */
/** 
 *  @}
 */ /* End of group RLMRSTFLG0 */

/** 
 *  @defgroup  RLMRSTFLG1  RLMRSTFLG1 Register
 *  @brief     Reset Flag Register 1.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 7:4   | -          |
 *  | 3     | OFDRSTF    |
 *  | 2     | WDTRSTF    |
 *  | 1     | LOCKRSTF   |
 *  | 0     | SYSRSTF    |
 *  @{
 */
/* ALL */
#define RLMRSTFLG1_W_CLEAR              ((uint8_t)0x00)           /*!< ALL BITS  :[W]  :Clear  */
/** 
 *  @}
 */ /* End of group RLMRSTFLG1 */

/** 
 *  @defgroup  RLMSHTDNOP  RLMSHTDNOP Register
 *  @brief     Shut Down Control Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:1  | -          |
 *  | 0     | PTKEEP     |
 *  @{
 */
/* PTKEEP */
#define RLMSHTDNOP_PTKEEP_RW_RELEASE    ((uint32_t)0x00000000)    /*!< PTKEEP :[R/W] :Release  */
#define RLMSHTDNOP_PTKEEP_RW_KEEP       ((uint32_t)0x00000001)    /*!< PTKEEP :[R/W] :Keep     */
/** 
 *  @}
 */ /* End of group RLMSHTDNOP */

/** 
 *  @defgroup  FCSR0  FCSR0 Register
 *  @brief     Flash Status Register 0.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:11 | -          |
 *  | 10    | RDYBSY2    |
 *  | 9     | RDYBSY1    |
 *  | 8     | RDYBSY0    |
 *  | 7:1   | -          |
 *  | 0     | RDYBSY     |
 *  @{
 */
/* RDYBSY */
#define FCSR0_RDYBSY_MASK               ((uint32_t)0x00000001)    /*!< RDYBSY :Mask            */
#define FCSR0_RDYBSY_R_RUNNING          ((uint32_t)0x00000000)    /*!< RDYBSY :[R] :Running    */
#define FCSR0_RDYBSY_R_DONE             ((uint32_t)0x00000001)    /*!< RDYBSY :[R] :Done       */
/** 
 *  @}
 */ /* End of group FCSR0 */


/** 
 *  @}
 */ /* End of group ISD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_define ISD Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group ISD_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_typedef ISD Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
} instance_t;


/**
 *  @}
 */ /* End of group ISD_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_variables ISD Private Variables
 *  @{
 */
static instance_t   instance;               /*!< Instance.                    */
static isd_t        isd_obj;                /*!< isd object */

/**
 *  @}
 */ /* End of group ISD_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_fuctions ISD Private Fuctions
 *  @{
 */
static void wdt_disable(void);
static void pll_disable(void);
static void internal_hosc_enable(void);
#if defined START_STOP2
static TXZ_Result low_power_start_stop2(void);
#else
static TXZ_Result low_power_start_stop1(void);
#endif
static TXZ_Result application_initialize_normal(void);
static TXZ_Result application_initialize_normal2(void);
/*--------------------------------------------------*/
/** 
  * @brief  WDT Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void wdt_disable(void)
{
    TSB_SIWD0->EN     = SIWDxEN_WDTE_RW_DISABLE;
    TSB_SIWD0->CR     = SIWDxCR_WDCR_W_DISABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  PLL Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void pll_disable(void)
{
    uint32_t work;

    /* [CGPLL0SEL]<PLL0SEL>: fosc           */
    work = (uint32_t)(TSB_CG->PLL0SEL & ~CGPLL0SEL_PLL0SEL_MASK);
    TSB_CG->PLL0SEL = (uint32_t)(work | CGPLL0SEL_PLL0SEL_RW_FOSC);
    /* [CGPLL0SEL]<PLL0ST> : wait for fosc  */
    while ((TSB_CG->PLL0SEL & CGPLL0SEL_PLL0ST_MASK) != CGPLL0SEL_PLL0ST_R_FOSC) 
    {
        /* no processing */
    }
    /* [CGPLL0SEL]<PLL0ON> : PLL Disable    */
    work = (uint32_t)(TSB_CG->PLL0SEL & ~CGPLL0SEL_PLL0ON_MASK);
    TSB_CG->PLL0SEL = (uint32_t)(work | CGPLL0SEL_PLL0ON_RW_DISABLE);
}

/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc_enable(void)
{
    uint32_t work;

    /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);
    /* [CGOSCCR]<OSCSEL>   : IHOSC          */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_OSCSEL_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_OSCSEL_RW_IHOSC);
    /* [CGOSCCR]<OSCF>     : Read(wait for IHOSC) */
    while ((TSB_CG->OSCCR & CGOSCCR_OSCF_MASK) != CGOSCCR_OSCF_R_IHOSC) 
    {
        /* no processing */
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Low Power Mode Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for STOP2 Mode.
  */
/*--------------------------------------------------*/
#if defined START_STOP2
static TXZ_Result low_power_start_stop2(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t work;

    /*----------------------*/
    /* Application          */
    /*----------------------*/
    /*--- LED ---*/
    /* Set LED Port before IO Keep. */
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            /* LED STOP2 :on  */
            if (i == (uint32_t)BSP_LED_3)
            {
                p_led->init.state           = LED_STATE_ON;
            }
            /* default   :off */
            else
            {
                p_led->init.state           = LED_STATE_OFF;
            }
            led_initialize(p_led);
        }
    }
    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    wdt_disable();
    /*----------------------*/
    /* Flash Wait           */
    /*----------------------*/
    /* [FCSR0]<RDYBSY>   : Read(wait for done) */
    while ((TSB_FC->SR0 & FCSR0_RDYBSY_MASK) != FCSR0_RDYBSY_R_DONE)
    {
        /* no processing */
    }
    /*----------------------*/
    /* IO Keep              */
    /*----------------------*/
    /* RLMSHTDNOP:<PTKEEP=1>   */
    TSB_RLM->SHTDNOP = RLMSHTDNOP_PTKEEP_RW_KEEP;
    /*----------------------*/
    /* Set STBY             */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :STOP2           */
    TSB_CG->STBYCR = CGSTBYCR_STBY_RW_STOP2;
    /*----------------------*/
    /* PLL Disable          */
    /*----------------------*/
    pll_disable();
    /*----------------------*/
    /* Internal HOSC Enable */
    /*----------------------*/
    internal_hosc_enable();
    /*----------------------*/
    /* External HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : EHOSC Disable  */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_EOSCEN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_EOSCEN_RW_UNUSE);
    /*----------------------*/
    /* OFD Internal HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2EN> : OFD IHOSC Disable  */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC2EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC2EN_RW_DISABLE);
    /*----------------------*/
    /* External HOSC Disable Check*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : Read  */
    while ((TSB_CG->OSCCR & CGOSCCR_EOSCEN_MASK) != CGOSCCR_EOSCEN_RW_UNUSE) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* OFD Internal HOSC Disable Wait*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2F>  : Read(wait for OFD IHOSC Disable)  */
    while ((TSB_CG->OSCCR & CGOSCCR_IHOSC2F_MASK) != CGOSCCR_IHOSC2F_R_STOP) 
    {
        /* no processing */
    }
    return (result);
}
#else
/*--------------------------------------------------*/
/** 
  * @brief  Low Power Mode Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   STOP1 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result low_power_start_stop1(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t work;

    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    wdt_disable();
    /*----------------------*/
    /* Flash Wait           */
    /*----------------------*/
    /* [FCSR0]<RDYBSY>   : Read(wait for done) */
    while ((TSB_FC->SR0 & FCSR0_RDYBSY_MASK) != FCSR0_RDYBSY_R_DONE)
    {
        /* no processing */
    }
    /*----------------------*/
    /* HOSC Warm-up Wait    */
    /*----------------------*/
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    while ((TSB_CG->WUPHCR & CGWUPHCR_WUEF_MASK) != CGWUPHCR_WUEF_R_DONE)
    {
        /* no processing */
    }
    /*----------------------*/
    /* Internal HOSC Select */
    /*----------------------*/
    /* [CGWUPHCR]<WUCLK> : IHOSC */
    work = (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUCLK_MASK);
    TSB_CG->WUPHCR = (uint32_t)(work | CGWUPHCR_WUCLK_RW_IHOSC);
    /*----------------------*/
    /* Internal HOSC Warm-up Time Set*/
    /*----------------------*/
    /*-----------------------------------------------*/
    /* "1"counter  (s) = 1 / IHOSC                   */
    /* "1"counter (us) = (10^6) / IHOSC              */
    /* "x"counter (us) = time                        */
    /*-----------------------------------------------*/
    /* x : time = 1 : (10^6) / IHOSC                 */
    /*-----------------------------------------------*/
    {
        uint64_t x = (uint64_t)((uint64_t)(CFG_IHOSC_WARM_UP_TIME) * (uint64_t)(CFG_IHOSC_CLOCK));
        x = (uint64_t)(x / (uint64_t)(1000000));
        if (x > (uint64_t)(0xFFFF))
        {
            /* invalid value */
        }
        work = (uint32_t)x;
    }
    work  &= (uint32_t)(0xFFFFFFF0);
    work <<= 16;
    work  |= (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUPT_HIGH_MASK);
    TSB_CG->WUPHCR = work;
    /*----------------------*/
    /* Set STBY             */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :STOP1           */
    TSB_CG->STBYCR = CGSTBYCR_STBY_RW_STOP1;
    /*----------------------*/
    /* PLL Disable          */
    /*----------------------*/
    pll_disable();
    /*----------------------*/
    /* Internal HOSC Enable */
    /*----------------------*/
    internal_hosc_enable();
    /*----------------------*/
    /* External HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : EHOSC Disable  */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_EOSCEN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_EOSCEN_RW_UNUSE);
    /*----------------------*/
    /* OFD Internal HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2EN> : OFD IHOSC Disable  */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC2EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC2EN_RW_DISABLE);
    /*----------------------*/
    /* External HOSC Disable Check*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : Read  */
    while ((TSB_CG->OSCCR & CGOSCCR_EOSCEN_MASK) != CGOSCCR_EOSCEN_RW_UNUSE) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* OFD Internal HOSC Disable Wait*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2F>  : Read(wait for OFD IHOSC Disable)  */
    while ((TSB_CG->OSCCR & CGOSCCR_IHOSC2F_MASK) != CGOSCCR_IHOSC2F_R_STOP) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            /* LED STOP1 :on  */
            if (i == (uint32_t)BSP_LED_2)
            {
                p_led->init.state           = LED_STATE_ON;
            }
            /* default   :off */
            else
            {
                p_led->init.state           = LED_STATE_OFF;
            }
            led_initialize(p_led);
        }
    }
    return (result);
}
#endif
/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize_normal(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;
        
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];
            
            p_led->init.id          = (uint32_t)p_led;
            p_led->init.p_gpio      = bsp_get_gpio_instance_address();
            p_led->init.port.group  = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num    = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            /* LED Normal :on */
            if (i == (uint32_t)BSP_LED_0)
            {
                p_led->init.state               = LED_STATE_ON;
            }
            /* default    :off      */
            else
            {
                p_led->init.state               = LED_STATE_OFF;
            }
            led_initialize(p_led);
        }
    }
    
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize2
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize_normal2(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;
        
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];
            
            p_led->init.id          = (uint32_t)p_led;
            p_led->init.p_gpio      = bsp_get_gpio_instance_address();
            p_led->init.port.group  = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num    = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            /* LED Normal :on */
            if (i == (uint32_t)BSP_LED_1)
            {
                p_led->init.state               = LED_STATE_ON;
            }
            /* default    :off      */
            else
            {
                p_led->init.state               = LED_STATE_OFF;
            }
            led_initialize(p_led);
        }
    }
    
    return (result);
}


/**
 *  @}
 */ /* End of group ISD_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_functions ISD Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  The main function of ISD demo
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
     /*----------------------*/
    /* Reset Flag Check     */
    /*----------------------*/
    if ((TSB_RLM->RSTFLG0 & RLMRSTFLG0_STOP2RSTF_MASK) == RLMRSTFLG0_STOP2RSTF_R_RESET)
    {
        /* It's  executed, when STOP2 mode is released. */
        /*----------------------*/
        /* BSP initialize       */
        /*----------------------*/
        bsp_initialize();
        /*----------------------*/
        /* Release "IO Keep"    */
        /*----------------------*/
        TSB_RLM->SHTDNOP = RLMSHTDNOP_PTKEEP_RW_RELEASE;
        /* Clear Flag */
        TSB_RLM->RSTFLG0 = RLMRSTFLG0_W_CLEAR;
        TSB_RLM->RSTFLG1 = RLMRSTFLG1_W_CLEAR;
        /*--- Jump to the Interrupt routine, STOP2 mode is released. ---*/
        /*  Interrupt Enable */
        TSB_IA->IMC53 = (uint8_t)(0x07);        /* INTEN=1 */
        /* NVIC Enable */
        NVIC_EnableIRQ(INTISDB_IRQn);
        
        if(application_initialize_normal2() != TXZ_SUCCESS)
        {
            /* no process */
        }
    }
    else
    {
        /*----------------------*/
        /* BSP initialize       */
        /*----------------------*/
        bsp_initialize();
      #if defined START_STOP2
        if(low_power_start_stop2() != TXZ_SUCCESS)
        {
            /* no process */
        }
      #else
        if(low_power_start_stop1() != TXZ_SUCCESS)
        {
            /* no process */
        }
      #endif
        
        isd_obj.p_instance = TSB_ISDB;
        
        isd_obj.init.div = ISD_DIV_X1;
        isd_obj.init.sc = ISD_SC_FS;
        isd_obj.init.ms = ISD_MS_MASTER;
        isd_obj.init.op = ISD_OP_HIGH;
        isd_obj.init.t2 = 0x00;
        isd_obj.init.t1 = 0x01;
        isd_obj.init.t0 = 0xff;
        isd_obj.init.i3m = ISD_Inx_DISABLE;
        isd_obj.init.i2m = ISD_Inx_DISABLE;
        isd_obj.init.i1m = ISD_Inx_DISABLE;
        isd_obj.init.i0m = ISD_Inx_HIGH_LOW;
        isd_obj.init.inten = ISD_INTEN_ENABLE;
        
        ISD_Initialize(&isd_obj);
        
        ISD_Start(&isd_obj);
        
        /*  Interrupt Enable */
        TSB_IA->IMC53 = (uint8_t)(0x07);        /* INTEN=1, INTMODE=both edge */
        /* NVIC Enable */
        NVIC_EnableIRQ(INTISDB_IRQn);
        /*>>>Wait For Interrupt */
        __WFI();
        /*<<<Wait For Interrupt */
        /*----------------------*/
        /* BSP finalize         */
        /*----------------------*/
        bsp_finalize();
        /*----------------------*/
        /* System Initialize    */
        /*----------------------*/
        SystemInit();
        /*----------------------*/
        /* BSP initialize       */
        /*----------------------*/
        bsp_initialize();
        if(application_initialize_normal() != TXZ_SUCCESS)
        {
            /* no process */
        }
    }
    
    while(1)
    {
        __NOP();
    };
}

/*--------------------------------------------------*/
/** 
  * @brief  ISD interrupt request.
  * @note   -
  */
/*--------------------------------------------------*/
void irq_isd(void)
{
    uint32_t status;
    status = ISD_Status_Read(&isd_obj);
    if((status & 0x01) == 0x01)
    {
        __NOP();
    }
}



#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group ISD_Exported_functions */

/**
 *  @}
 */ /* End of group ISD */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
