/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TRMOSC Sample Application.
 * @version V1.0.0.0
 * $Date:: 2017-11-15 11:00:46 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_sample_def.h"
#include "IOSCadjustment.h"
#include "trm_t32a.h"
#include "bsp.h"
#include "bsp_uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TRMOSC TRMOSC Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_macro TRMOSC Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */

/** 
 *  @defgroup ELOSCConfiguration ELOSC Configuration
 *  @brief    ELOSC Configuration.
 *  @{
 */
#define ELOSC_CFG_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us). */
#define ELOSC_CFG_CLOCK         ((uint32_t)(32768))     /*!< Clock(hz).        */
/** 
 *  @}
 */ /* End of group ELOSCConfiguration */

/** 
 *  @defgroup  CGWUPLCR  CGWUPLCR Register
 *  @brief     LOSC warming up Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:27 | -          |
 *  | 26:12 | WUPTL      |
 *  | 11:8  | WUPTL      |
 *  | 7:2   | -          |
 *  | 1     | WULEF      |
 *  | 0     | WULON      |
 *  @{
 */
/* WUPTL  */
#define CGWUPLCR_WUPTL_HIGH_MASK        ((uint32_t)0x07FFF000)    /*!< WUPTL   :High Bit Mask.  */
/* WULEF  */
#define CGWUPLCR_WULEF_MASK             ((uint32_t)0x00000002)    /*!< WULEF   :Mask.           */
#define CGWUPLCR_WULEF_R_DONE           ((uint32_t)0x00000000)    /*!< WULEF   :[R]  :Done      */
#define CGWUPLCR_WULEF_R_RUNNING        ((uint32_t)0x00000002)    /*!< WULEF   :[R]  :Running   */
/* WULON  */
#define CGWUPLCR_WULON_MASK             ((uint32_t)0x00000001)    /*!< WULON   :Mask.           */
#define CGWUPLCR_WULON_W_ENABLE         ((uint32_t)0x00000001)    /*!< WULON   :[W]  :Enable     */
/** 
 *  @}
 */ /* End of group CGWUPLCR */

/** 
 *  @defgroup  RLMLOSCCR  RLMLOSCCR Register
 *  @brief     LOSC Control Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:1  | -          |
 *  | 0     | XTEN       |
 *  @{
 */
/* XTEN */
#define RLMLOSCCR_XTEN_RW_DISABLE       ((uint32_t)0x00000000)    /*!< XTEN :[R/W]  :Disable  */
#define RLMLOSCCR_XTEN_RW_ENABLE        ((uint32_t)0x00000001)    /*!< XTEN :[R/W]  :Enable   */
/** 
 *  @}
 */ /* End of group RLMLOSCCR */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
#define COUNTMAX               (10)            /*!< retry counter max value1 */
/*----------------------*/
/*  UART Setting        */
/*----------------------*/
/** 
 *  @defgroup UART_Setting UART Setting
 *  @{
 */
#define CFG_UART_BAUDRATE               ((uint32_t)115200)  /*!< Boudrate(bps) */
/** 
 *  @}
 */ /* End of group UART_Setting */

#define MAIN_NULL       ((void *)0)     /*!< NULL. */
#define USER_ID_REMOTE  ((uint32_t)0)   /*!< User ID :Remote Control (Any value) */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */


/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_typedef TRMOSC Private Typedef
 *  @{
 */

/**
 *  @}
 */ /* End of group TRMOSC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_variables TRMOSC Private Variables
 *  @{
 */
static uint32_t RetryCount=0;
static int32_t  AdjustResult;

/**
 *  @}
 */ /* End of group TRMOSC_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static TXZ_Result application_initialize(void);
/** 
 *  @defgroup TRMOSC_Private_fuctions TRMOSC Private Fuctions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = (uint32_t)CFG_UART_BAUDRATE;
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/**
 *  @}
 */ /* End of group TRMOSC_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Exported_functions TRMOSC Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  The main function of TRMOSC demo
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t tmp, i;

    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
#ifdef TRMOSC_LOSC
    /* TRIGGER SELECTOR initialize                  */
    /* T32A ch8 A input trigger : LOSC              */
    /* T32A ch8 B input trigger : T32A08OUTA        */
    TSB_TSEL0->CR8  = 0x00030000;
    TSB_TSEL0->CR11 = 0x00001300;
#endif
    /* Application initialization */
    if(application_initialize() == TXZ_ERROR){
        /* Do Nothing */
    }
    InitT32A_Sout();
    InitT32A_RefIn();
    InitTRM();                   /* for debug */

    if(printf("%s\r\n", "---------------") < 0){ /* Do Nothing */ };
    if(printf("%s\r\n", "| TRMOSC_demo |") < 0){ /* Do Nothing */ };
    if(printf("%s\r\n", "---------------") < 0){ /* Do Nothing */ };

    if(printf("%s\r\n", "please wait...") < 0){ /* Do Nothing */ };
    for(i=0; i<0x00400000; i++){};
    if(printf("%s\r\n", "---------------") < 0){ /* Do Nothing */ };
    if(printf("%s\r\n", "|    start    |") < 0){ /* Do Nothing */ };
    if(printf("%s\r\n", "---------------") < 0){ /* Do Nothing */ };

    for(RetryCount=0; RetryCount <= COUNTMAX; RetryCount++) 
    {
        AdjustResult = IOSCAdjustment();
        if (AdjustResult == TRMOSC_DONE) 
        {
            if(printf("%s\r\n", "TRMOSC_DONE") < 0){ /* Do Nothing */ };
            if(GetTrimmingControlValue() == TRMOSC_CONTROL_ENABLE) 
            {
                tmp = GetCoarseTrimValueSet();
                if(printf("%s%X\r\n", "<TRIMSETC>:", tmp) < 0){ /* Do Nothing */ };
                tmp = GetFineTrimValueSet();
                if(printf("%s%X\r\n", "<TRIMSETF>:", tmp) < 0){ /* Do Nothing */ };
            }
            else 
            {
                if(printf("%s\r\n", "Inital trimming Value") < 0){ /* Do Nothing */ };
            }
            break;
        }
        else if (AdjustResult == TRMOSC_RUN) 
        {
            if(printf("%s\r\n", "TRMOSC_RUN") < 0){ /* Do Nothing */ };
            if(GetTrimmingControlValue() == TRMOSC_CONTROL_ENABLE) 
            {
                tmp = GetCoarseTrimValueSet();
                if(printf("%s%X\r\n", "<TRIMSETC>:", tmp) < 0){ /* Do Nothing */ };
                tmp = GetFineTrimValueSet();
                if(printf("%s%X\r\n", "<TRIMSETF>:", tmp) < 0){ /* Do Nothing */ };
            }
            else 
            {
                if(printf("%s\r\n", "Inital trimming Value") < 0){ /* Do Nothing */ };
            }
        }
        else 
        {
            if(printf("%s\r\n", "TRMOSC_ERROR") < 0){ /* Do Nothing */ };
        }
    }

    while(1){};
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}


#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group TRMOSC_Exported_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
