/**
 *******************************************************************************
 * @file    trm.c
 * @brief   This file provides API functions for TRM.
 * @version V1.0.0.0
 * $Date:: 2017-11-15 13:52:12 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "trm.h"

#if defined(__TRM_H)
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup TRMOSC
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRMOSC_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRMOSC_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRMOSC_Private_define
 *  @{
 */

#define TRMOSCPRO_ENABLE           (0x000000C1UL)      /* Writing register control enable. */
#define TRMOSCPRO_DISABLE          (0x00000000UL)      /* Writing register control disable. */
#define TRMOSCEN_MASK              (0x00000001UL)      /* Trimming Control enable. */
#define TRIMCOARSE_MASK            (0x00003F00UL)      /* Trimming Coarse value mask. */
#define TRIMFINE_MASK              (0x0000000FUL)      /* Trimming Fine value mask. */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRMOSC_Private_fuctions
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group TRMOSC_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRMOSC_Exported_functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Set TRMOSC register
  * @param  TRMOSC Control Structure
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void SetTrmOsc(TRM_ControlTypeDef *ControlStruct)
{
    uint32_t tmp = 0;

    TSB_TRM->OSCPRO = TRMOSCPRO_ENABLE;
    TSB_TRM->OSCEN  = ControlStruct->TrimmingControl;
    tmp = (ControlStruct->CoarseTrimmingValue << 8) & TRIMCOARSE_MASK;
    tmp |= ControlStruct->FineTrimmingValue & TRIMFINE_MASK;
    TSB_TRM->OSCSET = tmp;
    TSB_TRM->OSCPRO = TRMOSCPRO_DISABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Trimming Control Value
  * @param  -
  * @retval Trimming Control value
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetTrimmingControlValue(void)
{
    return (TSB_TRM->OSCEN & TRMOSCEN_MASK);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Initial Coarse Trimming Value
  * @param  -
  * @retval Initial Coarse Trimming value
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetInitCoarseTrimValue(void)
{
    return ((TSB_TRM->OSCINIT & TRIMCOARSE_MASK) >> 8);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get initial Fine Trimming Value
  * @param  -
  * @retval Initial Fine Trimming value
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetInitFineTrimValue(void)
{
    return (TSB_TRM->OSCINIT & TRIMFINE_MASK);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Coarse Trimming value setting
  * @param  -
  * @retval Coarse Trimming value setting
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetCoarseTrimValueSet(void)
{
    return ((TSB_TRM->OSCSET & TRIMCOARSE_MASK) >> 8);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Fine Trimming value setting
  * @param  -
  * @retval Fine Trimming value setting
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetFineTrimValueSet(void)
{
    return (TSB_TRM->OSCSET & TRIMFINE_MASK);
}

/**
 *  @}
 */ /* End of group TRMOSC_Exported_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__TRM_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */


