/**
 *******************************************************************************
 * @file    trm_t32a.c
 * @brief   This file provides API functions for Periph driver interface.
 * @version V1.0.0.0
 * $Date:: 2017-11-15 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "trm_t32a.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TRMOSC TRMOSC Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_macro TRMOSC Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_typedef TRMOSC Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group TRMOSC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_variables TRMOSC Private Variables
 *  @{
 */

static t32a_t   timer_sout;                /*!< Instance       :Timer Driver  */
static t32a_t   timer_refin;               /*!< Instance       :Timer Driver  */
/**
 *  @}
 */ /* End of group TRMOSC_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_fuctions TRMOSC Private Fuctions
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group TRMOSC_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Exported_functions TRMOSC Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Signal Output Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitT32A_Sout(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Signal Output Timer         */
    /*------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_sout.p_instance = BSP_T32A_SOUT;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_sout.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_sout.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_sout) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer A ---*/
    {
        t32a_initial_setting_t *p_init = &timer_sout.init[T32A_TIMERA];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_1;
        p_init->crx.clk            = T32A_CLKx_PRSCLx;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_TREGx;
        p_init->crx.stop           = T32A_STOP_NON;
        p_init->crx.start          = T32A_START_NON;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_INVERSION;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_INVERSION;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = 1;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Counter Capture Control A0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control A1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
        if (t32a_timer_init(&timer_sout, T32A_TIMERA) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
        BSP_T32A_SOUT_RUNA_SFTSTAA = 1;
    }
    if (result != TXZ_SUCCESS)
    {
        /* no processing */
    }
}
#ifdef TRMOSC_LOSC
/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Reference Signal Input Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitT32A_RefIn(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*-------------------------------*/
    /*  Reference Signal Input Timer */
    /*-------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_refin.p_instance = BSP_T32A_REFIN;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_refin.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_refin.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_refin) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer A ---*/
    {
        t32a_initial_setting_t *p_init = &timer_refin.init[T32A_TIMERA];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_1;
        p_init->crx.clk            = T32A_CLKx_INTRG;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_TREGx;
        p_init->crx.stop           = T32A_STOP_NON;
        p_init->crx.start          = T32A_START_NON;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_CLR;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_INVERSION;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_DISABLE;;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = 64;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Capture Control */
        p_init->capcrx.capmx1      = T32A_CAPMx1_DISABLE;
        p_init->capcrx.capmx0      = T32A_CAPMx0_DISABLE;
        /* Counter Capture Control A0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control A1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;

        if (t32a_timer_init(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
        BSP_T32A_REFIN_RUNA_SFTSTAA = 1;
    }
    if (result != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*---------------------------*/
    /*--- Initialize: Timer B ---*/
    /*---------------------------*/
    {
        t32a_initial_setting_t *p_init = &timer_refin.init[T32A_TIMERB];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_DONT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_2;
        p_init->crx.clk            = T32A_CLKx_PRSCLx;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_NON;
        p_init->crx.stop           = T32A_STOP_NON;
        p_init->crx.start          = T32A_START_NON;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_DISABLE;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_DISABLE;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = 0;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Capture Control */
        p_init->capcrx.capmx1      = T32A_CAPMx1_TIM_RISING_EDGE;
        p_init->capcrx.capmx0      = T32A_CAPMx0_DISABLE;
        /* Counter Capture Control B0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control B1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
    }
    if (result != TXZ_SUCCESS)
    {
        /* no processing */
    }
}
#else
/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Reference Signal Input Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitT32A_RefIn(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*-------------------------------*/
    /*  Reference Signal Input Timer */
    /*-------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_refin.p_instance = BSP_T32A_REFIN;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_refin.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_refin.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_refin) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer A ---*/
    {
        t32a_initial_setting_t *p_init = &timer_refin.init[T32A_TIMERA];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_2;
        p_init->crx.clk            = T32A_CLKx_PRSCLx;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_NON;
        p_init->crx.stop           = T32A_STOP_EXTTRG_RISING_EDGE;
        p_init->crx.start          = T32A_START_EXTTRG_TRAILING_EDGE;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_DISABLE;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_DISABLE;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = 0;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Capture Control */
        p_init->capcrx.capmx1      = T32A_CAPMx1_INx0_RISING_EDGE;
        p_init->capcrx.capmx0      = T32A_CAPMx0_INx0_TRAILING_EDGE;
        /* Counter Capture Control A0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control A1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
    }
    if (result != TXZ_SUCCESS)
    {
        /* no processing */
    }
}
#endif
/*--------------------------------------------------*/
/** 
  * @brief  Start Capture
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void StartCapture(void)
{
#ifdef TRMOSC_LOSC
    BSP_T32A_REFIN_RUNB_SFTSTPB = 1;
    if (t32a_deinit(&timer_refin, T32A_TIMERB) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    TSB_IB->IMC085 = 0x41;          /* T32A ch8 TimerB CAP1 */
    if (t32a_timer_init(&timer_refin, T32A_TIMERB) != TXZ_SUCCESS)
    {
        /* no processing */
    }
#else
    BSP_T32A_REFIN_RUNA_SFTSTPA = 1;
    if (t32a_deinit(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    TSB_IB->IMC021 = 0x41;              /* T32A ch2 TimerA CAP0 */
    TSB_IB->IMC022 = 0x41;              /* T32A ch2 TimerA CAP1 */
    if (t32a_timer_init(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Capture Data
  * @param  -
  * @retval Capture value
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetCaptureData(void)
{
#ifdef TRMOSC_LOSC
    uint32_t retval = 0;
    uint32_t captop = 0;
    uint32_t capend = 0;

    for (;;)
    {
        while ((TSB_IB->IMC085 & 0x10) == 0)
        {
            /* waiting raising edge ... */          
        }
        TSB_IB->IMC085 = 0x41;          /* T32A ch8 TimerB CAP1 */
        captop = (timer_refin.p_instance->CAPB1 & T32A_CAPx1_MASK);

        while ((TSB_IB->IMC085 & 0x10) == 0)
        {
            /* waiting raising edge ... */          
        }
        TSB_IB->IMC085 = 0x41;          /* T32A ch8 TimerB CAP1 */
        capend = (timer_refin.p_instance->CAPB1 & T32A_CAPx1_MASK);

        if(captop < capend) {
            retval = capend - captop;
        }
        else {
            retval = capend + (0xffff - captop);
        }
        retval >>= 1;
        break;
    }
#else
    uint32_t retval = 0;

    for (;;)
    {
        while((TSB_IB->IMC021 & 0x10) == 0x00)
        {
            /* waiting falling edge ... */
        }
        TSB_IB->IMC021 = 0x41;          /* T32A ch2 TimerA CAP0 */
        while((TSB_IB->IMC021 & 0x10) == 0x00)
        {
            /* waiting raising edge ... */          
        }
        TSB_IB->IMC022 = 0x41;          /* T32A ch2 TimerA CAP1 */
        retval = (timer_refin.p_instance->CAPA1 & T32A_CAPx1_MASK);
        break;
    }   
#endif
    return(retval); 
}

/*--------------------------------------------------*/
/** 
  * @brief  Stop Capture
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void StopCapture(void)
{
#ifdef TRMOSC_LOSC
    BSP_T32A_REFIN_RUNB_SFTSTPB = 1;
    if (t32a_deinit(&timer_refin, T32A_TIMERB) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    TSB_IB->IMC085 = 0x41;          /* T32A ch8 TimerB CAP1 */
#else
    BSP_T32A_REFIN_RUNA_SFTSTPA = 1;
    if (t32a_deinit(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    TSB_IB->IMC021 = 0x41;          /* T32A ch2 TimerA CAP0 */
    TSB_IB->IMC022 = 0x41;          /* T32A ch2 TimerA CAP1 */
#endif
}

/**
 *  @}
 */ /* End of group TRMOSC_Exported_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
